#!/usr/bin/perl
#-----------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

package Modules::ID::Location;

use strict;
use ExSite::Config;
use ExSite::Base;
use ExSite::Form;
use ExSite::Misc;
use ExSite::Object;

use vars qw(@ISA);
@ISA = qw(ExSite::Object);

sub my_type { return "location"; }

sub name {
    my $this = shift;
    if ($this->defined()) {
	return $this->getdata("name");
    }
    return "n/a";
}

sub contacts {
    my $this = shift;
    if ($this->loaded()) {
	if (! $this->{child}) {
	    $this->{child} = $this->get_children();
	}
	return $this->{child}{contact};
    }
    return undef;
}

# show options
# image = show image
# brochure = show downloadable document 
# contact = show contact info

sub show {
    my ($this,%opt) = @_;
    my $out;
    if ($this->exists()) {
	my ($img,$imgdata,$imghtml,%imgopt);
	my $ml = new ExSite::ML;
	$out = $ml->h2($this->showdata("name"));

	# IMAGE

	if (! defined $opt{image} || $opt{image}) {
	    $imgdata = $this->getdata("image");
	    if ($imgdata) {
		$img = new ExSite::Image($imgdata);
		($imgopt{width},$imgopt{height}) = $img->dim;
		my $file = $img->name();
		if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/ID/locations/$file") {
		    $imgopt{src} = "$config{server}{HTMLpath}/_Modules/ID/locations/$file";
		}
		else {
		    my $id = $this->id;
		    $imgopt{src} = "$config{server}{CGIpath}/$config{prog}{getdata}/location/$id/image/$file";
		}
		if ($config{Location}{image_float_maxwidth} &&
		    $imgopt{width} <= $config{Location}{image_float_maxwidth}) {
		    $imgopt{align} = "right";
		}
		$imgopt{alt} = $this->name();
		$imghtml = $ml->img(undef,\%imgopt);
	    }
	}

	$out .= $ml->p(
		      $imghtml .
		      $this->showdata("description")
		      );

	# BROCHURE

	if (! defined $opt{brochure} || $opt{brochure}) {
	    $imgdata = $this->getdata("brochure");
	    if ($imgdata) {
		my ($file,undef) = split /$config{form}{sepchar}/,$imgdata;
		my $url;
		if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/ID/locations/$file") {
		    $url = "$config{server}{HTMLpath}/_Modules/ID/locations/$file";
		}
		else {
		    my $id = $this->id;
		    $url = "$config{server}{CGIpath}/$config{prog}{getdata}/location/$id/brochure/$file";
		}
		my $message = $config{Location}{brochure_prompt} ||
		    "Download brochure:";
		my $anchor = $config{Location}{brochure_anchor} || $file;
		$out .= $ml->p($msg{$message}." ".$ml->a($anchor,{href=>$url}));
	    }
	}

	if (! defined $opt{contact} || $opt{contact}) {
	    my $contacts = $this->contacts();
	    my $contactinfo;
	    if ($contacts) {
		my $ncont = $contacts->count();
		while (my $contact = $contacts->next()) {
		    if ($ncont > 1) {
			$contactinfo .= $ml->h4(ucfirst($contact->showdata("type")).":");
		    }
		    $contactinfo .= $contact->show();
		}
	    }
	    if ($contactinfo) {
		$out .= $ml->h3($msg{"Contact Information"});
		$out .= $contactinfo;
	    }
	}

	$out = $ml->div($out,{class=>"location"});
    }
    return $out;
}

1;
