package Modules::Backup;
#----------------------------------------------------------------------------
#   The Backup module creates software, data, and content backups of your
#   website.
#-----------------------------------------------------------------------
#
#   Copyright 2001-2009 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Page;
use ExSite::Input;
use ExSite::Misc;

# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# The Backup tool has no public output

sub write {
    my ($this,$options) = @_;
    return;
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleInfo/) {
	return "The Backup plug-in lets you create backups of your pages,software, and data.";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/Category/) {
	return "System";
    }
    elsif (/isRestricted/) {
	# restrict this to make it harder to fill the disk with junk
	return 0;
    }
    elsif (/Access/) {
	return 3;
    }
}

sub ctrl_panel {
    my $this = shift;
    my $db = $share{DB};
    return $this->error("Permission denied.") if (! $db->is_admin());
    my $ml = new ExSite::ML;
    my $out = $ml->h1("Website Backup Utility");

    my $in = $this->{input};
    if ($in->{backup} eq "db") {
	$out .= $this->backup_db;
    }
    elsif ($in->{backup} eq "html") {
	$out .= $this->backup_html;
    }
    elsif ($in->{backup} eq "cgi") {
	$out .= $this->backup_cgi;
    }
    else {
	$out .= $ml->h2("Choose a backup:");

	my $t = new ExSite::Time;
	my $date = $t->write('iso_date');
	$out .= $ml->ul(
			$ml->li(
				$ml->a("Database dump (gzip)",{href=>$this->link(backup=>"db",format=>"tgz",'__path'=>{'_'=>"website-database-backup-$date.sql.gz"})})
				)
			);
	
	$out .= $ml->ul(
			$ml->li(
				$ml->a("Documents archive (gzip)",{href=>$this->link(backup=>"html",format=>"tgz",'__path'=>{'_'=>"website-documents-backup-$date.tgz"})})
				).
			$ml->li(
				$ml->a("Documents archive (zip)",{href=>$this->link(backup=>"html",format=>"zip",'__path'=>{'_'=>"website-documents-backup-$date.zip"})})
				)
			 );

	$out .= $ml->ul(
			$ml->li(
				$ml->a("Software archive (gzip)",{href=>$this->link(backup=>"cgi",format=>"tgz",'__path'=>{'_'=>"website-software-backup-$date.tgz"})})
				).
			$ml->li(
				$ml->a("Software archive (zip)",{href=>$this->link(backup=>"cgi",format=>"zip",'__path'=>{'_'=>"website-software-backup-$date.zip"})})
				)
			);
	$out .= $ml->p("Read the help file (? icon at upper right) before making any backups.");
    }

    return $out;
}

sub backup_html {
    my $this = shift;
    my $root = $config{server}{HTMLroot};
    return $this->backup_dir($root);
}

sub backup_cgi {
    my $this = shift;
    my $root = $config{server}{CGIroot};
    return $this->backup_dir($root);
}

sub backup_db {
    my $this = shift;
    my $root = $config{server}{CGIroot};
    my $ml = &get_obj("ML");
    $ml->cancel;
    my $mysqldump = "mysqldump -u $config{server}{db}{user} --password='$config{server}{db}{pass}' --single-transaction";
    if ($config{server}{db}{server} ne "localhost") {
	$mysqldump .= " -h $config{server}{db}{server}";
    }
    $mysqldump .= " $config{server}{db}{name}";
    my $zip;
#    print "Content-type:text/plain\n\n";
    if ($this->{input}{format} eq "zip") {
	print "Content-type: application/zip\n\n";
	$zip = "zip -0 -q -r";
    }
    elsif ($this->{input}{format} eq "tgz") {
	print "Content-type: application/x-compressed\n\n";
	$zip = "gzip";
    }
#    print "$mysqldump | $zip";
    print `$mysqldump | $zip`;
    return;
}

sub backup_dir {
    my ($this,$root) = @_;
    my @path = split(/\//,$root);
    my $dir = pop @path;
    my $base = join "/",@path;
    chdir $base or return $this->error("Failed to change to $base: $!");
    if ($this->{input}{format} eq "zip") {
	my $ml = &get_obj("ML");
	$ml->cancel;
	print "Content-type: application/zip\n\n";
	print `zip -0 -q -r - $dir`;
    }
    elsif ($this->{input}{format} eq "tgz") {
	my $ml = &get_obj("ML");
	$ml->cancel;
	print "Content-type: application/x-compressed\n\n";
	print `tar chzf - $dir`;
    }
    else {
	return $this->error("no backup format specified");
    }
}

1;

