#!/usr/bin/perl

=pod

=head1 Modules::Blog

This is a wrapper for the Zine plug-in, to make it easier to work with Blogs.

=cut

package Modules::Blog;

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Time;
use ExSite::HTML;
use Modules::Zine;
use Modules::Zine::Form;

use vars qw(@ISA $ml);
@ISA = qw(Modules::Zine);

sub write {
    my ($this,$options) = @_;
    my %opt = &DecodeString($options);
    my $in = $this->{input};
    my $cmd = $in->{zine} || $opt{zine};
    my $aid = $in->{aid} || $opt{aid};
    $this->load_article($aid);
    if ($cmd || $this->{article}) {
	# Zine can take it from here
	return $this->SUPER::write($options);
    }
    else {
	# see if there is a good default blog to use
	my $sid = $this->get_section_id();
	if ($sid) {
	    my @blog = $this->fetch_match({type=>"blog",section_id=>$sid,status=>"active"});
	    my $nblog = scalar @blog;
	    if ($nblog == 1) {
		$this->load_article($blog[0]);
		return $this->show();
	    }
	    elsif ($nblog > 1) {
		return $this->zinedex("blog",$sid);
	    }
	}
    }
    return $this->error("No blogs available.");
}

sub ioctl {
    my ($this) = shift;
    $_ = shift;

    if (/ControlPanel/) {
	return \&ctrl_panel;
    }
    if (/Summary/) {
	return \&summary;
    }
    elsif (/ModuleName/) {
	return "Blogs";
    }
    elsif (/isRestricted/) {
	return $this->{config}{isRestricted};
    }
    elsif (/isService/) {
	return $this->{config}{isService};
    }
    elsif (/Publish/) {
	return \&Modules::Zine::Base::publish;
    }
#   elsif (/ISA/) {
#	return "Zine";
#   }
    elsif (/DynContentParam/) {
	my $sid = shift;
	my @blog = $this->fetch_match({type=>"blog",section_id=>$sid,status=>"active"});
	$ml = &get_obj("ML");
	my $opt = 
	    $ml->option("&lt;default&gt;",{value=>""}).
	    $ml->option("show all blogs",{value=>""});
	foreach my $b (@blog) {
	    $opt .= $ml->option($b->{title},{value=>"aid=$b->{article_id}&amp;zine=show"});
	}
	return "Blogs: " . $ml->select($opt,{name=>"param"});
    }
    elsif (/Dependencies/) {
	return "Zine";
    }
}

sub topview_admin {
    my ($this,$opt) = @_;
    if (! $share{DB}->is_manager) {
	return $this->error("Permission denied.");
    }
    return $this->zinedex("blog",$this->{input}{section_id});
}

sub zine_selector_form {
    my ($this,$sid) = @_;
    return undef;
}

sub new_zine_button {
    my $this = shift;
    my $url = $this->link(zine=>"newblog");
    return &ExSite::HTML::Button(url=>$url,label=>"Create a new Blog");
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    $ml = &get_obj("ML");
    my $sid = $this->get_section_id();
    my $in = $this->{input};

    if ($in->{zine} eq "newblog") {
	$out .= $this->new_blog();
    }
    else {
	$out .= $this->SUPER::ctrl_panel();
    }
    return $out;
}

sub topview_admin {
    my ($this,$opt) = @_;
    my $db = $share{DB};
    return $this->zinedex(
			  type=>"blog",
			  section_id=> $db->is_admin ? 0 : $share{DB}->my_user_record()->{section_id},
			  #no_section_name=>1,
			  );
}

sub new_blog {
    my $this = shift;
    my $out = $ml->h1("Start a new blog");
    $out .= $ml->p("Give your blog a title that describes the types of 
articles that you will be posting.".
		   $ml->br.
		   "For example, ".
		   $ml->i("News and Announcements").", or ".
		   $ml->i("Tips of the Trade").".");
    my $zf = new Modules::Zine::Form();
    my $f = new ExSite::FormBuilder(action=>$this->link(zine=>"submit"));
    $f->input(
	      name=>"title",
	      prompt=>"Name/Title of Blog:",
	      type=>"text",
	      required=>1,
	      size=>60,
	      );
    $f->input(
	      name=>"type",
	      type=>"hidden",
	      value=>"blog",
	      );
    $f->input(
	      name=>"section_id$config{form}{sepchar}key",
	      type=>"hidden",
	      value=>$this->get_section_id(),
	      );
    $f->input(
	      name=>"ctrlinfo",
	      type=>"hidden",
	      value=>$zf->ctrlinfo({})
	      );
    $out .= $f->make;
    return $out;
}

sub summary {
    my $this = shift;
    return $this->zinedex(
			  type=>"blog",
			  nolink=>1,
			  section_id=>$share{DB}->my_user_record()->{section_id},
			  no_section_name=>1,
			  );
}

1;
