package Modules::CMenu;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2006 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------
# Modules::CMenu
# Author: Morgan Burke
# (c) 2006 Exware Solutions
#-----------------------------------------------------------------------

=pod

=head1 Modules::CMenu - cascading DHTML menu-builder for ExSite

=cut

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Page;
use ExSite::Tree;
use ExSite::ML;

# inherit from BaseDCD class
use Modules::BaseDCD;
use base qw(Modules::BaseDCD);

sub write {
	my ($this, $opt) = @_;
	$this->config_setup($opt);
	# setup
	my $Page = $share{Page};
	return "" if (!$Page);
	my $sid = $Page->id("section");
	if ($Page->is_page()) {
		my $version = $Page->get("page","version");
		my $s = new ExSite::Section(id=>$sid);
		$this->{map} = $s->get_mapped_pages(0,{version=>$version});
	}else {
		# probably a template; make a fake map
		$this->{map} = new ExSite::Tree("page_id","parent_id",
			({page_id=>1,parent_id=>0,label=>"Home",access=>0,url=>"#",section_id=>$sid},
			{page_id=>2,parent_id=>0,label=>"About",access=>0,url=>"#",section_id=>$sid},
			{page_id=>3,parent_id=>0,label=>"Contact",access=>0,url=>"#",section_id=>$sid},
			{page_id=>4,parent_id=>0,label=>"Links",access=>0,url=>"#",section_id=>$sid},
	       ));
	}
	# generate a menu
	return "\n<!--CMenu-->\n".$this->include_files().$this->menu($opt)."\n<!--CMenu-->\n";
}
sub ioctl {
	my $this = shift;
	$_ = shift;
	if (/ModuleInfo/) {
		return "CMenu builds multi-level cascading menus.  It automatically 
assembles site menus and site maps by inspecting the relationships 
between your webpages.  It displays the top-level pages
in a horizontal \"menu bar\".  Secondary pages are displayed in drop-down
menus that appear when you mouse over the menu bar.  Tertiary pages appear
in drop-down menus that spring off the secondary menus.";
	}
	return undef;
}
# Gather module configuration
sub config_setup{
	my ($this,$opt) = @_;
	my %opt = ExSite::Misc::DecodeString($opt);
	$this->{config} = $config{CMenu};
	foreach my $k (keys(%opt)){
		next if !$opt{$k};
		$config{config}{$k} = $opt{$k};
	}
	if ($opt =~ /vertical/i){
		$this->vertical(1);
	}
	return;
}
# Vertical Menu
sub vertical{
	my ($this,$value) = @_;
	if (defined $value){
		$this->{config}{vertical} = $value ? 1  : 0;
	}
	return $this->{config}{vertical} || 0;
}
sub jquery_file{
	my ($this) = @_;
	if (!$this->{config}{jquery_file}){
		return "";
	}
	return ExSite::Config::get_obj("ML")->script(undef,{src=>$this->{config}{jquery_file}});
}
sub include_files{
	my $this = shift;
	my $ml = ExSite::Config::get_obj("ML");
	my $html_css = $ml->style("ul.CMenu, .CMenu ul{margin: 0px;padding: 0px;list-style-type: none;list-style-position: outside;position: relative;z-index: 3000;} .CMenu ul {position:absolute;display:none;} .CMenu a:link, .CMenu a:active, .CMenu a:visited{display:block;} .CMenu li{float:left;position:relative;} .CMenu.CMenu_vertical li {float: none;display:block;} .CMenu.CMenu_vertical ul li a {display:block;float: none;position: relative;}",{type=>"text/css"});
	return $html_css.$this->jquery_file().$ml->script(undef,{src=>"$config{server}{server}$config{server}{HTMLpath}/_Modules/CMenu/js/jquery.CMenu.js"}).$ml->script("\$(document).ready(function(){\$.CMenu();})");
}
# Show this page on the menu?
sub page_visible{
	my ($this,$p) = @_;
	if ($p->{visibility} =~ /^(hidden|sitemap-only)$/){
		return 0;
	}
	if ($this->{config}{no_default_version} && ($share{Page}->get("page","version") ne $p->{version})){
		return 0;
	}
	return 1;
}
sub custom_link{
	# $where: before|after
	my ($this,$where) = @_;
	if ($this->{config}{item}{$where}{url} && $this->{config}{item}{$where}{label}){
		return {label=>$this->{config}{item}{$where}{label},url=>$this->{config}{item}{$where}{url},id=>($this->{config}{item}{$where}{id} || undef)};
	}
	return undef;
}
sub html_menu{
	my $this = shift;
	my $mdata = shift;
	my $ul_attr = shift || undef;
	return "" if (!$mdata || ref($mdata) !~ /array/i);
	my $ml = ExSite::Config::get_obj("ML");
	my $out;
	foreach my $mdat(@$mdata){
		$out .= $ml->li($ml->a($mdat->{a_label},$mdat->{a_attr}).$this->html_menu($mdat->{submenu}),$mdat->{li_attr});
	}
	return $out ? $ml->ul($out,$ul_attr) : "";
}
sub generate_menu{
	my $this = shift;
	my $opt = shift;
	# get the menubar links
	my @map = $this->{map}->get_topnodes_data();
	my @menu_data;
	my %submenu_data = (current=>($share{Page} ? $share{Page}->id() : 0));
	foreach my $p (@map){
		next if !$this->page_visible($p);
		my $menuPage = new ExSite::Page(page=>$p);
		if ($submenu_data{current} == $p->{page_id}){
			$submenu_data{page}{$p->{page_id}} = 1;
		}
		push(@menu_data,{pid=>$p->{page_id},li_attr=>{id=>"page_$p->{page_id}",class=>"mLabel"},a_attr=>{href=>$menuPage->get_url(),class=>"menu"},a_label=>$menuPage->get_my("label"),submenu=>$this->generate_menu_r($menuPage->id(),$p->{page_id},\%submenu_data)});
	}
	if (!$this->{config}{disable_thispage_id}){
		foreach my $m(@menu_data){
			if ($submenu_data{page}{$m->{pid}}){
				$m->{li_attr}{id} = "thispage";
			}
		}
	}
	foreach my $where (("before","after")){
		my $custom = $this->custom_link($where);
		next if !$custom;
		my %cLink = (li_attr=>{id=>($custom->{id} ? $custom->{id} : "link_${where}"),class=>"mLabel"},a_attr=>{href=>$custom->{url},class=>"menu"},a_label=>$custom->{label});
		if ($where eq "before"){
			unshift @menu_data, \%cLink;
		}elsif ($where eq "after"){
			push  @menu_data, \%cLink;
		}
	}
	return \@menu_data;
}
sub generate_menu_r {
	my $this = shift;
	my $id = shift;
	my $pageid = shift;
	my $submenu_data = shift;
	return [] if !$id;
	if ($submenu_data->{current} == $id){
		$submenu_data->{page}{$pageid} = 1;
	}
	my @items = $this->{map}->get_child_data($id);
	return [] if scalar(@items) <= 0;
	my @menu;
	foreach my $p (@items) {
		next if !$this->page_visible($p);
		my $menuPage = new ExSite::Page(page=>$p);
		push(@menu,{li_attr=>{id=>"page_".$menuPage->id(),class=>"mItem"},a_attr=>{href=>$menuPage->get_url(),class=>"menu"},a_label=>$menuPage->get_my("label"),submenu=>$this->generate_menu_r($menuPage->id(),$pageid,$submenu_data)});
	}
	if (scalar(@menu) > 0){
		return \@menu;
	}
	return [];
}
sub menu {
	my $this = shift;
	my $opt = shift;
	my $menu_data = $this->generate_menu($opt);
	if (scalar(@$menu_data) > 0){
		return $this->html_menu($menu_data,{id=>"CMenu",class=>join(" ",("CMenu",($this->vertical() ? "CMenu_vertical" : ())))});
	}
	return "";
}
1;
