package Modules::Classified;
use strict;
use ExSite::Config;
use ExSite::Mail;
use ExSite::Misc;
use ExSite::Util;
use Modules::BaseDCD;
use base ("Modules::BaseDCD");

#---------------------------------------------------------------------
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return "Classifieds";
	}elsif (/ControlPanel/){
		return \&control_panel;
	}elsif (/ModuleInfo/){
        	return "Classifieds allow users to post classified ads.";
	}elsif (/isRestricted/){
		return exists $config{Classified}{isRestricted} ?
		    $config{Classified}{isRestricted} : 1;
	}
}
sub write{
	my ($this,$options) = @_;
	my (%opt,$action,$out);
	my %opt = &DecodeString($options);
	my $action = $this->{input}{action} || $opt{action};
	my $sectionid = $this->get_section_id();
	if ($this->{input}{cmd} eq "process_new"){	
		$out = $this->processNew();
	# Edit a classified
	}elsif ($this->{input}{cmd} eq "edit" && $this->{input}{classified_id}){
		$out = $this->edit($this->{input}{classified_id});
	# Delete a classified
	}elsif ($this->{input}{cmd} eq "del" && $this->{input}{classified_id}){
		$this->deleteClassified($this->{input}{classified_id});
		$this->redirect($this->reply());	
	# Send an email to the advertiser
	}elsif ($action eq "contact_post"){
		$out = $this->processContactForm($this->{input});
	# Post a new classified
	}elsif ($sectionid && $action eq "new"){
		$out = $this->newClassified($sectionid);
	# Contact form
	}elsif ($action eq "contact" && $this->{input}{classified_id}){
		$out = $this->contactForm($this->{input}{classified_id});
	# Show all classifieds in a category
	}elsif ($sectionid && $this->{input}{category}){
		$out = $this->showClassified($sectionid,$this->{input}{category});
	# Show categories
	}else{
		$out = $this->showCategoryList($sectionid);
	}
	return &get_obj("ML")->div($out, { class => "classifieds" });
}
#---------------------------------------------------------------------
# Control panel
sub control_panel{
	my $this = shift;
	my $out;
	# Form action
	my $action = $this->{input}{action};
	# Admin command
	my $cmd = $this->{input}{cmd};
	# Set the classified to active
	if ($cmd eq "set_active" && $this->{input}{classified_id}){
		$this->setActive($this->{input}{classified_id});
		$this->redirect($this->reply());
	# Create a new classified
	}elsif ($action eq "new" && $this->{input}{section_id}){
		return $this->newClassified($this->{input}{section_id});
	# Remove all expired classifieds
	}elsif ($cmd eq "removeallexpired"){
		$this->removeAllExpired();
	# Run public accessable function through write()
	}elsif ($this->{input}{category} || $this->{input}{action} eq "contact_post" || $cmd =~ /edit|del|process_new/i){
		return $this->write();
	# View current ads
	}elsif ($cmd eq "current" && $this->{input}{section_id}){		
		return $this->showCategoryList($this->{input}{section_id});
	}
	# Show the control panel
	if ($this->{input}{section_id}){
		$out = $this->classified_control_panel($this->{input}{section_id});
	# Section selector
	}else{
		$out = $this->set_section_id();
	}
	return $out;
}
# Control panel
sub classified_control_panel{
	# $sectionid: section id
	my ($this,$sectionid) = @_;
	# Expire classifieds
	$this->expireClassified();
	# Fetch 'inactive' classifieds
	my $classifieds = $this->pendingClassified($sectionid);
	my @data;
	# Report header
	my @columns = ("member_id","category","scope","title","email","location","expiry_date");
	my $_C = new Modules::Classified::Classified();
	my @header = map { $_C->label($_) } @columns;
	unshift @header, "Options";
	foreach my $clssfd(@$classifieds){
		my $C = new Modules::Classified::Classified(data=>$clssfd);
		# Report options for each record
		my @options = (
		{url=>"javascript:activate(".$C->id().")",image=>"active",image_attr=>{label=>"Activate ".$C->showdata("title")}},
		{url=>$this->link(cmd=>"edit",classified_id=>$C->id()),image=>"edit",image_attr=>{height=>$config{db_ops}{edit}{img_height},width=>$config{db_ops}{edit}{img_width}}},
		{url=>"javascript:delete_record(".$C->id().")",image=>"delete",image_attr=>{height=>$config{db_ops}{delete}{img_height},width=>$config{db_ops}{delete}{img_width}}},
		);
		# Report row data
		my @rdata = map {$C->showdata($_)} @columns;
		unshift @rdata, $this->recordOption(\@options);
		push(@data,\@rdata);
	}
	my %report = (
		title=>"Pending approval classifieds",
		headers=>\@header,
		data=>\@data,
		tools=>$this->recordOption([{url=>$this->link(action=>"new"),image=>"new",image_attr=>{height=>$config{db_ops}{new}{img_height},width=>$config{db_ops}{new}{img_width}}}]),
	);
	# Create the report
	my %data = (
		report=>$this->ReportBuilder(\%report),
		report_option=>$this->adminReportOption(),
	);
	return $this->merge($this->html_controlpanel_template(),\%data);
}
# Expire classifieds
sub expireClassified{
	my ($this) = @_;
	# Expire classifieds
	$this->DB()->custom_query("update classified set status = 'expired' where expiry_date < ?",$this->getCurrentDate());
	return;
}
# Admin report option
sub adminReportOption{
	my ($this) = @_;
	# Section id
	my $sectionid = $this->{input}{section_id};
	# Admin options
	my @options = (
	{url=>$this->link(cmd=>"current"),label=>"View Current Ads"},
	{url=>"$config{server}{server}$config{server}{CGIpath}/webdb.cgi?action=list&tab=classified",label=>"View All Ads"},
	{url=>$this->link(cmd=>"removeallexpired",section_id=>$sectionid),label=>"Remove Expired Ads"},	
	);
	return $this->commandOption($this->html_optionBulletStyleTemplate(),\@options);
}
# Pending for approval classifieds
sub pendingClassified{
	my ($this,$sectionid) = @_;
	my @records = $this->DB()->fetch_match("classified",{section_id=>$sectionid,status=>"inactive"});
	return \@records
}

# Record options
sub recordOption{
	# $options: command options(array of hash references)
	my ($this,$options) = @_;
	return $this->commandOption($this->html_optionSimpleStyleTemplate(),$options);
}
# Take command option data and turn them into links
sub commandOption{
	# $template: template
	# $options: commands options(array of hash references)
	my ($this,$template,$options) = @_;
	my $out;
	foreach my $rec(@$options){
		my %data = (
			record_url=>$rec->{url},
			# Use icon / text for the link
			record_anchor=>(exists($rec->{image}) ? $this->getIcon($rec->{image},$rec->{image_attr}) : $rec->{label}),
		);
		$out .= $this->merge($template,\%data)
	}
	return $out;
}
# Get a predefined icon
sub getIcon{
	# $name: name of the icon
	# $opt: optional parameters
	my ($this,$name,$opt) = @_;
	my $path = "$config{server}{HTMLpath}/_Modules/Classified";
	# Define icons
	my %icons = (
		edit=>{url=>"$path/edit12.gif",height=>13,width=>12,label=>"Edit"},
		delete=>{url=>"$path/rm12.gif\" height=\"$config{db_ops}{delete}{img_height}",height=>13,width=>12,label=>"Delete"},
		new=>{url=>"$path/new12.gif",height=>13,width=>12,label=>"New"},
		active=>{url=>"$path/approve12.gif",height=>13,width=>12,label=>"Active"},
	);
	my %data = (
		icon_url=>$icons{$name}{url},
		icon_width=>($opt->{width} || $icons{$name}{width}),
		icon_height=>($opt->{height} || $icons{$name}{height}),
		icon_label=>($opt->{label} || $icons{$name}{label}),
	);
	my $template = $this->html_iconTemplate();
	return $this->merge($template,\%data);	
}
# Delete a classified
sub deleteClassified{
	# $classifiedid: classified id
	my ($this,$classifiedid) = @_;
	my $C = new Modules::Classified::Classified(id=>$classifiedid);
	$C->delete();
	return;
}

# New classified
sub newClassified{
	# $sectionid: section id
	my ($this,$sectionid) = @_;
	if (!$this->allowNewPost()){
		return $this->run_handler("ClassifiedPostDenied",$this) || "Access Denied";
	}
	my %data = (
		section_id=>$sectionid,
		category=>$this->{input}{category},
	);
	# Access level
	my $access = $this->access();
	# Hide form fields
	my @hide = ("section_id","member_id","expiry_date","status");
	# Member/admin
	if ($access){
		# Set classified status
		$data{status} = $this->setNewStatus();
		# Member id
		$data{member_id} = $this->DB()->my_uid();
	}else{
		$data{status} = "inactive";
	}
	# If allow_scope setting isn't set to both local and global
	if ($this->isNotLocalGlobal()){
		# Set the scope to global
		if ($this->isGlobal()){
			$data{scope} = "global";
		# Set the scope to local
		}else{
			$data{scope} = "local";
		}
		# Hide scope input field
		push(@hide,"scope");
	}
	# Set default expiry date
	$data{expiry_date} = $this->getExpiryDate();
	# input_exsite handler
	$this->DB()->handler("input_exsite",\&custom_input_exsite);
	my %make_option = (
		table=>"classified",
		data=>\%data,
		hide=>\@hide,
		action=>$this->link(section_id=>undef,action=>undef,cmd=>undef,category=>undef),
		extra=>{
			reply=>$this->reply(),
			cmd=>"process_new",
		},
	);
	# Create the new classified form
	return $this->DB()->make(%make_option);
}
# Process new classified
sub processNew{
	my ($this) = @_;
	my %rec;
	foreach my $key (keys(%{$this->{input}})){
		# Gather classified info
		if ($key =~ /classified##(.+)/i){
			my $column = $1;
			$rec{$column} = $this->{input}{$key};
		}
	}
	# If a non-admin user tries to add an ad, its status is set to 'inactive'
	if ($rec{scope} =~ /global/i && !$this->authorize()){
		$rec{status} = "inactive";
	}
	my $C = new Modules::Classified::Classified();
	foreach my $key (keys(%rec)){
		$C->setdata($key,$rec{$key});
	}
	$C->save();
	$this->redirect($this->{input}{reply});
	return;
}

# Set status for new classified
sub setNewStatus{
	my ($this) = @_;
	# Access level
	my $access = $this->access();
	# Set the status to 'active', if this user is an admin
	if ($this->authorize()){
		return "active";
	}
	# Set the status to 'active', if this user is a member and the allow_scope is set to 'local'
	if ($access && $this->isLocal()){
		return "active";
	}
	return "inactive";
}
# Is the scope setting local?
sub isLocal{
	my ($this) = @_;
	return 1 if $config{Classified}{allow_scope} =~ /local/i;
	return 0;
}
# Is the scope setting global?
sub isGlobal{
	my ($this) = @_;
	return 1 if $config{Classified}{allow_scope} =~ /global/i;
	return 0;
}
# Is the scope setting  set to either local or global?
sub isNotLocalGlobal{
	my ($this) = @_;
	return 1 if !$this->isLocal();
	return 1 if !$this->isGlobal();
	return 0;
}
# Is the scope setting set to both local and global?
sub isLocalGlobal{
	my ($this) = @_;
	return 1 if $this->isLocal() && $this->isGlobal();
	return 0;
}
# Edit a classified
sub edit{
	# $classifiedid: classified id
	my ($this,$classifiedid) = @_;
	# Fetch classified record
	my %rec = $this->DB()->fetch("classified",$classifiedid);
	# Hide these input fields
	my @hide = ("section_id","member_id","expiry_date","status");
	my %make_option = (
		table=>"classified",
		data=>\%rec,
		hide=>\@hide,
		extra=>{
			reply=>$this->reply()
		},
	);
	return $this->DB()->make(%make_option);
}
# Set a classified to active
sub setActive{
	# $classifiedid: classified id
	my ($this,$classifiedid) = @_;
	return if !$this->authorize();
	$this->setStatus($classifiedid,"active");
	return;
}
# Set the status of a classified
sub setStatus{
	# $id: classified id
	# $status: status
	my ($this,$id,$status) = @_;
	my $C = new Modules::Classified::Classified(id=>$id);
	$C->setdata("status",$status);
	$C->save();
	return;
}
# Delete all expired classifieds
sub removeAllExpired{
	my ($this) = @_;
	return if !$this->authorize();
	# Delete all 'expired' classifieds
	my @records = $this->DB()->custom_query("select * from classified where status = ?","expired");
	foreach my $rec(@records){
		my $C = Modules::Classified::Classified(data=>$rec);
		$C->delete();
	}
	return;	
}
# Fetch active classifieds
sub activeClassified{
	# $sectionid: section id
	# $category: category
	my ($this,$sectionid,$category) = @_;
	# Get current date
	my $today = $this->getCurrentDate();
	my @param = ("active",$today);
	# By default show local ads
	my $sql = "select * from classified where status = ? && expiry_date >= ?";
	if ($category){
		$sql .= " && category = ?";
		push(@param,$category);
	}
	my @records = $this->DB()->custom_query($sql,@param);
	return\@records;
}
# Counting the number of classifieds in a category
sub groupCategory{
	# $records: records
	my ($this,$records) = @_;
	my %category;
	foreach my $rec(@$records){
		$category{$rec->{category}}++;
	}
	return \%category;
}
# Show category list
sub showCategoryList{
	# $sectionid: section id
	my ($this,$sectionid) = @_;
	my %data;
	# Classifieds
	my $classifieds = $this->activeClassified($sectionid);
	if (scalar(@$classifieds) < 1){
		$data{category_list} = "No classifieds found";
	}
	# Count the number of classifieds in each category
	my %category = %{$this->groupCategory($classifieds)};
	my @category;
	foreach my $cat(sort {$a cmp $b} keys(%category)){
		my $category = &html_escape($cat);
		my %rdata = (
			category_url=>$this->link(category=>$category,action=>undef),
			category_name=>$category,
			category_count=>$category{$cat},
		);
		# Report rows
		my $template = $this->html_categoryStyleTemplate();
		$data{category_list} .= $this->merge($this->html_categoryStyleTemplate,\%rdata);
	}
	my $new_url = $this->link(action=>"new");
	my $new_text = "Post a classified ad";
	$data{classified_add} = <<END;
<div class="ClassifiedNew"><a href="$new_url" class="ClassifiedNewLink">$new_text</a></div>
END
	return $this->merge($this->html_categoryListTemplate(),\%data);
}
# Sort classifed by section id, date and title
sub sortClassified{
	# $data: classifieds
	# $sectionid: section id
	my ($this,$data,$sectionid) = @_;
	# Change all local global ads' scope to 'local'
	for(my $i = 0; $i < @$data; $i++){
		if ($data->[$i]{section_id} == $sectionid){
			$data->[$i]{scope} = "local";
		}
	}
	# Sort by:
	# 1. scope descending
	# 2. section id accending
	# 3. creation date descending
	my @sorted = sort { $b->{scope} cmp $a->{scope} || $a->{section_id} <=> $b->{section_id} || $b->{date} cmp $a->{date}} @$data;
	return \@sorted;
}
# Show classifieds
sub showClassified{
	# $sectionid: section id
	# $category: category
	my ($this,$sectionid,$category) = @_;
	# Fetch active classifieds
	my $classifieds = $this->activeClassified($sectionid,$category);
	# Sort classifieds
	my $sorted = $this->sortClassified($classifieds,$sectionid);
	my %data = (
		classified_category=>$category,
	);
	foreach my $clssfd(@$sorted){
		my $C = new Modules::Classified::Classified(data=>$clssfd);
		# Contact form url
		my $classified = $C->get();
		$classified->{classified_url} = $this->link(action=>"contact",classified_id=>$C->id());
		$classified->{classified_title} = $C->showdata("title");
		$classified->{classified_description} = $C->showdata("description");
		$classified->{classified_location} = $C->showdata("location");
		$classified->{classified_creationdate} = $C->showdata("date");
		$classified->{classified_edit_url} = $this->link(cmd=>"edit",classified_id=>$C->id());
		$classified->{classified_delete_url} = "javascript:delete_record(".$C->id().")";
		my $template = $this->authorize() ? $this->html_classifiedStyleTemplateAdmin() : $this->html_classifiedStyleTemplatePublic();
		$data{classified_list} .= $this->merge($template,$classified);

		$data{classified_add_new_text} = "Post a classified ad";
		$data{classified_add_new_url} = $this->link(action=>"new");
	}
	my $template = $this->authorize() ? $this->html_classifiedListTemplateAdmin() : $this->html_classifiedListTemplatePublic();
	return $this->merge($template,\%data);
}
# Alow the user to post new classifieds?
sub allowNewPost{
	my ($this) = @_;
	# Access level
	my $access = $this->access();
	# Members are allowed to post new classifieds
	return 1 if $access;
	# Config setting allows anyone to post new classifieds
	return 1 if $config{Classified}{allow_anonymous};
	return 0;
}
# Send the contact an email
sub processContactForm{
	# $in: post data
	my ($this,$in) = @_;
	my @error;
	if ($in->{email} !~ /\@/){
		push @error,"email address";
	}
	if (!$in->{message}){
		push @error, "message";
	}
	if (scalar(@error) > 0){
		return "<p>Please enter your ".join(" and ",@error).".<br><br><input type=\"button\" value=\"&lt; Back\" onClick=\"history.back()\"></p>";
	}
	# Classified data
	my %classified = $this->DB()->fetch("classified",$in->{classified_id});
	my %bdata = (
		title=>"$classified{title}",
		description=>"$classified{description}",
		message=>&html_to_plaintext("$in->{message}"),
		footer=>"$config{Classified}{email_reply_footer}"
	);
	my $to = $classified{email};
	my $from = $in->{email};
	if ($to && $from){	
		&SendMail($to,$from,"Re: $classified{title}",&substitute("Classified:\n\n[[title]]\n\n[[description]]\n\n\nResponse:\n\n[[message]]\n\n\n[[footer]]",\%bdata));
	}
	return "<p>Your message has been sent to the advertiser.<br><a href=\"$in->{reply}\">Click here to go back to the listing.</a></p>";
}
# Contact email form
sub contactForm{
	my ($this,$classifiedid) = @_;
	my %classified = $this->DB()->fetch("classified",$classifiedid);
	my %form = (
		contact_reply=>$this->link(action=>undef,classified_id=>undef),
		contact_post_url=>$this->link(action=>"contact_post",classified_id=>undef,category=>undef,section_id=>undef),
		contact_email=>$classified{email},
		contact_subject=>&html_escape("Re: $classified{title}"),
		contact_message=>&html_escape($classified{description}),
		contact_classified_id=>$classified{classified_id},
	);
	return $this->merge($this->html_contactFormTemplate(),\%form);
}
# Control panel template
sub html_controlpanel_template{
	my ($this) = @_;
	my $out = $this->javascript();
	$out .= "
[[report]]
[[report_option]]
	";
	return $out;
}
# Contact form template
sub html_contactFormTemplate{
	my ($this) = @_;
	my $disclaimer = $config{Classified}{disclaimer} || <<END;
Buyer beware! We have not confirmed that the goods or services offered by this ad are legitimate or of the stated condition/quality.  
The buyer is solely responsible for evaluating the purchase, and negotiating a fair exchange with the seller.  
If you are unable to view the goods directly (because you are in a different city, for instance), 
then the risk of buying sight unseen is entirely yours.
END
	return <<END;
<h1 class="ClassifiedCategory">Reply Form</h1>
<form method="post" action="[[contact_post_url]]">
<input type="hidden" name="reply" value="[[contact_reply]]">
<input type="hidden" name="classified_id" value="[[contact_classified_id]]">
<table cellspacing="5">
<tr>
	<td valign="top"><b>Your email address:(required)</b></td><td><input type="text" name="email" value=""></td>
</tr>
<tr>
	<td valign="top">Subject:</td><td>[[contact_subject]]</td>
</tr>
<tr>
	<td valign="top">Message:</td><td><textarea cols="80" rows="15" name="message"></textarea></td>
</tr>
<tr>
	<td valign="top" colspan="2"><hr><i>$disclaimer</i><hr></td>
</tr>
<tr>
	<td>&nbsp;</td><td><input type="submit" value="Submit">&nbsp;&nbsp;&nbsp;&nbsp;<input type="reset" value="Reset"></td>
</tr>
</table>
</form>
END
}
# Classified template - admin
sub html_classifiedStyleTemplateAdmin{
	my ($this) = @_;
	return <<END;
<div class="ClassifiedItem">
<div class="ClassifiedSection">Section: [[section_id]]</div>
<div class="ClassifiedOpition">[<a href="[[classified_edit_url]]">Edit</a> | <a href="[[classified_delete_url]]">Delete</a>]</div>
<div class="ClassifiedTitle"><a href="[[classified_url]]">[[classified_title]]</a> </div>
<div class="ClassifiedDescription">[[classified_description]]</div>
<div class="ClassifiedLocation">[[classified_location]]</div>
<div class="ClassifiedDate">posted [[classified_creationdate]]</div>
</div>
END
}
# Classified template - public
sub html_classifiedStyleTemplatePublic{
	my ($this) = @_;
	return <<END;
<div class="ClassifiedItem">
<div class="ClassifiedTitle"><a href="[[classified_url]]">[[classified_title]]</a></div>
<div class="ClassifiedDescription">[[classified_description]]</div>
<div class="ClassifiedLocation">[[classified_location]]</div>
<div class="ClassifiedDate">posted [[classified_creationdate]]</div>
</div>
END
}
# Classified list template - admin
sub html_classifiedListTemplateAdmin{
	my ($this) = @_;
	return $this->javascript().<<END;
<style>
.ClassifiedItem {padding: 10px 0px 10px 0px}
</style>
<h1 class="ClassifiedCategory">[[classified_category]]</h1>
<div class="ClassifiedNewTop"><a href="[[classified_add_new_url]]" class="ClassifiedNewLink">[[classified_add_new_text]]</a></div>
[[classified_list]]
<div class="ClassifiedNewBottom"><a href="[[classified_add_new_url]]" class="ClassifiedNewLink">[[classified_add_new_text]]</a></div>
END
}
# Classified list template - public
sub html_classifiedListTemplatePublic{
	my ($this) = @_;
	return <<END;
<h1 class="ClassifiedCategory">[[classified_category]]</h1>
[[include:classified_add_new_url]]
<div class="ClassifiedNewTop"><a href="[[classified_add_new_url]]" class="ClassifiedNewLink">[[classified_add_new_text]]</a></div>
[[/include:classified_add_new_url]]
[[classified_list]]
[[include:classified_add_new_url]]
<div class="ClassifiedNewBottom"><a href="[[classified_add_new_url]]" class="ClassifiedNewLink">[[classified_add_new_text]]</a></div>
[[/include:classified_add_new_url]]
END
}
# Category list template
sub html_categoryListTemplate{
	my ($this) = @_;
	return <<END;
[[classified_add]]
[[category_list]]
END
}
# Category template
sub html_categoryStyleTemplate{
	my ($this) = @_;
	return <<END;
<div class="ClassifiedCategoryItem"><span class="ClassifiedCategoryName"><a href="[[category_url]]" class="ClassifiedCategoryUrl">[[category_name]]</a></span> <span class="ClassifiedCategoryCount">([[category_count]])</span></div>
END
}
# Member report javascript
sub javascript{
	my ($this) = @_;
	my $url_delete = $this->link(cmd=>"del",classified_id=>undef,__plaintext=>1);
	my $url_activate = $this->link(cmd=>"set_active",classified_id=>undef,__plaintext=>1);
	return <<END;
<script language="javascript">
<!--
function delete_record(id){
	if(confirm('Are you sure you want to delete this classified?')){
		location.href = '${url_delete}&classified_id=' + id;
	}
}
function activate(id){
	if(confirm('Are you sure you want to approve this classified?')){
		location.href = '${url_activate}&classified_id=' + id;
	}
}
// -->
</script>
END
}
# Icon template
sub html_iconTemplate{
	my ($this) = @_;
	return "<img src=\"[[icon_url]]\" align=\"middle\" hspace=\"3\" border=\"0\" width=\"[[icon_width]]\" height=\"[[icon_height]]\" title=\"[[icon_label]]\" alt=\"[[icon_label]]\">";
}
# Bulleted style template
sub html_optionBulletStyleTemplate{
	my ($this) = @_;
	return "<li><a href=\"[[record_url]]\">[[record_anchor]]</a></li>";
}
# Simple style template
sub html_optionSimpleStyleTemplate{
	my ($this) = @_;
	return "<a href='[[record_url]]'>[[record_anchor]]</a>";
}
#----------------------------------------------------------------------
sub publish_mode{
	my ($this) = @_;
	if ($share{Page}){
		if ($share{Page}->is_publishing()){
			return 1;
		}
	}
	return 0;
}
# Get member access level
sub access{
	my ($this) = @_;
	if ($this->publish_mode()){
		return 0;
	}
	return $this->DB()->authorize();
}
# authorization check
sub authorize{
	my ($this) = @_;
	if ($this->publish_mode()){
		return 0;
	}
	my $access = $this->access();
	return $access if $this->DB()->is_admin();
	return $access if $this->DB()->owns();
	return 0;
}
sub reply{
	my ($this) = @_;
	return $this->link(classified_id=>undef,cmd=>undef,action=>undef,__plaintext=>1);
}
# Page redirection
sub redirect{
	# $reply: reply url
	my ($this,$reply) = @_;
	$reply = $reply || $this->link(action=>undef,__plaintext=>1);
	if ($reply !~ /^http/i){
		$reply = $config{server}{server}.$reply;
	}
	return ExSite::Util::redirect($reply);
}
# Data merge
sub merge{
	# $view: template
	# $data: merge data
	my ($this,$view,$data) = @_;
	return if !$view || !$data;
	return ExSite::Misc::substitute($view,$data);
}
# is admin?
sub is_admin{
	my ($this) = @_;
	return $this->DB()->is_admin();
}
#----------------------------------------------------------------------
# input_exsite handler
sub custom_input_exsite{
	my ($this,%opt) = @_;
	return undef if $this->is_admin();
	# Create drop list fields with default value selected
	if ($opt{table} eq "classified" && $opt{column} =~ /category|scope/i){
		# Get datatype
		my $dtype = $this->{map}->get_datatype_attr($opt{datatype},"regexp");
		my $out = &exsite_list(\%opt,$dtype);
		$opt{input} = $out;
		return $out;
	}
	return undef;
}

# Create the drop down list input field
sub exsite_list{
	my ($data,$datatype) = @_;
	my $out = "<select name=\"$data->{name}\">";
	my @dtype = split(/\|/,$datatype);
	foreach my $type(@dtype){
		my $val = &html_escape($type);
		$out .= "<option value=\"$val\"";
		$out .= $val eq $data->{value} ? " selected" : "";
		$out .= ">$val</option>";
	}
	$out .= "</select>";
	return $out;
}
#----------------------------------------------------------------------
# Get the ExSite::Time object
sub Time{
	my ($this) = @_;
	return &get_obj("Time");
}
# Build reports using ExSite::ReportBuilder
sub ReportBuilder{
	# $opt: option(hash reference)
	# 	title: report title
	# 	data: report data
	# 	tools: report tool option
	# 	headers: report header
	# 	foot: report footer
	# 	width: report width
	my ($this,$opt) = @_;	
	my $rb = &get_obj("ReportBuilder");
	$rb->init(
		title=>$opt->{title},
		data=>$opt->{data},
		tools=>$opt->{tools},
		headers=>$opt->{headers},
		foot=>$opt->{foot},
		width=>$opt->{width},
	);
	return $rb->make();
}
sub DB{
	return $share{DB};
}
#----------------------------------------------------------------------
# Get current date
sub getCurrentDate{
	my ($this) = @_;
	$this->Time()->set();
	return $this->Time()->write("sql_date");
}
# Get expiry date
sub getExpiryDate{
	my ($this) = @_;
	my $lifetime = $config{Classified}{ad_lifetime} || 0;
	$this->Time()->set();
	if ($lifetime){
		$this->Time()->add($lifetime,"days");
	}
	return $this->Time()->write("sql_date");
}
#----------------------------------------------------------------------
package Modules::Classified::Classified;
use strict;
use ExSite::Object;
use base qw (ExSite::Object);

sub my_type{
	return "classified";
}
sub label{
	my ($this,$key) = @_;
	return "" if !$key;
	return $this->DB()->{map}->get_column_attr($this->type(),$key,"label") || "";
}
1;
