package Modules::Dashboard;

#----------------------------------------------------------------------------
#
#   Copyright (C) 2009 - Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
#  Displays summary reports from various other plug-ins.
#
#----------------------------------------------------------------------------

use strict;

# useful kernel libs; you can/should remove any of these that are not needed

use ExSite::Config;          # required
use ExSite::Input;           # optional input manager
use ExSite::Misc;            # optional utils
use ExSite::Util;            # optional utils
use ExSite::ML;              # optional markup-language generation
use ExSite::FormBuilder;     # optional form generation
use ExSite::ReportBuilder;   # optional report generation
use ExSite::Form;            # optional database manager
use ExSite::Module qw(module_list get_module icon);

# recommended base class for plug-in modules

use Modules::BaseDCD;

# declare package globals

use vars qw(@ISA $ml);       # Perl inheritance global & markup language object

# define our class inheritance

@ISA = qw(Modules::BaseDCD); # inherit from this base class



# write method (builds output for regular web pages)

sub write {
    my ($this,$options) = @_;
    my @mod = $options ? split(/,/,$options) : @{$config{Dashboard}{modules}};
    my $out = $this->dashboard($share{DB}->my_gid,@mod);
    return $out;
}

# ioctl method (used by ExSite to query the module for its functionality)

sub ioctl {
    my $this = shift;
    $_ = shift;           # $_ is the ioctl request

    if (/isRestricted/) {
	return 0;
    }
    elsif (/isService/) {
	return 0;
    }
    elsif (/ModuleName/) {
	return "Dashboard";
    }
    elsif (/ModuleInfo/) {
	return "Displays summary reports from various other plug-ins.";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/Category/) {
	return "System";
    }
}

sub ctrl_panel {
    my $this = shift;
    my @mod = $config{Dashboard}{modules}?
	@{$config{Dashboard}{modules}} :      # preconfigured list
	&module_list;                         # all modules
    $this->{admin} = 1;
    my $section_id = $this->get_section_id;
    my $out = $section_id ?
	$this->dashboard($this->get_section_id,@mod) :
	$this->set_section_id();
    return $out;
}

sub dashboard {
    my ($this,$sid,@mod) = @_;
    my $out;
    my $ml = &get_obj("ML");
    my $level = $share{DB}->level;
    return "Dashboard: permission denied" if ($level < 2);
    foreach my $modname (@mod) {
	my $mod = &get_module($modname);
	my $title = $modname;
	my $pane;
	if ($mod) {
	    $title = $mod->ioctl("ModuleName") || $modname;
	    my $summary = $mod->ioctl('Summary');
	    if (ref $summary eq "CODE") {
		my $iconurl = &icon($modname);
		# find the control panel URL
		my $url = "$config{server}{CGIpath}/$config{prog}{ctrlpanel}/$modname?section_id=$sid";
		$pane = $ml->div($mod->summary($sid),{class=>"DashboardContainer"});

		my $mod_access = $mod->ioctl('Access');
		if (! defined $mod_access || $mod_access <= $level) {
		    $pane .= $ml->div(
				      $ml->a(
					     $ml->img(undef,{src=>$iconurl,height=>16,width=>16}).
					     " launch ".$ml->b($title)." control panel",
					     {href=>"javascript:popup_large('$url')",class=>"button"}
					     ),
				      { class=>"DashboardLauncher" }
				      );

		}
	    }
	}
	elsif ($config{debug}) {
	    $pane = $@;
	}
	if ($pane) {
	    $out .= &ExSite::HTML::BasicBox(
					    title=>$title,
					    pane=>$pane,
					    );
	}
    }
    return $out;
}

1;
