#!/usr/bin/perl
#-----------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

package Modules::Event::Date;

use strict;
use ExSite::Config;
use ExSite::Base;
use ExSite::Form;
use ExSite::Misc;
use ExSite::Time;
use ExSite::ML;
use ExSite::Object;

use vars qw(@ISA);
@ISA = qw(ExSite::Object);
#@ISA = qw(ExSite::Time ExSite::Object);

sub my_type { return "evt_date"; }    # generic object has no preset type

sub name {
    my $this = shift;
    if ($this->defined()) {
	return $this->getdata("description") || $this->getdata("type");
    }
    return "n/a";
}

sub showdata {
    my ($this,$key) = @_;
    if ($key =~ /time/) {
	# zero times are treated as undefined
	my $tdata = $this->getdata($key);
	if ($tdata !~ /^0+:0+:0+$/) {
	    my $t = new ExSite::Time;
	    $t->set($tdata,"hms");
	    return $t->write("time");
	}
	else {
	    return " ";
	}
    }
    elsif ($key =~ /date/) {
	my $t = new ExSite::Time;
	$t->set($this->getdata($key),"sql_date");
	return $t->write("weekday_long").", ".$t->write("date_long");
    }
    return $this->SUPER::showdata($key);
}

sub show {
    my ($this,%opt) = @_;
    my $out = $this->getdata("description");
    if ($out) { $out .= ": "; }
    $out .= $this->show_date_range();
    return $out;
}

# display start and end date as a single value
# can optionally pass the start/end datetimes

sub show_date_range {
    my ($this,$startdate,$enddate,$ndate) = @_;
    $ndate or $ndate = 1;
    my $t1 = $startdate ? 
	new ExSite::Time($startdate,"sql_datetime") : $this->get_start();
    my $t2 = $enddate ? 
	new ExSite::Time($enddate,"sql_datetime") : $this->get_end();

    if ($t1->diff($t2) <= 0) {
	return $this->show_start($t1);
    }

    if ($t2->write("sql_date") ne $t1->write("sql_date")) {
	# different days
	if ($ndate==1 && ($t1->write("time24") !~ /^[\s0]0:00/ || $t2->write("time24") !~ /^[\s0]0:00/)) {
	    # time is significant, show complete dates
	    return $this->show_start($t1) . " - " . $this->show_end($t2);
	}
	elsif ($t1->write("year") != $t2->write("year") || $t1->write("raw_month") != $t2->write("raw_month")) {
	    # different months
 	    return $t1->write("date_long") . " - " . $t2->write("date_long");
 	}
 	elsif ($t1->write("raw_day") != $t2->write("raw_day")) {
	    # different days in same month
 	    return $t1->write("month_long") . " " . $t1->write("raw_day") . " - " . $t2->write("raw_day") . ", " . $t1->write("year");
 	}
    }
    else {
	# same day 
        my $out = $t1->write("date")." ";

	if ($t1->write("time") != /^[\s0]0:00/) {
	    $out .= $t1->write("time");
	}
        if ($t2->write("time") ne $t1->write("time") && $t2->write("time") != /^[\s0]0:00/) {
	    $out .= " - " . $t2->write("time");
	}
	return $out;
    }
}

sub show_start {
    my ($this,$start) = @_;
    $start or $start = $this->get_start();
    return $this->show_date($start);
}

sub show_end {
    my ($this,$end) = @_;
    $end or $end = $this->get_end();
    return $this->show_date($end);
}

sub show_date {
    my ($this,$date) = @_;
    $date or $date = $this->get_start();
    if ($date->write("time24") =~ /^[\s0]0:00/) {
	# ignore 00:00 times
	return $date->write("date_long");
    }
    else {
	return $date->write("datetime2");
    }
}

sub get_start {
    my ($this,%opt) = @_;
    my $t = new ExSite::Time;
    $t->set($this->get_datetime(),"sql_datetime");
    return $t;
}

sub get_end {
    my ($this,%opt) = @_;
    my $t = new ExSite::Time;
    $t->set($this->get_datetime("enddate","endtime"),"sql_datetime");
    return $t;
}

sub getTime {
    my $this = shift;
    return $this->get_start(@_);
}

sub get_datetime {
    my ($this,$dkey,$tkey) = @_;
    $dkey or $dkey = "date";
    $tkey or $tkey = "time";
    my $time = $this->{data}{$tkey} || "00:00";
    return $this->{data}{$dkey}." ".$time;
}

1;

