package Modules::EvtCal;

# Calendar tool that works with the Event framework

use strict;
use ExSite::Config;
use ExSite::Input;
use ExSite::Misc;
use ExSite::Util;
use ExSite::URI;
use ExSite::Mail;
use ExSite::Image;
use ExSite::Module qw(&site_license);
use ExSite::Time;
use ExSite::RSS;

use Modules::BaseDCD;
use Modules::Event::Event;
use Modules::Event::Date;
use Modules::ID::Contact;

use vars qw(@ISA $EvtCal);
@ISA = qw(Modules::BaseDCD);

use vars qw($ml @wd @wdy @weekday @mon @month @dayinmonth $notification);

@wd = qw(Su Mo Tu We Th Fr Sa);
@wdy = qw(Sun Mon Tue Wed Thu Fri Sat);
@weekday = qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday);
@mon = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
@month = qw(January February March April May June July August September October November December);
@dayinmonth = qw(31 28 31 30 31 30 31 31 30 31 30 31);

$notification = "Posted by: [[from]]
<h1>[[name]]</h1>
[[description]]
<h2>Date</h2>
[[date]] [[time]]\n";

# write:
# show / next
# list / upcoming
# calendar

sub write {
    my ($this,$options) = @_;
    my $out;
    $ml = new ExSite::ML;
    my %opt = &DecodeString($options);
    $EvtCal = $this;

    if ($this->{input}{fee}) {
	# fees are handled by Event Registration
	return "<!--&EvtReg()-->\n";
    }

    # which section's calendar are we working with?
    if ($opt{section_id} && $config{EvtCal}{share_calendars}) {
	$this->{section_id} = $opt{section_id};
    }
    else {
	$this->{section_id} = $share{Page}->id("section");
    }

    # get the event
    my $evt = $this->{input}{evt} || $opt{evt};
    if ($evt) {
	$this->fetch($evt);
	if (! $this->allow_view($this->{section_id})) {
	    return $this->error("Event $evt: permission denied");
	}
    }
    $this->setup_querylib();
    $this->set_date();

    # include default CSS
    if ($opt{css}) {
	$out = $ml->link(undef,{rel=>"stylesheet",href=>"$config{server}{HTMLpath}/_ExSite/css/HTML.css",type=>"text/css"});
	$out .= $ml->link(undef,{rel=>"stylesheet",href=>"$config{server}{HTMLpath}/_Modules/EvtCal/EvtCal.css",type=>"text/css"});
    }

    my $cmd = $this->{input}{evtcmd};
    if (! $cmd) {
	if ($evt) {
	    $cmd = "show";
	}
	elsif ($opt{evtcmd}) {
	    $cmd = $opt{evtcmd};
	}
	else {
	    $cmd = "list";
	}
    }

    if ($cmd eq "list") {
	if (! exists $opt{showmonth}) {
	    $opt{showmonth} = $config{EvtCal}{showmonth};
	}
	$out .= $this->list(%opt);
	$out .= $this->rss_link();
    }
    elsif ($cmd eq "show" && $this->{evt}) {
	$opt{url} = new ExSite::URI;
	$out .= $this->show(%opt);
	$out .= $ml->p($ml->a($msg{"Back to calendar"},{href=>$this->link(evt=>undef)}),{class=>"EvtCalCalendarLink"});
    }
    elsif ($cmd eq "vcal" && $this->{evt}) {
	return $this->vcalendar();
    }
    elsif ($cmd eq "new") {
	if ($config{EvtCal}{new_event_access} eq "public" || 
	    ($config{EvtCal}{new_event_access} eq "members" && $share{DB}->level())) {
	    # users can create new events
	    $out .= $this->newevent_simple();
	}
	else {
	    $out .= $this->error($msg{"Sorry, only $config{EvtCal}{new_event_access} may post new events."});
	}
    }
    else { #if ($cmd eq "cal") {
	$out .= $this->calendar(%opt);
	$out .= $this->rss_link();
    }
    return $ml->div($out, { class => "EvtCal" });
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/ModuleName/) {
	return "Event Calendar";
    }
    elsif (/isRestricted/) {
	return defined $config{EvtCal}{isRestricted} ?
	    $config{EvtCal}{isRestricted} : 0;
    }
    elsif (/isService/) {
	return 1;
    }
    elsif (/Publish/) {
	return \&publish;
    }
    elsif (/Unpublish/) {
	return \&unpublish;
    }
    elsif (/Summary/) {
	return \&summary;
    }
    elsif (/DynContentParameter/) {
	$ml = new ExSite::ML;
	my $selectOptions = $ml->select([
			      		        ["option", "(default)", { value => "" }],
					        ["option", "Upcoming Events", { value => "evtcmd=list" }],
					        ["option", "Next Event", { value => "evtcmd=show" }],
					        ["option", "Month view", { value => "evtcmd=>cal" }],
					        ["option", "New Event Form", { value => "evtcmd=new" }],
			      		      ],
			      		      { name => "param" });
	
	return $ml->Element("p", $ml->b("Option:") . $selectOptions);
    }
    elsif (/ModuleInfo/) {
	return "The Event Calendar tool displays and manages your event
listings. Users can navigate the calendar by month, or visit particular 
days that contain events.  Individual events may be visited, to view event 
details, view event web pages, or download brochures or other info.";
    }
    elsif (/Dependencies/) {
	return ["EvtReg","AddrBook"];
    }
    return;
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    my $in = $this->{input};
    my $db = $share{DB};
    $EvtCal = $this;
    $ml = &get_obj("ML");
    $this->setup_querylib();
    $this->set_date();

    $this->{admin} = 1;  # set admin flag to add admin tools

    my @path = ( $ml->a("Top",{href=>$this->link(caladm=>undef,evt=>undef,section_id=>undef)}) );
    if ($in->{section_id}) {
	my $s = $db->fetch("section",$in->{section_id});
	push @path, $ml->a($s->{title},{href=>$this->link(caladm=>undef,evt=>undef)})
    }
    my $evt;
    if ($in->{evt}) {
	if (! $in->{caladm}) { $in->{caladm} = "show"; }
	$evt = $this->fetch($in->{evt});
    	return $ml->p("Invalid event ID.", { class => "error" }) if (! $evt);
	push @path, $ml->a($evt->name,{href=>$this->link(caladm=>"show")})
    }
    if ($in->{fee}) {
	return $this->register();
    }
    $out .= $ml->p(join("&nbsp;&gt;&nbsp;",@path));

    $_ = $in->{caladm};
    if (/^new$/) {
	$out .= $this->newevent;
    }
    elsif (/new_evt_date/) {
	$out .= $this->new_evt_date();
    }
    elsif (/edit_evt/) {
	$out .= $this->edit_evt();
    }
    elsif (/^schedule$/) {
	$out .= $this->show_schedule_admin();
    }
    elsif (/edit_schedule/) {
	$out .= $this->edit_schedule();
    }
    elsif (/del_schedule/) {
	$out .= $this->del_schedule();
    }
    elsif (/new_schedule/) {
	$out .= $this->new_schedule();
    }
    elsif (/venue/) {
	$out .= $this->venues();
    }
#    elsif (/edit_contacts/) {
#	$out .= $this->contacts();
#    }
#    elsif (/new_contact/) {
#	$out .= $this->new_contact();
#    }
    elsif (/edit/) {
	my $action = $this->link(caladm=>"doform");
	$out .= $ml->h1($config{EvtCal}{heading}{edit});
	$out .= $db->make(table=>"evt",
			  record=>$in->{evt},
			  action=>$action,
			  extra=>{caladm=>"doform"},
			  );
	return $out;
    }
    elsif (/doform/) {
	$out .= $this->doform;
    }
    elsif (/delete/) {
	$share{DB}->trash_key("evt",$in->{evt});
	$ml->location($this->link(__plaintext=>1,caladm=>undef,evt=>undef));
	# errorcheck?
    }
    elsif (/show/) {
	$out .= $this->show_evt_admin($in->{evt});
    }
    elsif (/list/) {
	if ($share{DB}->is_admin()) {
	    $out .= $ml->h1("Shared Events");
	    $out .= $this->report(0);
	}
	$out .= $this->set_section_id;
	$this->{section_id} = $this->get_section_id;
	if ($this->{section_id}) {
	    $out .= $ml->h1($config{EvtCal}{heading}{myevents});
	    $out .= $this->report($this->{section_id});
	}
    }
    else {
	$out .= $this->set_section_id;
	$this->{section_id} = $this->get_section_id;
#	if ($this->{section_id}) {
	    $out .= $ml->p(
			   &ExSite::HTML::Button(label=>"List all events",url=>$this->link(caladm=>"list")).
			   &ExSite::HTML::Button(label=>"add new event",url=>$this->link(caladm=>"new"))
			   );
	    my $date = $in->{date} || $this->today;
	    my ($y,$m,$d) = split /-/,$date;
	    if (! $d) { $d = 1; }
	    $this->{date} = [$y,$m,$d];
	    my @events = $this->fetch_events_by_month("$y-$m-01",$this->{share_section});
	    $out .= $this->show_month({format=>"large"},@events);
#	}
    }
    return $out;
}

sub set_date {
    my $this = shift;
    my ($y,$m,$d,$t);
    my $date = $this->{input}{date};
    if ($date) {
	($y,$m,undef,$d) = $date =~ /^(\d+)-(\d+)(-(\d+))?$/;
	$t = new ExSite::Time([$y,$m,($d || 1),12,0,0],"raw");
    }
    else {
	$t = new ExSite::Time();
	($y,$m,$d,undef,undef,undef) = @{$t->write("raw")};
    }
    $this->{date} = [$y,$m,$d];
    $this->{Time} = $t;
}

sub list {
    my ($this,%opt) = @_;
    my (@events,@mevents,$eventspan,$limit);
    my $db = $share{DB}; 
    my $heading;
    my $t = $this->{Time};
    if ($this->{input}{date}) {
	if ($this->{input}{date} =~ /^\d+-\d+-\d+$/) {
	    # specific day requested
	    @events = $this->fetch_events_by_date($t->write("sql_date"),$this->{section_id});
	    @mevents = $this->fetch_events_by_month($t->write("year-month"),$this->{section_id});
	    $eventspan = "date";
	    $heading = $t->write("date_long");
	}
	else {
	    # a month was requested
	    @events = $this->fetch_events_by_month($t->write("year-month"),$this->{section_id});
	    @mevents = @events;
	    $eventspan = "month";
	    $heading = $t->write("month_long")." ".$t->write("year");
	}
    }
    else {
	# no date requested - list upcoming
	$limit = $opt{limit} || $config{EvtCal}{max_upcoming} || 10;
	@events = $this->fetch_events($t->write("sql_date"),$this->{section_id},$opt{type});
	$heading = $msg{$config{EvtCal}{heading}{upcoming}};
	$eventspan = "website";
    }
    
    my $out .= $ml->h1($heading);
    if ($opt{showmonth}) {
	$opt{format} = "small";
	$out .= $this->show_month(\%opt,@mevents);
    }
    
    if (@events) {
	$out .= $ml->p($msg{"For more details, click on the event link."});
	my $eventSummaries;
	my $lastdate = "0000-00-00";
	my ($dt,$dd,%done,$nevt);
	foreach my $evt (@events) {
	    next if ($evt->{status} && $evt->{status} ne "active");
	    next if ($opt{type} && $evt->{type} ne $opt{type});
	    next if ($evt->{parent} && ! $config{EvtCal}{activities}{list});
	    next if ($evt->{privacy} && $evt->{privacy} ne "public" && $config{EvtCal}{hide_private_events} && ! $db->level);
	    #next if ($done{$evt->{evt_id}});
	    my $e = new Modules::Event::Event(data=>$evt);
	    if ($evt->{date} ne $lastdate) {
		if ($dt && $dd) {
		    $eventSummaries .= $ml->dt($dt).$ml->dd($dd);
		    $dd = undef;
		}
		my $t = new ExSite::Time($evt->{date},"sql_date");
		$dt = $t->write("date_long");
		$lastdate = $evt->{date};
	    }
	    $dd .= $ml->p($ml->a($e->name,{href=>$this->link(evtcmd=>undef,evt=>$e->id)}));
	    $done{$evt->{evt_id}} = 1;
	    $nevt++;
	    last if ($limit && $nevt >= $limit);
	}
	$eventSummaries .= $ml->dt($dt).$ml->dd($dd);
	$out .= $ml->dl($eventSummaries,{class=>"EvtCalList"});
    }
    else {
    	$out .= $ml->p($msg{"No events were found for this $eventspan."});
    }
    return $out;
}

sub show {
    my $this = shift;
    my $out;
    if ($this->{evt} !~ /Event/) {
	# no event specified; show the next event
	my @event = $this->fetch_events($this->{Time}->write("sql_date"),1,$this->{section_id});
	if (scalar @event == 0) {
	    return $this->error($msg{"There are no events in the calendar."});
	}
	$this->{evt} = $this->fetch($event[0]);
	$out .= $ml->h1($msg{$config{EvtCal}{heading}{next}});
    }

    my $url = new ExSite::URI;
    $out = $this->{evt}->show(
			      url=>$url,
			      show_activities=>$config{EvtCal}{activities}{show}
			      );

    my $evtname = $this->{evt}->name;
    $out .= $ml->div($ml->a(&substitute($msg{"Add <i>[[event]]</i> to my calendar."},{event=>$this->{evt}->name}),{href=>$this->link(evtcmd=>"vcal")}),{class=>"vCalendarLink"});

    # back to calendar

    return $out;
}

sub vcalendar {
    my $this = shift;
    if ($this->{evt}) {
	my $p = $share{Page} || $share{ML};
	$p->cancel;
	print "Content-type: text/calendar\ncontent-disposition: attachment; filename=event.ics\n\n";
	#print "Content-type: text/calendar\n\n";
	print $this->{evt}->vcalendar();
	return;
    }
}

sub register {
    my $this = shift;
    my $out;
    if ($this->{evt} =~ /Event/) {
	my $feeid = $this->{input}{fee};
	if ($feeid =~ /^\d+$/) {
	    my $eid = $this->{evt}->id;
	    if ($share{Page}) {
		return "<!--&EvtReg(evt=$eid&fee=$feeid)-->";
	    }
	    else {
		# assume we're on an admin screen - switch to EvtReg module
		my $ml = &get_obj("ML");
		$ml->location("$config{server}{server}$config{server}{CGIpath}/$config{prog}{ctrlpanel}/EvtReg?evt=$eid&fee=$feeid");
		return;
	    }
	}
    }
    return $this->error("Invalid registration parameters.");
}

sub calendar {
    my ($this,%opt) = @_;
    # fetch events for the selected month
    my $date = $this->{Time}->write("year-month")."-01";
    my @events = $this->fetch_events_by_month($date,$this->{section_id});

    if (! defined $opt{linktype}) { 
	$opt{linktype} = "popup";
    }
    if (! defined $opt{format}) { 
	$opt{format} = "large";
    }
    my $out = $this->show_month(\%opt,,@events);

    return $ml->div($out,{class=>"EvtCal"});
}

sub admin {
    my $this = shift;
    return ($config{EvtCal}{new_event_access} =~ /public/) ||
	($config{EvtCal}{new_event_access} =~ /users|members/ && 
	 $share{DB}->authorize);
}

sub show_data {
    my ($this,$col) = @_;
    
    return undef if (! $this->{event});
    if ($col eq "upload" && $this->{event}{$col}) {
	my $img = new ExSite::Image($this->{event}{upload});
	my $filename = $img->name;
	my $pubpath = "$config{server}{HTMLpath}/_Modules/Calendar/$this->{event}{event_id}/$filename";
	if (-e "$config{server}{HTMLroot}$pubpath") {
	    #return "<a href=\"$pubpath\">$filename</a>";
	    return $ml->a($filename, { href => $pubpath });
	}
    }
    elsif ($col eq "photo" && $this->{event}{$col}) {
	my $img = new ExSite::Image($this->{event}{photo});
	my $filename = $img->name;
	my $pubpath = "$config{server}{HTMLpath}/_Modules/Calendar/$this->{event}{event_id}/$filename";
	if (-e "$config{server}{HTMLroot}$pubpath") {
	    #return "<p><img src=\"$pubpath\"></p>\n";
	    return $ml->p($ml->img(undef, { src => "$pubpath" }));
	}
    }
    return $share{DB}->show_data("evt",$col,$this->{event}{$col},
				 $this->{event}{event_id});
}

# show_month : display calendar month, with links to events

sub show_month {
    my ($this,$opt,@events) = @_;
    return $this->warn("No date selected.") if (! $this->{date});

    # get date 
    my ($year,$mon,$day) = @{$this->{date}};

    # get events
    if (@events == 0) {
	@events = $this->fetch_events_by_month($this->{date},$this->{section_id});
    }

    # flag the days with events
    my %day;
    foreach my $evt (@events) {
	next if ($evt->{status} && $evt->{status} ne "active" && ! $share{DB}->is_manager);
	next if ($opt->{type} && $evt->{type} ne $opt->{type});
	next if ($evt->{parent} && ! $config{EvtCal}{show_activities});
	next if ($evt->{privacy} && $evt->{privacy} ne "public" && 
		 $config{EvtCal}{hide_private_events} && ! $share{DB}->level);
	my ($sy,$sm,$sd) = split /-/, $evt->{date};
	my ($ey,$em,$ed) = (! $evt->{enddate} || $evt->{enddate} =~ /^0/) ?
	    ($sy,$sm,$sd) : 
	    split /-/, $evt->{enddate};
	
	next unless ($sy <= $year && 
		     $sm <= $mon && 
		     $ey >= $year && 
		     $em >= $mon) ||
		    ($sy < $year && $ey > $year);	

	my $beginday = ($sm == $mon ? $sd : 1);
	my $endday = ($em == $mon ? $ed : $dayinmonth[$mon-1]);
	if ( $ey > $year ) {
	    $endday = $dayinmonth[$mon-1];
	}
	
	foreach my $eventday ($beginday..$endday) {
	    if ($opt->{format} eq "large") {
		if ((ref $day{$eventday}) ne "ARRAY") {
		    $day{$eventday} = [];
		}
		push @{$day{$eventday}},new Modules::Event::Event(data=>$evt);
	    }
	    else {
		$day{$eventday} = $this->link(evt=>undef,evtcmd=>undef,date=>"$year-$mon-$eventday");
	    }
	}
    }

    # make a month selector for the calendar header
    my $calhdr;
    my $url = $this->link(evt=>undef,evtcmd=>undef,date=>undef);
    if ($opt->{format} eq "large") {
	$calhdr = $this->month_picker($year,$mon);
    }
    else {
	$calhdr = $this->month_selector($year,$mon);
    }

    return $this->show_calendar(year=>$year,month=>$mon,events=>\%day,header=>$calhdr,%$opt);
}

sub month_picker {
    my ($this,$year,$mon) = @_;
    my $dates;
    my ($before_month_menu,$after_month_menu);
    my $byear = $year - 1;
    my $bmon = $mon;
    if ($bmon < 1) {
	$bmon = 12;
	$byear--;
    }
    my ($prevmonth,$nextmonth);
    foreach my $m (1..12) {
	if ($bmon > 12) {
	    $bmon = 1;
	    $byear++;
	}
	my $dateanchor = $msg{$mon[$bmon-1]}." ".$byear;
	$prevmonth = "$byear-$bmon";
	$before_month_menu .= $ml->a($dateanchor,{href=>$this->link(date=>$prevmonth)}).$ml->br();
	$bmon++;
    }
    my $ayear = $year;
    my $amon = $mon + 1;
    foreach my $m (1..12) {
	if ($amon > 12) {
	    $amon = 1;
	    $ayear++;
	}
	my $dateanchor = $msg{$mon[$amon-1]}." ".$ayear;
	$after_month_menu .= $ml->a($dateanchor,{href=>$this->link(date=>"$ayear-$amon")}).$ml->br();
	$nextmonth = "$ayear-$amon" if (! $nextmonth);
	$amon++;
    }
    my $out = &ExSite::HTML::Popup(
				   pane=>$before_month_menu,
				   #label=>$ml->span("&#9664;",{class=>"EvtCalHeader"}),
				   label=>$ml->span("&laquo;",{class=>"EvtCalHeader"}),
				   #label=>$ml->span("&#8810;",{class=>"EvtCalHeader"}),
				   width=>80,
				   height=>170,
				   closelabel=>"&times;",
				   );
    $out .= $ml->a($ml->span(" &#8249; ",{class=>"EvtCalHeader"}),{href=>$this->link(date=>$prevmonth)});
    $out .= " ".$ml->span(
			  $msg{$month[$mon-1]}." ".$year,
			  {class=>"EvtCalHeader"}
			  )." ";
    $out .= $ml->a($ml->span(" &#8250; ",{class=>"EvtCalHeader"}),{href=>$this->link(date=>$nextmonth)});
    $out .= &ExSite::HTML::Popup(
				 pane=>$after_month_menu,
				 #label=>$ml->span("&#9654;",{class=>"EvtCalHeader"}),
				 label=>$ml->span("&raquo;",{class=>"EvtCalHeader"}),
				 #label=>$ml->span("&#8811;",{class=>"EvtCalHeader"}),
				 width=>80,
				 height=>170,
				 closelabel=>"&times;",
				 );
    return $out;
}

sub month_selector {
    my ($this,$year,$mon) = @_;
    my $monthOptions;
    my $date_span = $config{EvtCal}{date_selector_range} || 4;
    my $frommonth = $mon - $date_span;
    my $tomonth = $mon + $date_span;
    for (my $imon = $frommonth; $imon <= $tomonth; $imon++) {
	my $y = $year;
	my $dmon = $imon + 1;
	if ($dmon < 1) { $y--; $dmon += 12; }
	if ($dmon > 12) { $y++; $dmon -= 12; }
	my $tmpmon = $dmon - 1;
	my $monthopt = { value => $y . "-" . $dmon };
	if ($imon == $mon-1) { $monthopt->{selected} = undef; }
	$monthOptions .= $ml->option(
				     $msg{$mon[$tmpmon]} . " " . $y, 
				     $monthopt
				     );
    }
    my $formcontents = $ml->select($monthOptions, { name => "date", class => "Calendar", onchange => "document.calform.submit()" });
    $formcontents .= $config{page}{dyn_url_type} eq "id" ? 
	$ml->input(undef,{type=>"hidden",name=>"_id",value=>$this->{input}{_id}}) : "";
    if ($config{EvtCal}{view_switcher}) {
	$formcontents .= $ml->a($ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/EvtCal/expand.gif",border=>0,height=>12,width=>12,alt=>"full size",title=>"full size"}),{href=>$this->link(evtcmd=>"cal")});
    }
    return $ml->form(
		     $formcontents,
		     { method => "get", 
		       action => $this->link(evtcmd=>undef,date=>undef,evt=>undef), 
		       name => "calform" }
		     );
}

# show_calendar
# display a month view for a certain year/month
# events can be one of:
# hash of days => URLs -> calendar can shrink to a compact size
# hash of days => array of events -> calendar should be fullscreen
# header is the calendar titlebar

# options:
# year, month
# events = hash of day => event objects for that date
# header = HTML to place in calendar "title bar"
# linktype = normal, popup

sub show_calendar {
    my ($this,%opt) = @_;
    my $out;
    my $year = $opt{year};
    my $month = $opt{month};
    my $events = $opt{events};
    my $header = $opt{header};
    my $t = new ExSite::Time("$year-$month-01","iso_date");
    if (! $header) {
	my $lmonth = $t->write("month_long");
	$header = "$lmonth $year";
    }
    my $first_wday = $t->write("raw_weekday");
    
    # if the current month, note today for special highlighting
    my ($thisyear,$thismonth,$thisday) = $this->today();
    if ($thisyear != $year || $thismonth != $month) {
	$thisday = undef;
    }

    # are we looking at February in a leap year?
    my $leap = ((($year % 4) == 0) && (($year % 100) != 0)) ||
	(($year % 400) == 0);
    if ($leap) { $dayinmonth[1] += 1; }

    # select format
    my $longformat = $opt{format} eq "large";
 
    # build the calendar HTML

    my ($tableattr,$cellattr);
    if ($longformat) {
	$tableattr = { 
	    class => "EvtCal", 
	    width => "100%",
	};
	$cellattr = {}
    }
    else {
	$tableattr = { 
	    cellspacing => "1", 
	    cellpadding => "0", 
	    border => "0", 
	    class => "EvtCalSm", 
	    align => "right",
	};
	$cellattr = {};
    }
    #my $out = $ml->script(undef,{src=>"$config{server}{HTMLpath}/_ExSite/js/misc.js",type=>"text/javascript"});
    my $tdDays;
    my $currentWeeksTds;
    my $tdDay;
    my @today = $this->today;
    if ($first_wday < 0) { $first_wday += 7; }
    my $weekday = 0;
    foreach (0..$first_wday-1) {
	$currentWeeksTds .= $ml->td(undef,{class=>"NoDay"});
	$weekday++;
    }
    foreach my $d (1..$dayinmonth[$month-1]) {
	my %currentTdAttributes = %$cellattr;
	if ($weekday == 7) {
	    $tdDays .= $ml->tr($currentWeeksTds);
	    $weekday = 0;
	}
	my $day = $d < 10 ? "0$d" : $d;
	if ($weekday == 0) {
	    $currentWeeksTds = "";
	}
	if ($events->{$day}) {
	    $currentTdAttributes{class} = "EvtDay";
	}
	elsif ($d == $thisday) {
	    # today is yellow
	    $currentTdAttributes{class} = "Today";
	}
	if ($events->{$day}) {
	    my $url = $this->link(date=>"$year-$month-$day",evt=>undef);
	    if ((ref $events->{$day}) eq "ARRAY") {
		# expanded view
		$tdDay = $ml->div($d,{class=>"EvtDay"});
	        my @todaysEvents = @{$events->{$day}};
		foreach my $evt (@todaysEvents) {
		    my $class = "Evt";
		    if (! $evt->getdata("section_id")) {
			$class .= " SharedEvt";
		    }
		    elsif ($evt->getdata("section_id") != $this->{section_id}) {
			$class .= " OtherEvt";
		    }
		    if ($opt{linktype} eq "popup") {
			my $anchor = $config{EvtCal}{more_info} || "More info...";
			$tdDay .= $ml->div(&ExSite::HTML::Popup(label=>$evt->name,pane=>$evt->show_brief() . $ml->p($ml->a($msg{$anchor},{href=>$this->link(evt=>$evt->id,evtcmd=>undef)}),{class=>$class})),{class=>$class});
		    }
		    else {
			$tdDay .= $ml->div($ml->a($evt->name,{href=>$this->link(evt=>$evt->id)}),{class=>$class});
		    }
		}
	    }
	    else {
		# linked date only
		$tdDay = $ml->a($d, { href => $url });
	    }
	}
	else {
	    $tdDay = $d;
	}
	$currentWeeksTds .= $ml->td($tdDay, \%currentTdAttributes);

	$weekday++;
    }
    if ($weekday != 7) { 
	# fill out rest of week with empty days
	foreach my $d ($weekday+1 .. 7) { 
	    $currentWeeksTds .= $ml->td(undef,{class=>"NoDay"}); 
	}
    }
    $tdDays .= $ml->tr($currentWeeksTds);

    my $weekdays;
    foreach my $iwkdy (0..6) {
	$weekdays .= $longformat ?
	    $ml->th($msg{$weekday[$iwkdy]}) :
	    $ml->th($msg{$wd[$iwkdy]});
    }
    
    $out .= $ml->table(
		       $ml->tr(
			       $ml->th($header, { colspan => "7" })
			       ) .
    		       $ml->tr($weekdays) .
    		       $tdDays, 
    	    	       $tableattr
		       );
    return $out;
}

sub today {
     my $t = new ExSite::Time;
     
     my $today = $t->write("sql_date");
     
     if (wantarray) {
	return split("-", $today);
     }
     
     return $today;
}

# show date/time : display a datetime, but with more logic than the default
# method in Report::show_data

sub show_date {
    my ($this, $startdate, $enddate) = @_;
    return "" if (!$startdate);
    my $t1 = new ExSite::Time($startdate,"sql_date");
    if (! $enddate || $enddate =~ /^0/) {
	return $t1->write("date");
    }
    else {
	my $t2 = new ExSite::Time($enddate,"sql_date");
	if ($t2->write("sql_date") ne $t1->write("sql_date")) {
	    if ($t1->write("year") != $t2->write("year") || $t1->write("raw_month") != $t2->write("raw_month")) {
		return $t1->write("date") . " - " . $t2->write("date");
	    }
	    else {
		return $t1->write("month") . " " . $t1->write("raw_day") . " - " . $t2->write("raw_day") . ", " . $t1->write("year");
	    }
	}
	else {
	    return $t1->write("date");
	}
    }
}

sub show_time {
    my ($this,$starttime,$endtime,$startdate,$enddate) = @_;
    return undef if ($starttime =~ /^[\s0]0:00/);

    if ($startdate && $enddate) { 
	my $t1 = new ExSite::Time("$startdate $starttime", "sql_datetime");
	my $t2 = new ExSite::Time("$enddate $endtime", "sql_datetime");
	if ($t1->diff($t2) > 86400) { 
	    # don't show start/end times if separated by more than 1 day
	    return undef;
	}
    }
    
    my $t1 = new ExSite::Time($starttime, "time");
    my $t2 = new ExSite::Time($endtime, "time");

    my $out = $t1->write("time");
    if ($t2->write("time") ne $t1->write("time") && 
	$t2->write("time") != /^[\s0]0:00/) {
	$out .= " - " . $t2->write("time");
    }
    return $out;
}

sub show_evt_admin {
    my ($this,$evtid) = @_;
    my $out;
    if ($evtid) {
	$this->fetch($evtid);
	if ($this->{evt}) {
	    $out .= &ExSite::HTML::BasicBox(pane=>$this->show,title=>"Event Details");
	    if ($this->{evt}->owns()) {
		my $buttons = &ExSite::HTML::Button(url=>$this->link(evt=>$evtid,caladm=>"edit_evt"),label=>"Edit details");
		$buttons .= &ExSite::HTML::Button(url=>$this->link(evt=>$evtid,caladm=>"schedule"),label=>"Edit schedule");
		$buttons .= &ExSite::HTML::Button(url=>$this->link(evt=>$evtid,caladm=>"new",parent=>$evtid),label=>"Add Activity");
		$buttons .= &ExSite::HTML::Button(url=>$this->link(evt=>$evtid,caladm=>"venues",parent=>$evtid),label=>"Sites/Venues");
		my %module = map { $_ => 1 } &ExSite::Module::module_list();
		if ($module{EvtReg}) {
		    # registrations enabled
		    $buttons .= &ExSite::HTML::Button(url=>"javascript:popup_large('$config{server}{CGIpath}/$config{prog}{ctrlpanel}/EvtReg?evt=$evtid')",label=>"Registrations");
		}
		$buttons .= &ExSite::HTML::Button(url=>"javascript:confirm_custom('Really delete this event?','".$this->link(evt=>$evtid,caladm=>"delete")."')",label=>"DELETE");
		$out .= $ml->p($buttons);
	    }
	}
    }
    else {
	$out .= $this->error("No event selected.");
    }
    return $out;
}

sub edit_evt {
    my $this = shift;
    my $evt = new Modules::Event::Event(id=>$this->{input}{evt});
    my $in = new ExSite::Input();
    my $data = $in->post();
    my $out;
    if (keys %$data > 0) {
	###
	$out .= $evt->do_edit();
	$out .= $this->show_evt_admin($this->{input}{evt});
    }
    else {
	#$out .= $evt->edit(action=>$this->link(),hide=>["organizer","status"]);
	$out .= $evt->edit(action=>$this->link());
    }
}

sub show_schedule_admin {
    my ($this) = @_;
    my $out;
    if ($this->{evt}->loaded()) {
	my $times = $this->{evt}{children}{evt_date};
	my $r = new ExSite::ReportBuilder(
					  title=>"Schedule",
					  headers=>["Date","Description","Options"]
					  );
	my $startdate;
	$times->sort("date","time");
	my $gotstart;
	while (my $date = $times->next()) {
	    my $options = $this->admbutton("edit",$this->link(caladm=>"edit_schedule",date=>$date->getdata("evt_date_id")),"edit");
	    $options .= " ";
	    $options .= $this->admbutton("del","javascript:confirm_custom('Really delete this date from the schedule?','".$this->link(caladm=>"del_schedule",date=>$date->getdata("evt_date_id"))."')","delete");
	    $r->push($date->show, $date->showdata("description"),$options);
	}
	$r->tools($this->admbutton("add",$this->link(caladm=>"new_schedule")),"add date to schedule");
	$r->nodata("This event does not have a start date.".$ml->br."It may not appear correctly in the calendar.");
	$out .= $r->make;
    }
    return $out;
}

sub edit_schedule {
    my $this = shift;
    my $db = $share{DB};
    my $in = new ExSite::Input;
    my $data = $in->post();
    my $out;
    if ($this->{input}{date}) {
	my $ed = new Modules::Event::Date(id=>$this->{input}{date});
	if ($ed->loaded()) {
	    if (scalar keys %$data > 0) {
		$db->set_action("update");
		$out .= $ed->do_edit();
		if ($db->errorcheck()) {
		    $out .= $db->show_diagnostics("error","html");
		}
		else {
		    $ml->location($this->link(caladm=>"schedule",date=>undef,__plaintext=>1));
		}
	    }
	    else {
		$out = $ml->h1("Edit Date");
		$out .= $ed->edit(
				  action=>$this->link(),
				  );
		$out .= $ml->p("Note that times of 12:00 AM are ignored.  If you want to show the date, but not the time, set the time to 12:00 AM.");
	    }
	}
    }
    return $out;
}

sub new_schedule {
    my $this = shift;
    my $db = $share{DB};
    my $in = new ExSite::Input;
    my $data = $in->post();
    my $out;
    my $ed = new Modules::Event::Date();
    if (scalar keys %$data > 0) {
	return $this->add_dates($data);
    }
    if ($this->{evt}) {
	# default the date fields to the dates of the last entered date
	my $dates = $this->{evt}->get_dates();
	my $date = $dates ? $this->{evt}->get_dates()->last : undef;
	my ($d,$t,$d2,$t2);
	if ($date) {
	    $d = $date->getdata("date");
	    $t = $date->getdata("time");
	    $d2 = $date->getdata("enddate");
	    $t2 = $date->getdata("endtime");
	}

	# add single date

	$out = $ml->h1("New Date");
	$out .= $ml->p("Note that times of 12:00 AM are ignored.  If you want to show the date, but not the time, set the time to 12:00 AM.");

	my $form = $ed->input_record({
	    data=>{
		evt_id=>$this->{input}{evt},
		date=>$d,
		time=>$t,
		enddate=>$d2,
		endtime=>$t2,
	    },
	    hide=>["evt_id"]
	    });

	$form .= &ExSite::HTML::FrameBox(
					 bgcolor=>"#eee",
					 color=>"#999",
					 title=>"Recurring events only",
					 pane=>$ml->p("Repeat every ".
						      $ml->input(undef,{type=>"text",name=>"repeat_interval",value=>"0",size=>4}).
						      " ".
						      $ml->select(
								  $ml->option("days").
								  $ml->option("weeks"),
								  {name=>"repeat_unit"}
								  ).
						      " until ".
						      $ml->input(undef,{type=>"text",name=>"repeat_until",placeholder=>"YYYY-MM-DD",size=>10}).
						      "(YYYY-MM-DD)")
					 );

	$form .= $ml->input(undef,{type=>"submit", value=>"Submit"})." ";
	$form .= $ml->input(undef,{type=>"button", value=>"Cancel", onClick=>"javscript:history.back()"})." ";
	$form .= $ml->input(undef,{type=>"reset", value=>"Reset"});
	$out .= $ml->form($form,
			  {action=>$this->link(),method=>"post"});

    }
    return $out;
}

sub add_dates {
    my ($this,$data) = @_;

    my $db = $share{DB};
    my $ed = new Modules::Event::Date();

    my $rpt_interval = $data->{repeat_interval};
    my $rpt_unit = $data->{repeat_unit};
    my $rpt_until = $data->{repeat_until};

    $db->set_action("insert");
    my $out .= $ed->do_make();
    if ($db->errorcheck()) {
	$out .= $db->show_diagnostics("error","html");
	return $out;
    }
    # insert succeeded, populate the object
    $ed->setup(data=>$share{DB}{form}{evt_date}{_});

    # first date went in okay, try the repeat dates

    if ($rpt_interval && $rpt_unit && $rpt_until) {
	my $st = $ed->get_start();
	my $end = $ed->get_end();
	my $use_end = $st->diff($end) > 0;
	my $descr = $ed->getdata("description");
	my $until = new ExSite::Time($rpt_until,"iso_date"); # wrong time
	my $err = $until->validate();
	if ($err) {
	    $out .= $this->error("Until date: ".$err);
	}
	else {
	    if ($rpt_interval =~ /^\d+$/ && 
		$rpt_interval > 0 && 
		$rpt_unit =~ /day|week/) {
		$st->add_approximate($rpt_interval,$rpt_unit);
		while ($st->write("iso_date") lt $until->write("iso_date")) {
		    $ed = new Modules::Event::Date();
		    $ed->setdata("evt_id",$this->{evt}->id);
		    $ed->setdata("date",$st->write("sql_date"));
		    $ed->setdata("time",$st->write("time24"));
		    if ($use_end) {
			$end->add_approximate($rpt_interval,$rpt_unit);
			$ed->setdata("enddate",$end->write("sql_date"));
			$ed->setdata("endtime",$end->write("time24"));
		    }
		    $ed->setdata("description",$descr) if $descr;
		    $ed->save or 
			$out .= $this->error("Failed to save ".$st->write("date"));
		    $st->add_approximate($rpt_interval,$rpt_unit);
		}
	    }
	}
    }

    if (! $db->errorcheck()) {
	$ml->location($this->link(caladm=>"schedule",date=>undef,__plaintext=>1));
    }
    return $out;
}

sub del_schedule {
    my $this = shift;
    my $ed = new Modules::Event::Date(id=>$this->{input}{date});
    if ($ed->exists() && $ed->owns()) {
	$ed->delete();
    }
    $ml->location($this->link(caladm=>"schedule",date=>undef,__plaintext=>1));
}

# deprecated
sub new_evt_date {
    my $this = shift;
    my $date = new Modules::Event::Date();
    my $in = new ExSite::Input;
    my $data = $in->post();
    my $out = $ml->h1($msg{"Add new date/time to schedule"});
    if (keys %$data > 0) {
	$out .= $this->doform();
	$out .= $this->edit_schedule();
    }
    elsif ($this->{evt}) {
	my $date = $this->{evt}->get_date();
	if ($date) {
	    my $d = $date->getdata("date");
	    my $t = $date->getdata("time");
	    $out .= $date->make(
				action=>$this->link(),
				show=>["type","date","time"],
				data=>{
				    evt_id=>$this->{input}{evt},
				    date=>$d,
				    time=>$t,
				},
				extra=>{action=>"insert"},
				);
	}
    }
}

sub contacts {
    my $this = shift;
    my $ename = $this->fetch()->name();
    my $out = $ml->h1("Site and Contact Information for $ename");
    my $db = $share{DB};

    my @contact = $db->get_query("event contacts",$this->{input}{evt});
    
    my $r = new ExSite::ReportBuilder(title=>"Sites / Contacts");
    $r->nodata("No site or contact information has been defined for this event.");
    $r->headers(["Location","City","Phone","Email"]);
    foreach my $c (@contact) {
	my $options = $ml->a("view",{href=>$this->link(caladm=>"view_contact",contact=>$c->{location_id})});
	$r->push(
		 $ml->a($c->{name},{href=>$this->link(caladm=>"view_contact",contact=>$c->{location_id})}),
		 $c->{city},
		 $c->{phone1},
		 $c->{email});
    }
    $r->tools($ml->a("+ add new site / contact",{href=>$this->link(caladm=>"new_contact")}));
    $out .= $r->make();
    return $out;
}

sub new_contact {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    $db->set_action("insert");
    my $in = new ExSite::Input;
    my $data = $in->post();
    if (scalar keys %$data > 0) {
	$out .= $db->do();
    }
    my $eid = $this->{input}{evt};
    my $evt = new Modules::Event::Event(id=>$eid);
    if ($evt) {
	$out .= $ml->h2("Site / Venue / Facility for ".$evt->name);
	$out .= $db->input_record({table=>"location",data=>{evt_id=>$eid},show=>["name","image","brochure"]});
	$out .= $db->input_record({table=>"contact",data=>{location_id=>"_1"},hide=>["account_id","location_id"]});
	$out .= $ml->input(undef,{type=>"submit",value=>"Submit"});
	$out .= $ml->input(undef,{type=>"reset"});
	$out .= $ml->input(undef,{type=>"button",value=>"Cancel",onClick=>"javascript:history.back()"});
	$out = $ml->form($out,{action=>$this->link(),enctype=>"multipart/form-data",method=>"post"});
    }
    else {
	$out .= $ml->p("Event $this->{input}{evt} not found.");
    }
    return $out;
}

sub edit_contacts {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    $db->set_action("update");
    my $in = new ExSite::Input;
    my $data = $in->post();
    if (scalar keys %$data > 0) {
	$out .= $db->do();
    }
    my $evt = $db->fetch("evt",$this->{input}{evt});
    if ($evt) {
	my @eloc = $db->fetch_match("evt_location",{evt_id=>$this->{input}{evt}});
	foreach my $eloc (@eloc) {
	    $out .= $ml->h2("Location");
	    my $loc = $db->fetch("location",$eloc->{location_id});
	    $out .= $db->input_record({table=>"location",data=>$loc,show=>["name","image","brochure"]});
	    my @cont = $db->fetch_match("contact",{location_id=>$loc->{location_id}});
	    foreach my $cont (@cont) {
		$out .= $ml->h3(ucfirst $cont->{type}." contact");
		$out .= $db->input_record({table=>"contact",data=>$cont,hide=>["account_id","location_id"]});
	    }
	}
	if ($out) {
	    $out .= $ml->input(undef,{type=>"submit",value=>"Submit"});
	    $out .= $ml->input(undef,{type=>"reset"});
	    $out .= $ml->input(undef,{type=>"button",value=>"Cancel",onClick=>"javascript:history.back()"});
	    $out = $ml->form($out,{action=>$this->link(),enctype=>"multipart/form-data",method=>"post"});
	}
	else {
	    $out .= $ml->p("No location or contact information found.");
	}
    }
    else {
	$out .= $ml->p("Event $this->{input}{evt} not found.");
    }
    return $out;
}

sub venues {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    my $eid = $this->{input}{evt};
    my $sid = $this->get_section_id();
    my $evt = new Modules::Event::Event(id=>$eid);
    if ($this->{input}{venue}) {
	my $in = new ExSite::Input;
	my $post = $in->post;
	if ($post->{venue}) {
	    $db->insert("evt_location",{evt_id=>$eid,location_id=>$post->{venue},status=>"active"}) or 
		$out .= $db->show_diagnostics("error","html");
	}
	elsif ($this->{input}{caladm} eq "delvenue") {
	    $db->trash("evt_location",$this->{input}{venue});
	}
    }
    if ($evt->exists) {
	my @evtloc = $db->get_query("event contacts",$eid);
	my $r = new ExSite::ReportBuilder(title=>"Sites/Venues/Facilities for ".$evt->name);
	$r->headers("Venue","Contact Info","Options");
	$r->nodata("No site or contact information has been defined for this event.");
	foreach my $loc (@evtloc) {
	    my $c = new Modules::ID::Contact(data=>$loc);
	    my $options = $ml->a("delete",{href=>$this->link(venue=>$loc->{evt_location_id},caladm=>"delvenue")});
	    $r->push($loc->{name},$c->show,$options);
	}
	$out .= $r->make;

	$out .= $ml->h2("Add a site/venue/facility to this event");
	my @loc = $db->fetch_child("location","section",$sid);
	if (scalar @loc > 0) {
	    my @opt = map { [ $_->{location_id}, $_->{name} ] } @loc;
	    my $f = new ExSite::FormBuilder();
	    $f->input(prompt=>"Venue",
		      name=>"venue",
		      type=>"select",
		      options=>\@opt,
		      required=>1,
		      nullvalue=>"== select venue ==");
	    $out .= $f->make();
	}
	else {
	    $out .= $ml->p("There are no venues available to choose from.");
	}
	$out .= $ml->p("If the venue does not appear in this list, you can add new locations and venues in your ".
		       $ml->a("address book",{href=>"javascript:popup_large('$config{server}{CGIpath}/$config{prog}{ctrlpanel}/AddrBook?section_id=$sid&amp;tab=location')"}).
		       ". Then you can refresh this screen to see the new venues.");
    }
    else {
	$out .= $ml->p("Event $this->{input}{evt} not found.");
    }
    return $out;
}

# newevent_simple - simplified event creation for non-admins

sub newevent_simple {
    my $this = shift;
    my $out;
    my $in = $this->{input};
    return $this->do_newevent_simple() if ($in->{name});
    my $action = $this->link(caladm=>"doform");
    my $date = new ExSite::Time();
    my $year = $date->write("raw_year");
    my $month = $date->write("raw_month");
    my $day = $date->write("raw_day");
    my $months = [
		  [1, "Jan"],
		  [2, "Feb"],
		  [3, "Mar"],
		  [4, "Apr"],
		  [5, "May"],
		  [6, "Jun"],
		  [7, "Jul"],
		  [8, "Aug"],
		  [9, "Sep"],
		  [10, "Oct"],
		  [11, "Nov"],
		  [12, "Dec"],
		  ];
    my $types = $share{DB}{map}->regexp("list:evt_type");

    my $f = new ExSite::FormBuilder(action=>$this->link(evtcmd=>"new"));
    $f->input(type=>"text",name=>"name",size=>60,prompt=>"Event",required=>1);
    $f->input(type=>"select",name=>"type",prompt=>"Type",options=>$types,nullvalue=>$msg{"== select =="});
    $f->input(type=>"textarea",name=>"description",prompt=>"Details/Description");
    $f->input(type=>"select",name=>"year",options=>[$year .. $year + 2],required=>1);
    $f->input(type=>"select",name=>"month",options=>$months,value=>$month,required=>1);
    $f->input(type=>"select",name=>"day",options=>[1..31],value=>$day,required=>1);
    $f->template(
		 $ml->p("[[name:prompt]]".$ml->br."[[name:input]]").
		 $ml->p("[[type:prompt]]".$ml->br."[[type:input]]").
		 $ml->p("[[description:prompt]]".$ml->br."[[description:input]]").
		 $ml->p("Date".$ml->br."[[year:input]][[month:input]][[day:input]]")
		 );
    
    $out .= $ml->h1($msg{$config{EvtCal}{heading}{new}});
    $out .= $f->make();

    return $out;
}

sub do_newevent_simple {
    my $this = shift;
    my $out;
    my $in = $this->{input};
    if (! ($in->{name} && $in->{year} && $in->{month} && $in->{day})) {
	return $this->error("You must provide an event name and date.");
    }
    my $db = $share{DB};
    my $sid = $this->get_section_id();
    if (! $sid) { 
	return $this->error("Unable to determine website section.");
    }
    my $date = new ExSite::Time("$in->{year}-$in->{month}-$in->{day}","sql_date");
    my $err = $date->validate();
    if ($err) {
	return $this->error("New event: malformed date: $err");
    }

    my $description = $db->clean_text($in->{description});
    if ($description !~ /<(!--|[\/a-zA-Z])/) {
	my $br = $ml->br;
	$description =~ s/\n/$br/g;
    }

    my $type = $in->{type};
    my $typere = $share{DB}{map}->regexp("list:evt_type");
    if ($type !~ /^$typere$/) {
	return $this->error("New event: invalid event type");
    }

    my %evt = ( 
		section_id => $sid,
		name => &safehtml($in->{name}),
		description => $description,
		type => $type,
		status => "active",
		privacy => "public"
		);

    my $e = new Modules::Event::Event(data=>\%evt);
    my $eid = $e->force_save();
    if ($eid) {
	my %date = (
		    evt_id=>$eid,
		    date=>$date->write("sql_date"),
		    #type=>"start",
		    );
	my $d = new Modules::Event::Date(data=>\%date);
	my $edid = $d->force_save();
	if (! $edid) {
	    return $this->error("Failed to save event date.");
	}
	my $out = $ml->h1("$evt{name} added to calendar");
	$out .= $ml->p($ml->a("View my event.",{href=>$this->link(evtcmd=>"show",evt=>$eid)}));
	$out .= $ml->p($ml->a("Back to calendar.",{href=>$this->link(evtcmd=>undef,evt=>undef)}));
	
	# notification
	my $email = $db->owner_email($db->this_site());
	if ($email) {
	    my $uname = $db->level ? $db->my_name() : "unknown";
	    my $message = $e->show();
	    $message .= $ml->hr;
	    $message .= $ml->p("Posted by $uname.");
	    $message .= $ml->p($ml->a("View event",{href=>$this->link(evtcmd=>"show",evt=>$eid)}));
	    &ExSite::Mail::send(
				to=>$email,
				from=>($db->my_email || $email),
				subject=>"New Event Added: ".$e->name,
				body=>$message
				);
	}

	return $out;
    }
    else {
	return $this->error("Failed to save event.");
    }
}

sub newevent {
    my $this = shift;
    my $in = $this->{input};
    my $db = $share{DB};
    my $sid = $in->{section_id};
    if (! $sid) { 
	$sid = $share{Page} ? $share{Page}->id("section") : 0;
    }
    my $action = $this->link(caladm=>"doform");
    
    my $out;

    if ($in->{parent}) {
	# activity
	$out .= $ml->h1($msg{$config{EvtCal}{heading}{newactivity}});
	$out .= $ml->p("An activity is a sub-event that happens as part of, or in connection with, the main event. Otherwise, it is just like another event. The activities and main event will be joined together in calendar views, and registration in the activity will depend on registration in the main event.");
    }
    else {
	$out .= $ml->h1($msg{$config{EvtCal}{heading}{new}});
    }

    $db->form(method=>"post",action=>$this->link(caladm=>"doform"));
    $db->{action} = "insert";
    $db->input_record({table=>"evt",record=>"_1",data=>{section_id=>$sid,status=>"active",parent=>$in->{parent}},hide=>["section_id"]});
    $db->form()->input(
		       type=>"preformatted",
		       prompt=>$ml->h3("Date &amp Time"),
		       input=>$ml->p("The time is optional&mdash;leave it at 12:00 AM to ignore it.".$ml->br."The schedule can be expanded with additional dates and times after the event is created."),
		       name=>"date_heading"
		       );
    my $startdate = $in->{date} =~ /^\d\d\d\d-\d\d?$/ ? "$in->{date}-01" : undef;
    $db->input_record({table=>"evt_date",record=>"_2",data=>{type=>"start",evt_id=>"_1",date=>$startdate},hide=>["description","type"]});
    $db->form()->input(type=>"hidden",
		       name=>"action",
		       value=>$db->{action});
    $out .= $db->form()->make();
    return $out;
}

sub doform {
    my $this = shift;
    my $in = $this->{input};
    my $db = $share{DB};
    
    my $out;
    $db->get_data;
    my $site = $this->this_site()->{section_id};
    my $evtid;
    if (exists $db->{form}{evt}) {
	# tweak the form data
	my @evtid = keys %{$db->{form}{evt}};
	$evtid = $evtid[0] || "_";
	$db->{action} = (exists $db->{form}{evt}{$evtid}{event_id} ? 
			 "update" : "insert");

	if (! $db->{form}{evt}{$evtid}{status}) {
	    # default the event status to active
	    $db->{form}{evt}{$evtid}{status} = "active";
	}
	if (! $db->{form}{evt}{$evtid}{privacy}) {
	    # default the event access to public
	    $db->{form}{evt}{$evtid}{privacy} = "public";
	}
	# tie event to current site
	if ($db->{form}{evt}{$evtid}{section_id}) {
	    # ownership check
	    if (! $db->owns($db->{form}{evt}{$evtid}{section_id})) {
		return $this->error($msg{"You do not have permission to add events to this website."});
	    }
	}
	else {
	    if ($site) {
		$db->{form}{evt}{$evtid}{section_id} = $site;
	    }
	    elsif ($db->is_admin) {
		$db->{form}{evt}{$evtid}{section_id} = 0;
	    }
	    else {
		return $this->error($msg{"You do not have permission to add events to the general calendar."});
	    }
	}

	# tie event to member
	if ($db->{form}{evt}{$evtid}{member_id}) {
	    if (! $db->is_admin &&
		$db->{form}{evt}{$evtid}{member_id} != $db->my_uid) {
		$db->{form}{evt}{$evtid}{member_id} = $db->my_uid;
	    }
	}
	elsif (! $this->admin) {
	    $db->{form}{evt}{$evtid}{member_id} = $db->my_uid;
	}

	# sanitize inputs if untrusted user
	if ($db->level < 2) {
	    $db->{form}{evt}{$evtid}{name} = $db->clean_text($db->{form}{evt}{$evtid}{name},0);
	    $db->{form}{evt}{$evtid}{description} = $db->clean_text($db->{form}{evt}{$evtid}{description},1);
	}
    }
    
    # process the form
    $out = $db->do;

    # notification
    if ($config{EvtCal}{notification} && $db->{action} eq "insert") {
	my $evt = $db->{form}{evt}{$evtid};
	$evt->{date} = $db->{form}{evt_date}{_2}{date};
	$evt->{time} = $db->{form}{evt_date}{_2}{time};
	$evt->{from} = $db->my_name;
	my $message = &substitute($notification,$evt);
	my $calendar_mgr;
	if ($site) {
	    my $s = $db->fetch("section",$site);
	    $calendar_mgr = $db->owner_email($s);
	}
	else {
	    $calendar_mgr = $config{site}{administrator};
	}
	&ExSite::Mail::send(
			    to=>$calendar_mgr,
			    from=>$calendar_mgr,
			    subject=>"$config{site}{name} - Calendar Update: $evt->{name}",
			    body=>$message
			    );
    }

    if ($out =~ /Done\!/) {
	my $eid = $evtid =~ /^\d+$/ ? $evtid : undef;
	$ml->location($this->link(caladm=>undef,evt=>$eid,__plaintext=>1));
    }
    return $out;
}

sub report {
    my ($this,$section_id) = @_;
    my $out;
    my $db = $share{DB};
    my $r = new ExSite::ReportBuilder();
    my @e = $db->get_query("all events",$section_id);

    # combine recurring events
    my %e;
    foreach my $e (@e) {
	if (exists $e{$e->{evt_id}}) {
	    my $enddate = $e->{enddate};
	    if (! $enddate || $enddate =~ /^0/) {
		$enddate = $e->{date};
	    }
	    my $endtime = $e->{endtime};
	    if (! $endtime || $endtime =~ /^00:00:00$/) {
		$endtime = $e->{time};
	    }
	    $e{$e->{evt_id}}{enddate} = $enddate;
	    $e{$e->{evt_id}}{endtime} = $endtime;
	}
	else {
	    $e{$e->{evt_id}} = $e;
	}
    }

    my %done;
    
    # shared events
    if (! $section_id && $db->is_admin) {
	$r->init(title=>"Shared Events",
		 headers=>[ "Date","Event","Type","Status" ],
		 nodata=>"No events found.",
		 );

	    
	if (@e > 0) {
	    foreach my $e (reverse @e) {
		next if ($e->{section_id});
		next if ($done{$e->{evt_id}});
		$r->push($this->report_row($e{$e->{evt_id}}));
		$done{$e->{evt_id}} = 1;
	    }
	    my $new = $this->link(caladm=>"new",section_id=>0);
	    $r->tools($this->admbutton("add",$new,"new event"));
	    $out .= $r->make;
	    $out .= $ml->p();
	}
    }

    # site-specific events
    my $site = $db->fetch("section",$section_id);
    $r->init(
	     title=>$site->{title},
	     headers=>[ "Date","Event","Type","Status" ],
	     nodata=>"No events found.",
	     );

    if ($section_id && $db->owns($section_id)) {
	if (@e > 0) {
	    foreach my $e (reverse @e) {
		next if (! $e->{section_id});
		next if ($done{$e->{evt_id}});
		$r->push($this->report_row($e{$e->{evt_id}}));
		$done{$e->{evt_id}} = 1;
	    }
	}
	my $new = $this->link(caladm=>"new",section_id=>$this->{input}{section_id});
	my $imageAttributes = {
		src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{new}{img},
		border => "0",
		alt => "new event",
		title => "new event",
	};
	$r->tools($this->admbutton("add",$new,"new event"));
	$out .= $r->make;
	$out .= $ml->p();
    }
    return $out;
}

sub report_row {
    my ($this,$e) = @_;
    my @data;
    
    $this->{event} = $e;
    my $view = $this->link(evt=>$e->{evt_id},caladm=>undef,evtcmd=>undef,date=>undef);
    push @data, $this->show_date($e->{date},$e->{enddate});
    push @data, (
		 $ml->a($this->show_data("name"),{href=>$view}),
		 $this->show_data("type"),
		 $this->show_data("status")
		 );
    return @data;
}

### FETCHING EVENTS

sub setup_querylib {
    my ($this) = @_;
    my $db = $share{DB};
    $db->set_query("events after date",
		   ( 
		     sql => "select e.*,ed.date date,ed.time time from evt e,evt_date ed where ed.evt_id=e.evt_id and ed.date >= ? and (e.section_id=? or e.section_id=0) order by ed.date,ed.time desc",
		     nparam => 2,
		     mode => "r",
		     keys => ["evt","evt_date"],
		     )
		   );
    $db->set_query("events of type after date",
		   ( 
		     sql => "select e.*,ed.date date,ed.time time from evt e,evt_date ed where e.type=? and ed.evt_id=e.evt_id and ed.date >= ? and (e.section_id=? or e.section_id=0) order by ed.date,ed.time desc",
		     nparam => 3,
		     mode => "r",
		     keys => ["evt","evt_date"],
		     )
		   );
    $db->set_query("events before date",
		   ( 
		     sql => "select e.*,ed.date,ed.time from evt e,evt_date ed where ed.evt_id=e.evt_id and ed.date < ? and (e.section_id=? or e.section_id=0) order by ed.date,ed.time desc limit ?",
		     nparam => 3,
		     mode => "r",
		     keys => ["evt","evt_date"],
		     )
		   );
    $db->set_query("all events in section",
		   ( 
		     sql => "select e.*,ed.date,ed.time from evt e,evt_date ed where ed.evt_id=e.evt_id and e.section_id=? order by ed.date,ed.time desc",
		     nparam => 1,
		     mode => "r",
		     keys => ["evt","evt_date"],
		     )
		   );
    $db->set_query("all parent events in section",
		   ( 
		     sql => "select * from evt where section_id=? and parent=0 order by evt_id desc",
		     nparam => 1,
		     mode => "r",
		     keys => ["evt","evt_date"],
		     )
		   );
#    $db->set_query("activities",
#		   ( 
#		     sql => "select e.*,ed.date startdate,ed.time starttime from evt e,evt_date ed where ed.type=\"start\" and ed.evt_id=e.evt_id and e.parent=? order by ed.date,ed.time desc",
#		     nparam => 1,
#		     mode => "r",
#		     keys => ["evt","evt_date"],
#		     )
#		   );
#    $db->set_query("my events",
#		   ( 
#		     sql => "select * from evt where start >= ? and status = \"active\" and (section_id=? or section_id=0) and member_id=? order by start limit ?",
#		     nparam => 4,
#		     mode => "r",
#		     keys => "evt",
#		     )
#		   );
    $db->set_query("events by date",
		   ( 
		     sql => "select e.*, ed.* from evt e, evt_date ed where ed.evt_id=e.evt_id and ed.date >= ? and ed.date < ? and (e.section_id=? or e.section_id=0) order by ed.date,ed.time desc",
		     #"select e.*,ed1.date startdate,ed1.time starttime,ed2.date enddate,ed2.time endtime from evt e left join evt_date ed2 on ed2.evt_id=e.evt_id and ed2.type=\"end\",evt_date ed1 where ed1.type=\"start\" and ed1.evt_id=e.evt_id and ed1.date >= ? and ed1.date < ? and (e.section_id=? or e.section_id=0) order by ed1.date,ed1.time desc",
		     nparam => 3,
		     mode => "r",
		     keys => ["evt","evt_date"],
		     )
		   );
    $db->set_query("all events",
		   ( 
		     sql => "select e.*, ed.* from evt e, evt_date ed where ed.evt_id=e.evt_id and (e.section_id=? or e.section_id=0) order by ed.date,ed.time desc",
		     #"select e.*,ed1.date startdate,ed1.time starttime,ed2.date enddate,ed2.time endtime from evt e left join evt_date ed2 on ed2.evt_id=e.evt_id and ed2.type=\"end\",evt_date ed1 where ed1.type=\"start\" and ed1.evt_id=e.evt_id and (e.section_id=? or e.section_id=0) order by ed1.date,ed1.time desc",
		     nparam => 1,
		     mode => "r",
		     keys => ["evt","evt_date"],
		     )
		   );
    $db->set_query("event contacts",
		   ( 
		     sql => "select el.evt_location_id,l.location_id,l.name,c.* from evt_location el, location l, contact c where el.evt_id = ? and el.location_id=l.location_id and c.location_id=l.location_id",
		     nparam => 1,
		     mode => "r",
		     keys => ["evt_location","location","contact"],
		     )
		   );

    # add our DB handlers here as well

    $db->handler("select_foreign_key",\&select_parent_events);
}

sub fetch {
    my ($this,$evt) = @_;
    my $db = $share{DB};
    my $e;
    if (ref $evt) {
	$e = new Modules::Event::Event(data=>$evt);
    }
    elsif ($evt =~ /^\d+$/) {
	$e = new Modules::Event::Event(id=>$evt);
    }
    elsif ($this->{evt}) {
	# no change
	return $this->{evt};
    }
    else {
	$this->error("Undefined event");
	return undef;
    }
    $this->{evt} = $e;
    # new event - validate that event belongs to this site
    if (! $this->allow_in_section($this->{section_id})) {
	delete $this->{evt};
	return undef;
    }
    return $this->{evt};
}

# fetch events after a certain date

sub fetch_events {
    my ($this,$date,$section_id,$type) = @_;
    my $db = $share{DB};
    if ($date !~ /^\d{4}-\d{1,2}-\d{1,2}$/) { $date = $this->today; }
    $section_id or $section_id = $this->this_site()->{section_id};
    my @events = $type ?
	$db->get_query("events of type after date",$type,$date,$section_id) :
	$db->get_query("events after date",$date,$section_id);
    return @events;
}

# fetch a number of events before a certain date

sub fetch_past_events {
    my ($this,$date,$limit,$section_id) = @_;
    my $db = $share{DB};
    if ($date !~ /^\d{4}-\d{1,2}-\d{1,2}$/) { $date = $this->today; }
    if (! $limit) { $limit = $config{EvtCal}{max_list_size} || 10; }
    $section_id or $section_id = $this->this_site()->{section_id};
    my @events = $db->get_query("events before date",$date,$section_id,$limit);
    return @events;
}

# same as fetch_events, but fetches member-owned events only

#sub fetch_my_events {
#    my ($this,$date,$limit,$section_id) = @_;
#    my $db = $share{DB};
#    if ($date !~ /^\d{4}-\d{1,2}-\d{1,2}$/) { $date = $this->today; }
#    if (! $limit) { $limit = $config{EvtCal}{max_list_size} || 10; }
#    $section_id or $section_id = $this->this_site()->{section_id};
#    my $uid = $db->my_uid;
#    my @events = $db->get_query("my events",,$date,$section_id,$uid,$limit);
#    return @events;
#}

# fetch_events_by_date : year, month required; day and limit are optional

sub fetch_events_by_date {
    my ($this,$date,$section_id) = @_;
    my $db = $share{DB};
    if ($date !~ /^\d{4}-\d{1,2}-\d{1,2}$/) { $date = $this->today; }
    my $t = new ExSite::Time($date,"sql_date");
    $t->add(1,"day");
    my $enddate = $t->write("sql_date");
    $section_id or $section_id = $this->this_site()->{section_id};
    return $db->get_query("events by date",$date,$enddate,$section_id);
}

sub fetch_events_by_month {
    my ($this,$date,$section_id) = @_;
    my $db = $share{DB};
    if ($date !~ /^\d{4}-\d{1,2}(-\d{1,2})?$/) { $date = $this->today; }
    my ($y,$m,undef,$d) = ($date =~ /^(\d{4})-(\d{1,2})(-(\d{1,2}))?$/);
    my $date = "$y-$m-01";
    $m++;
    if ($m > 12) {
	$m = 1;
	$y++;
    }
    my $enddate = "$y-$m-01";
    $section_id or $section_id = $this->this_site()->{section_id};
    return $db->get_query("events by date",$date,$enddate,$section_id);
}

### ACCESS CONTROL

# is a section allowed to display an event?

sub allow_in_section {
    my ($this,$section_id) = @_;
    if ($this->{evt}) {
	my $evt_section_id = $this->{evt}->getdata("section_id");
	return 1 if (! $evt_section_id); # global event

	$section_id or $section_id = $this->{section_id} || 
	    $this->this_site()->{section_id};

	if ($evt_section_id == $section_id) {
	    # allow local events
	    return 1;
	}
	elsif ($this->{share_section} == $evt_section_id) {
	    # allow if we are explicitly sharing this section's calendar
	    return 1;
	}
    }
    return 0;
}

# is the user allowed to view this event?

sub allow_view {
    my ($this,$section_id) = @_;
    my $evt = $this->{evt};
    if ($evt && $evt->loaded) {
	my $privacy = $evt->getdata("privacy");
	if ($privacy =~ /members/) {
	    $section_id or 
		$section_id = $this->this_site()->{section_id};
	    if (! $share{DB}->role($section_id)) {
		return 0;
	    }
	}
	return $this->allow_in_section($section_id);
    }
    return undef;
}

# is the user allowed to manage this event?

sub allow_admin {
    my ($this,$section_id) = @_;
    if ($this->{evt} && $this->{evt}->loaded) {
	$section_id or 
	    $section_id = $this->this_site()->{section_id};
	return $this->owns($section_id);
    }
    return undef;
}

sub rss_link {
    my $this = shift;
    if ($this->{section_id}) {
	my $s = new ExSite::Section(id=>$this->{section_id});
	my $path = $s->diskpath;
	if (-e "$path/EvtCal.rss") {
	    my $url = $s->get_url;
	    return $ml->div(
			    $ml->a(
				   $ml->img(undef,{ border=>0, src=>"$config{server}{HTMLpath}/_Modules/EvtCal/rss.png"})."RSS",
				   { href=>"$url/EvtCal.rss" }
				   ),
			    { class=>"EvtCalRSS" }
			    );
	}
    }
    return undef;
}

sub rss {
    my $this = shift;

    # get section and service page
    my $site = $share{DB}->this_site();
    return undef if (! $site);
    my $s = new ExSite::Section(section=>$site);
    my $page = $s->service_page("EvtCal");
    return undef if (! $page);

    # get event list for feed
    my $limit = $config{EvtCal}{rss_length} || 10;
    my $t = new ExSite::Time(); # now;
    my @event = $this->fetch_events($t->write("sql_date"),$limit,$s->id);

    # compile feed
    my $rss = new ExSite::RSS;
    $rss->title($s->get_my("title")." - Event Calendar");
    $rss->description("Upcoming events as of ".$t->write("date"));
    $rss->link($page->get_url);

    foreach my $e (@event) {
	my $evt = new Modules::Event::Event(data=>$e);
	my $descr = &html_to_plaintext($e->{description});
	my $item = {
	    title=>$evt->name,
	    description=>$evt->show_date(undef,format=>"full")."\n\n".$descr,
	    link=>$page->link(evt=>$e->{evt_id}),
	    pubDate=>$evt->show_date(undef,format=>"arpa"),
	};
	$rss->item($item);
    }
    return $rss->write;
}

sub publish {
    my ($this) = @_;

    # get section and service page
    my $site = $share{DB}->this_site();
    return undef if (! $site);
    my $s = new ExSite::Section(section=>$site);
    my $sid = $s->id;
    my $page = $s->service_page("EvtCal");
    return undef if (! $page);
    my $ml = &get_obj("ML");
    $this->setup_querylib();
    my $spath = $s->diskpath();

    # spoof the service page to get the correct links
    $page->update_env();

    my $out = "Publishing Event Calendar RSS feed...".$ml->br;
    my $rss = $this->rss;

    # restore environment (just in case)
    $ENV{SCRIPT_NAME} = $share{ENV}{SCRIPT_NAME};
    $ENV{QUERY_STRING} = $share{ENV}{QUERY_STRING};
    $ENV{PATH_INFO} = $share{ENV}{PATH_INFO};

    my $file = "$spath/EvtCal.rss";
    $file =~ /^(((\/[\w-]+)+)\/([\w-]+\.[\w]+))$/;
    my $secure_filename = $1;
    if ($secure_filename) {
	$out .= "Writing EvtCal.rss... ";
	if (open OUTFILE, ">$secure_filename") {
	    print OUTFILE $rss;
	    close OUTFILE;
	    $out .= "succeeded.", $ml->br;
	}
	else {
	    $out .= $this->error("EvtCal::publish: publish $secure_filename failed: $!");
	}
    }
    return $out;
}

sub unpublish {
    my ($this) = @_;

    my $site = $share{DB}->this_site();
    return undef if (! $site);
    my $s = new ExSite::Section(section=>$site);
    my $spath = $s->diskpath();

    my $out = "Removing Event Calendar RSS feed...".$ml->br;
    my $file = "$spath/EvtCal.rss";
    $file =~ /^(((\/[\w-]+)+)\/([\w-]+\.[\w]+))$/;
    my $secure_filename = $1;
    if ($secure_filename) {
	if (-e $file) {
	    unlink $file;
	}
	else {
	    $out .= $this->warn("EvtCal: no feed to remove");
	}
    }
    else {
	$out .= $this->error("EvtCal::publish: publish $secure_filename failed: $!");
    }
    return $out;
}

sub admbutton {
    my ($this,$button,$url,$tooltip) = @_;
    return $ml->a(
		  $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/EvtCal/${button}.gif",
				  alt=>$tooltip,
				  title=>$tooltip,
				  border=>0,
				  height=>13,
				  width=>12,
			      }),
		  {href=>$url}
		  );
}

sub summary {
    my $this = shift;
    my $sid = shift;
    my $ml = &get_obj("ML");
    if (! $sid) {
	if ($share{Page}) {
	    $sid = $share{Page}->id("section");
	}
	else {
	    $this->read();
	    $sid = $this->{input}{section_id};
	}
    }
    $this->setup_querylib();
    my $today = ExSite::Time->new()->write("sql_date");
    my @evt = $share{DB}->get_query("events after date",$today,$sid,5);
    my $r = new ExSite::ReportBuilder(title=>"Upcoming Events");
    $r->headers("Date","Event");
    foreach my $e (@evt) {
	my $evt = new Modules::Event::Event(data=>$e);
	$r->push($evt->date,$evt->name);
    }
    return $r->make();
}

sub this_site {
    my $this = shift;
    my $site = $share{DB}->this_site();
    return $site ? $site : { section_id => 0 };
}

#--- handlers (not really in this class)

sub select_parent_events {
    my ($db,$table,$record,$column,$ftable) = @_;
    if ($table eq "evt" && $column eq "parent") {
	my $sid = $EvtCal->{section_id};
	$sid or $sid = $EvtCal->get_section_id;
	$sid = 0 if (! defined $sid);
	my $rdata = $db->get_query("all parent events in section",$sid);
	return $rdata;
    }
    return undef;
}

1;
