#!/usr/bin/perl

=pod

=head1 Modules::Forum

This is a wrapper for the Zine plug-in, to make it easier to work with Forums.

=cut

package Modules::Forum;

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Time;
use ExSite::HTML;
use Modules::Zine;
use Modules::Zine::Form;

use vars qw(@ISA $ml);
@ISA = qw(Modules::Zine);

sub setup {
    my ($this,$article,$opt) = @_;
    # make sure our DBmaps are loaded
    $share{DB}{map}->load_submaps("Zine");
    $this->SUPER::setup($article,$opt);
}

sub write {
    my ($this,$options) = @_;
    my %opt = &DecodeString($options);
    my $in = $this->{input};
    my $cmd = $in->{zine} || $opt{zine};
    my $aid = $in->{aid} || $opt{aid};
    if ($cmd || $aid) {
	# Zine can take it from here
	return $this->SUPER::write($options);
    }
    else {
	# see if there is a good default blog to use
	my $sid = $this->get_section_id();
	if ($sid) {
	    my @blog = $this->fetch_match({type=>"forum",section_id=>$sid,status=>"active"});
	    my $nblog = scalar @blog;
	    if ($nblog == 1) {
		$this->load_article($blog[0]);
		return $this->show();
	    }
	    elsif ($nblog > 1) {
		return $this->zinedex("forum",$sid);
	    }
	}
    }
    return $this->error("No forums available.");
}

sub ioctl {
    my ($this) = shift;
    $_ = shift;

    if (/ControlPanel/) {
	return \&ctrl_panel;
    }
    if (/Summary/) {
	return \&summary;
    }
    elsif (/ModuleName/) {
	return "Forums";
    }
    elsif (/isRestricted/) {
	return $this->{config}{isRestricted};
    }
    elsif (/isService/) {
	return $this->{config}{isService};
    }
    elsif (/Publish/) {
	return \&Modules::Zine::Base::publish;
    }
#   elsif (/ISA/) {
#	return "Zine";
#   }
    elsif (/DynContentParam/) {
	my $sid = shift;
	my @blog = $this->fetch_match({type=>"forum",section_id=>$sid,status=>"active"});
	$ml = &get_obj("ML");
	my $opt = 
	    $ml->option("&lt;default&gt;",{value=>""}).
	    $ml->option("show all forums",{value=>""});
	foreach my $b (@blog) {
	    $opt .= $ml->option($b->{title},{value=>"aid=$b->{article_id}&amp;zine=show"});
	}
	return "Forums: " . $ml->select($opt,{name=>"param"});
    }
    elsif (/Dependencies/) {
	return "Zine";
    }
}

sub topview_admin {
    my ($this,$opt) = @_;
    if (! $share{DB}->is_manager) {
	return $this->error("Permission denied.");
    }
    return $this->zinedex("forum",$this->{input}{section_id});
}

sub zine_selector_form {
    my ($this,$sid) = @_;
    return undef;
}

sub new_zine_button {
    my $this = shift;
    my $url = $this->link(zine=>"newforum");
    return &ExSite::HTML::Button(url=>$url,label=>"Create a new Forum");
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    $ml = &get_obj("ML");
    my $sid = $this->get_section_id();
    my $in = $this->{input};

    if ($in->{zine} eq "newforum") {
	$out .= $this->new_forum();
    }
    elsif ($in->{zine} eq "newcat") {
	$out .= $this->new_category();
    }
    else {
	$out .= $this->SUPER::ctrl_panel();
    }
    return $out;
}

sub topview_admin {
    my ($this,$opt) = @_;
    my $db = $share{DB};
    return $this->zinedex(
			  type=>"forum",
			  section_id=> $db->is_admin ? 0 : $share{DB}->my_user_record()->{section_id},
			  #no_section_name=>1,
			  );
}

sub new_forum {
    my $this = shift;
    my $out = $ml->h1("Start a new forum");
    $out .= $ml->p("Please give this forum a concise topic or description.");
    my $section_id = $share{DB}->my_user_record()->{section_id};
    my @book = $share{DB}->fetch_match("article",{type=>"book",status=>"active",parent_id=>0,section_id=>$section_id},"title");
    my @category;
    foreach my $book (@book) {
	push @category, { value=>$book->{article_id}, text=>$book->{title} };
    }
    my $zf = new Modules::Zine::Form();
    my $f = new ExSite::FormBuilder(action=>$this->link(zine=>"submit"));
    $f->input(
	      name=>"title",
	      prompt=>"Topic of forum:",
	      type=>"text",
	      required=>1,
	      size=>60,
	      );
    $f->input(
	      name=>"parent_id$config{form}{sepchar}key",
	      prompt=>"Category (optional):",
	      type=>"select",
	      options=>\@category,
	      nullvalue=>"== choose category ==",
	      tail=>$ml->a("+ add category",{href=>$this->link(zine=>"newcat")}),
	      );
    $f->input(
	      name=>"type",
	      type=>"hidden",
	      value=>"forum",
	      );
    $f->input(
	      name=>"section_id$config{form}{sepchar}key",
	      type=>"hidden",
	      value=>$section_id,
	      );
    $f->input(
	      name=>"ctrlinfo",
	      type=>"hidden",
	      value=>$zf->ctrlinfo()
	      );
    $out .= $f->make;
    return $out;
}

sub new_forum {
    my $this = shift;
    my $out = $ml->h1("Start a new forum");
    my $section_id = $share{DB}->my_user_record()->{section_id};
    my $zf = new Modules::Zine::Form();
    my $f = new ExSite::FormBuilder();
    if (exists $this->{input}{parent_id}) {
	$out .= $ml->p("Please give this forum a concise topic or description.");
	$f->action($this->link(zine=>"submit"));
	$f->input(
		  name=>"title",
		  prompt=>"Topic of forum:",
		  type=>"text",
		  required=>1,
		  size=>60,
		  );
	$f->input(
		  name=>"type",
		  type=>"hidden",
		  value=>"forum",
		  );
	$f->input(
		  name=>"section_id$config{form}{sepchar}key",
		  type=>"hidden",
		  value=>$section_id,
		  );
	$f->input(
		  name=>"ctrlinfo",
		  type=>"hidden",
		  value=>$zf->ctrlinfo({parent_id=>$this->{input}{parent_id}})
		  );
    }
    else {
	$out .= $ml->p("You can optionally place this forum into a separate category.");
	$f->action($this->link());
	my @book = $share{DB}->fetch_match("article",{type=>"book",status=>"active",parent_id=>0,section_id=>$section_id},"title");
	my @category;
	foreach my $book (@book) {
	    push @category, { value=>$book->{article_id}, text=>$book->{title} };
	}
	$f->input(
		  name=>"parent_id",
		  prompt=>"Category (optional):",
		  type=>"select",
		  options=>\@category,
		  nullvalue=>"== no category ==",
		  tail=>$ml->a("+ add category",{href=>$this->link(zine=>"newcat")}),
		  );
	$f->set("buttons",
		$ml->input(undef,{type=>"submit",value=>"Continue"})." ".
		$ml->input(undef,{type=>"button",value=>"Cancel",onclick=>"javasciprt:history.back()"})." ".
		$ml->input(undef,{type=>"reset",value=>"Reset"})
		);
    }
    $out .= $f->make;
    return $out;
}

sub new_category {
    my $this = shift;
    my $out = $ml->h1("New Forum Category");
    $out .= $ml->p("Forum categories are used to break a large number of forums into groups for easier browsing.");
    my $section_id = $share{DB}->my_user_record()->{section_id};
    my $zf = new Modules::Zine::Form();
    my $f = new ExSite::FormBuilder(action=>$this->link(zine=>"submit"));
    $f->input(
	      name=>"title",
	      prompt=>"Forum Category:",
	      type=>"text",
	      required=>1,
	      size=>60,
	      );
    $f->input(
	      name=>"type",
	      type=>"hidden",
	      value=>"book",
	      );
    $f->input(
	      name=>"section_id$config{form}{sepchar}key",
	      type=>"hidden",
	      value=>$section_id,
	      );
    $f->input(
	      name=>"ctrlinfo",
	      type=>"hidden",
	      value=>$zf->ctrlinfo({})
	      );
    $out .= $f->make;
    return $out;
}

sub summary {
    my $this = shift;
    return $this->zinedex(
			  type=>"forum",
			  nolink=>1,
			  section_id=>$share{DB}->my_user_record()->{section_id},
			  no_section_name=>1,
			  );
}

1;
