package Modules::JGallery;

#----------------------------------------------------------------------------
#
#   Copyright (C) 2010 - Matthew Wong
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;

# useful kernel libs; you can/should remove any of these that are not needed

use ExSite::Config;          # required
use ExSite::Captcha;
use ExSite::Input;           # optional input manager
use ExSite::Misc;            # optional utils
use ExSite::Util;            # optional utils
use ExSite::ML;              # optional markup-language generation
use ExSite::FormBuilder;     # optional form generation
use ExSite::ReportBuilder;   # optional report generation
use ExSite::Form;            # optional database manager

# recommended base class for plug-in modules

use Modules::BaseDCD;

# declare package globals

use vars qw(@ISA $ml);       # Perl inheritance global & markup language object

# define our class inheritance

@ISA = qw(Modules::BaseDCD); # inherit from this base class

# ioctl method (used by ExSite to query the module for its functionality)

sub ioctl {
	my $this = shift;
	$_ = shift;           # $_ is the ioctl request

	if (/isService/) {
		return 1;
	}
	elsif (/ControlPanel/) {
		return \&ctrl_panel;
	}
	elsif (/ModuleName/) {
		return "JGallery";
	}
	elsif (/DynContentParam/) {
		my $section_id = shift;
		my $ml = &get_obj("ML");
		my $opt;
		
		my @jgalleries = $share{DB}->fetch_match("jgallery", { section_id => $section_id });
		foreach my $jgallery (@jgalleries) {
			my $library = $share{DB}->fetch("library", $jgallery->{library_id});
			$opt .= $ml->option($library->{title} . " - " . $jgallery->{type},{ value => "cmd=display_jgallery&jgallery_id=".$jgallery->{jgallery_id} });
		}
		return "JGallery: " . $ml->select($opt,{ name => "param" });
	}	
}

sub ctrl_panel {
	my $this = shift;
	
	$ml = $share{ML};
	$this->{section_id} = $this->{input}{section_id};
	
	if (!$this->{section_id}) {
		my @sites = $share{DB}->my_real_sites();
		if (scalar @sites == 1) {
			$this->{section_id} = $sites[0]->{section_id};
		}
	}

	my $out;  
	my $cmd = $this->{input}{cmd};
	
	if ($this->{section_id} && $cmd =~ /^(add_edit_jgallery|delete_jgallery)$/) {
		$out .= $this->$cmd();
	}
	elsif ($this->{section_id}) {
		$out .= $this->jgallery_report();
	}
	else {
		$out .= $this->set_section_id();
	}
	
	return $out;
}

sub jgallery_report {
	my $this = shift;
	my $out;
	
	my $r = new ExSite::ReportBuilder();
	
	my @jgalleries = $share{DB}->fetch_match("jgallery", { section_id => $this->{section_id} });
		
	$r->init(title => "JGalleries",
		headers => ["Library", "Gallery Type", "Options"],
		nodata => "No JGalleries found.",
		tools => $this->add_button(),
	);
		
	foreach my $jgallery (@jgalleries) 
	{
		my @row = $this->report_row($jgallery);
		$r->push(@row) if (scalar @row);
	}
	
	my $section = $share{DB}->fetch("section", $this->{section_id});
	return $ml->h1("JGalleries for " . $section->{title}) . $r->make();
}

sub add_button {
	my $this = shift;
		
	my $newAttributes = { 
		    src => $config{server}{HTMLpath} . "/_Modules/JGallery/add.png",
		    border => "0",
		    alt => "new",
		    title => "new"
	};
	
	my $new = $this->link(section_id => $this->{section_id}, cmd => "add_edit_jgallery", doform => undef, jgallery_id => undef);
	my $out = $ml->a("Add JGallery" . $ml->img(undef, $newAttributes), { href => $new });
	return $out;
}

sub report_row {
	my ($this,$object) = @_;
	
	my $library = $share{DB}->fetch("library", $object->{library_id});
	my $warntext = "Delete " . $library->{title} . " - " . $object->{type} . "?";
		
	my $p = &ExSite::Module::service_page("JGallery", $this->{section_id});
	my $service_page_id = $p->id if ($p);
	my $section = $share{DB}->fetch("section", $this->{section_id});
	my $site_url = $section->{url} || $config{server}{server};	
	my $u = new ExSite::URI(uri=>$site_url);
	
	my $script_name = $share{ENV}{SCRIPT_NAME};
	$script_name =~ s/ctrl\-panel\.cgi/page\.cgi/;
	$u->script_name($script_name);
	$u->plaintext;
	$u->query(_id => $service_page_id, jgallery_id => $object->{jgallery_id}, doform => undef, cmd => "display_jgallery");	
	
	my $preview = $p ? $u->write_full : "javascript:alert('You must first setup a service page for this section before using the preview function.')";
	my $edit = $this->link(section_id => $this->{section_id}, jgallery_id => $object->{jgallery_id}, doform => undef, cmd => "add_edit_jgallery");
	my $delete = $this->link(section_id => $this->{section_id}, jgallery_id => $object->{jgallery_id}, doform => undef, cmd => "delete_jgallery");

	my $previewAttributes = {
		src => $config{server}{HTMLpath} . "/_Modules/JGallery/preview.png",
		border => "0",
		alt => "preview",
		title => "preview"
	};	
	my $editAttributes = { 
		src => $config{server}{HTMLpath} . "/_Modules/JGallery/user_edit.png",
		border => "0",
		alt => "edit",
		title => "edit"
	};
	my $deleteAttributes = {
		src => $config{server}{HTMLpath} . "/_Modules/JGallery/delete.png",
		border => "0",
		alt => "delete",
		title => "delete"
	};

	my $preview_link = { 
		href => $preview, 
		style =>"padding-right: 5px;"
	};
	$preview_link->{target} = "_blank" if ($p);
	
	my $imageLinks = $ml->a($ml->img(undef, $previewAttributes), $preview_link) .
		$ml->a($ml->img(undef, $editAttributes), { href => $edit, style =>"padding-right: 5px;" }) .
		$ml->a($ml->img(undef, $deleteAttributes), { href => "javascript:confirm_custom('$warntext','$delete')" }) . "\n";

	my @row;
	push(@row, 
		$library->{title},
		$object->{type},
		$imageLinks);
	
	return @row;
}

sub delete_jgallery {
	my $this = shift;
	my $out;

	if ($this->{input}{jgallery_id}) {
		my $jgallery = $share{DB}->fetch("jgallery", $this->{input}{jgallery_id});
		my $library = $share{DB}->fetch("library", $jgallery->{library_id});
		$share{DB}->trash_key("jgallery", $this->{input}{jgallery_id});

		$out .= $ml->h1("Deleted " . $library->{title} . " - " . $jgallery->{type});
	}

	$out .= $this->jgallery_report();
	return $out;
}

sub add_edit_jgallery {
	my $this = shift;
	my $out;
	
	if ($this->{input}{doform}) {
		return $this->do_jgallery_form();
	}
	
	my $jgallery = $share{DB}->fetch("jgallery", $this->{input}{jgallery_id});
	my %jgallery_config = &DecodeString($jgallery->{config});
	
	my $pirobox_display = ($jgallery->{type} eq "piroBox") ? "block" : "none";
	my $galleryview_display = ($jgallery->{type} eq "GalleryView") ? "block" : "none";
	my $galleriffic_display = ($jgallery->{type} eq "Galleriffic") ? "block" : "none";
		
	my $f = new ExSite::FormBuilder(action=>$this->link(doform=>1),method=>"POST");		
	$f->template('
<script type="text/javascript">
function showOptions (type) {
	hideAll();
	document.getElementById(type).style.display = "block";
}

function hideAll () {
	var option_boxes = document.getElementsByName("option_box");
	for(i = 0; i < option_boxes.length; i++) { 
		option_boxes[i].style.display = "none";
	}	
}
</script>
<style type="text/css">
div.add_form table td {
padding-top: 10px;
}
div.add_form table td.first {
padding-top: 0;
}
</style>
<div class="add_form">
<table><tr>
<td class="first">[[library_id:prompt]]<br/>[[library_id:input]]</td>
</tr>
<tr>
<td>[[type:prompt]]<br/>[[type:input]]</td>
</tr></table>
<br/>
<div name="option_box" id="piroBox" style="display: '.$pirobox_display.';">
<h2>piroBox Configuration</h2>
<table>
<tr>
<td class="first">[[pirobox_transition_delay:prompt]]<br/>[[pirobox_transition_delay:input]]</td>
</tr>
<tr>
<td>[[pirobox_transition_duration:prompt]]<br/>[[pirobox_transition_duration:input]]</td>
</tr>
<tr>
<td>[[pirobox_display_image_titles:prompt]]<br/>[[pirobox_display_image_titles:input]]</td>
</tr>
<tr>
<td>[[pirobox_display_image_captions:prompt]]<br/>[[pirobox_display_image_captions:input]]</td>
</tr>
<tr>
<td>[[pirobox_background_opacity:prompt]]<br/>[[pirobox_background_opacity:input]]</td>
</tr>
</table>
</div>

<div name="option_box" id="GalleryView" style="display: '.$galleryview_display.';">
<h2>GalleryView Configuration</h2>
<table>
<tr>
<td class="first">[[galleryview_theme:prompt]]<br/>[[galleryview_theme:input]]</td>
</tr>
<tr>
<td>[[galleryview_transition_delay:prompt]]<br/>[[galleryview_transition_delay:input]]</td>
</tr>
<tr>
<td>[[galleryview_transition_duration:prompt]]<br/>[[galleryview_transition_duration:input]]</td>
</tr>
<tr>
<td>[[galleryview_display_image_titles:prompt]]<br/>[[galleryview_display_image_titles:input]]</td>
</tr>
<tr>
<td>[[galleryview_display_image_captions:prompt]]<br/>[[galleryview_display_image_captions:input]]</td>
</tr>
<tr>
<td>[[galleryview_show_panels:prompt]]<br/>[[galleryview_show_panels:input]]</td>
</tr>
<tr>
<td>[[galleryview_panel_position:prompt]]<br/>[[galleryview_panel_position:input]]</td>
</tr>
<tr>
<td>[[galleryview_panel_opacity:prompt]]<br/>[[galleryview_panel_opacity:input]]</td>
</tr>
<tr>
<td>[[galleryview_inactive_frame_opacity:prompt]]<br/>[[galleryview_inactive_frame_opacity:input]]</td>
</tr>
<tr>
<td>[[galleryview_show_filmstrip:prompt]]<br/>[[galleryview_show_filmstrip:input]]</td>
</tr>
<tr>
<td>[[galleryview_filmstrip_position:prompt]]<br/>[[galleryview_filmstrip_position:input]]</td>
</tr>
<tr>
<td>[[galleryview_num_thumbs:prompt]]<br/>[[galleryview_num_thumbs:input]]</td>
</tr>
</table>
</div>

<div name="option_box" id="Galleriffic" style="display: '.$galleriffic_display.';">
<h2>Galleriffic Configuration</h2>
<table>
<tr>
<td class="first">[[galleriffic_delay:prompt]]<br/>[[galleriffic_delay:input]]</td>
</tr>
<tr>
<td>[[galleriffic_transition_duration:prompt]]<br/>[[galleriffic_transition_duration:input]]</td>
</tr>
<tr>
<td>[[galleriffic_display_image_titles:prompt]]<br/>[[galleriffic_display_image_titles:input]]</td>
</tr>
<tr>
<td>[[galleriffic_display_image_captions:prompt]]<br/>[[galleriffic_display_image_captions:input]]</td>
</tr>
<tr>
<td>[[galleriffic_num_thumbs:prompt]]<br/>[[galleriffic_num_thumbs:input]]</td>
</tr>
<tr>
<td>[[galleriffic_enable_top_pager:prompt]]<br/>[[galleriffic_enable_top_pager:input]]</td>
</tr>
<tr>
<td>[[galleriffic_enable_bottom_pager:prompt]]<br/>[[galleriffic_enable_bottom_pager:input]]</td>
</tr>
<tr>
<td>[[galleriffic_max_pages:prompt]]<br/>[[galleriffic_max_pages:input]]</td>
</tr>
<tr>
<td>[[galleriffic_render_ss_controls:prompt]]<br/>[[galleriffic_render_ss_controls:input]]</td>
</tr>
<tr>
<td>[[galleriffic_render_nav_controls:prompt]]<br/>[[galleriffic_render_nav_controls:input]]</td>
</tr>
<tr>
<td>[[galleriffic_auto_start:prompt]]<br/>[[galleriffic_auto_start:input]]</td>
</tr>
</table>
</div>
</div>
<br/>
');			

	my $s = new ExSite::Section(id => $this->{section_id});
	my @libs = $s->fetch_libraries();
	
	my $options;
	foreach my $lib (@libs) {
		my $contains_images = 0;
		my @pics = $share{DB}->fetch_child("content","page",$lib->{page_id},"name");
		
		foreach my $pic (@pics) {
			my $c = new ExSite::Content(content=>$pic);	    
			$c->set_context("child");
			if ($c->content_type() =~ /^image\/(jpe?g|gif|png)$/) {
				$contains_images = 1;
				last;
			}
		}
		
		next unless ($contains_images);
		
		push(@$options, { value => $lib->{page_id}, text => $lib->{title} });
	}
	
	$f->input(
		type => "select",
		name => "library_id",
		prompt => "Library",
		options => $options,
		nullvalue => $msg{"== select =="},
		value => $jgallery->{library_id},
		required => 1
	);	
	
	$f->input(
		type => "select",
		name => "type",
		prompt => "Type",
		options => [{ value => "piroBox", text => "piroBox" }, 
			#{ value => "GalleryView", text => "GalleryView" }, 
			{ value => "Galleriffic", text => "Galleriffic" }],
		nullvalue => $msg{"== select =="},
		onChange => "showOptions(this.value)",
		value => $jgallery->{type},
		required => 1
	);
	
	# Galleriffic Configuration Inputs #
	$f->input(
		type => "text",
		name => "galleriffic_delay",
		prompt => $msg{"Transition delay (in seconds)"},
		size => 10,
		value => $jgallery_config{galleriffic_delay} || "3"
	);
	
	$f->input(
		type => "text",
		name => "galleriffic_transition_duration",
		prompt => $msg{"Transition duration (in seconds)"},
		size => 10,
		value => $jgallery_config{galleriffic_transition_duration} || "1"
	);
	
	$f->input(
		type => "select",
		name => "galleriffic_display_image_titles",
		prompt => $msg{"Display image titles"},
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{galleriffic_display_image_titles} || "true"
	);		
	
	$f->input(
		type => "select",
		name => "galleriffic_display_image_captions",
		prompt => $msg{"Display image captions"},
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{galleriffic_display_image_captions} || "true"
	);	
	
	$f->input(
		type => "text",
		name => "galleriffic_num_thumbs",
		prompt => $msg{"Number of thumbnails to show per page"},
		size => 10,
		value => $jgallery_config{galleriffic_num_thumbs} || "9"
	);	
	
	$f->input(
		type => "select",
		name => "galleriffic_enable_top_pager",
		prompt => $msg{"Enable top page navigation"},
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{galleriffic_enable_top_pager} || "false"
	);	
	
	$f->input(
		type => "select",
		name => "galleriffic_enable_bottom_pager",
		prompt => $msg{"Enable bottom page navigation"},
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{galleriffic_enable_bottom_pager} || "true"
	);	
	
	$f->input(
		type => "text",
		name => "galleriffic_max_pages",
		prompt => $msg{"Maximum number of pages to display on page navigation"},
		size => 10,
		value => $jgallery_config{galleriffic_max_pages} || "5"
	);	
	
	$f->input(
		type => "select",
		name => "galleriffic_render_ss_controls",
		prompt => $msg{"Display play/pause links"},
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{galleriffic_render_ss_controls} || "true"
	);	
	
	$f->input(
		type => "select",
		name => "galleriffic_render_nav_controls",
		prompt => $msg{"Display next/previous links"},
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{galleriffic_render_nav_controls} || "true"
	);	
	
	$f->input(
		type => "select",
		name => "galleriffic_auto_start",
		prompt => $msg{"Automatically start the slideshow"},
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{galleriffic_auto_start} || "false"
	);	
	
	################################
	
	# piroBox Configuration Inputs #
	
	$f->input(
		type => "text",
		name => "pirobox_transition_delay",
		prompt => $msg{"Transition delay (in seconds)"},
		size => 10,
		value => $jgallery_config{pirobox_transition_delay} || "4"
	);	
	
	$f->input(
		type => "text",
		name => "pirobox_transition_duration",
		prompt => $msg{"Transition duration (in seconds)"},
		size => 10,
		value => $jgallery_config{pirobox_transition_duration} || "0.6"
	);	
	
	$f->input(
		type => "select",
		name => "pirobox_display_image_titles",
		prompt => $msg{"Display image titles"},
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{pirobox_display_image_titles} || "true"
	);		
	
	$f->input(
		type => "select",
		name => "pirobox_display_image_captions",
		prompt => $msg{"Display image captions"},
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{pirobox_display_image_captions} || "true"
	);		
	
	$f->input(
		type => "text",
		name => "pirobox_background_opacity",
		prompt => $msg{"Background opacity (percentage)"},
		size => 10,
		value => $jgallery_config{pirobox_background_opacity} || "50"
	);
	
	################################
	
	# GalleryView Configuration Inputs #
	
	$f->input(
		type => "select",
		name => "galleryview_theme",
		prompt => $msg{"Gallery Theme"},
		size => 10,
		options => ["light", "dark"],
		value => $jgallery_config{galleryview_theme} || "light"
	);	
	
	$f->input(
		type => "text",
		name => "galleryview_transition_delay",
		prompt => $msg{"Transition delay (in seconds)"},
		size => 10,
		value => $jgallery_config{galleryview_transition_delay} || "4"
	);	
	
	$f->input(
		type => "text",
		name => "galleryview_transition_duration",
		prompt => $msg{"Transition duration (in seconds)"},
		size => 10,
		value => $jgallery_config{galleryview_transition_duration} || "0.6"
	);		
		
	$f->input(
		type => "select",
		name => "galleryview_show_panels",
		prompt => $msg{"Show panel portion of the gallery"},
		size => 10,
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{galleryview_show_panels} || "true"
	);
	
	$f->input(
		type => "select",
		name => "galleryview_panel_position",
		prompt => $msg{"Panel position"},
		size => 10,
		options => ["bottom", "top", "left", "right"],
		value => $jgallery_config{galleryview_panel_position} || "bottom"
	);		
	
	$f->input(
		type => "text",
		name => "galleryview_panel_opacity",
		prompt => $msg{"Panel opacity (in percent)"},
		size => 10,
		value => $jgallery_config{galleryview_panel_opacity} || "70"
	);	
	
	$f->input(
		type => "text",
		name => "galleryview_inactive_frame_opacity",
		prompt => $msg{"Non-active frame opacity (in percent)"},
		size => 10,
		value => $jgallery_config{galleryview_inactive_frame_opacity} || "30"
	);		
	
	$f->input(
		type => "select",
		name => "galleryview_show_filmstrip",
		prompt => $msg{"Show filmstrip portion of the gallery"},
		size => 10,
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{galleryview_show_filmstrip} || "true"
	);		
	
	$f->input(
		type => "select",
		name => "galleryview_filmstrip_position",
		prompt => $msg{"Filmstrip position"},
		size => 10,
		options => ["bottom", "top", "left", "right"],
		value => $jgallery_config{galleryview_filmstrip_position} || "bottom"
	);		
	
	$f->input(
		type => "select",
		name => "galleryview_display_image_titles",
		prompt => $msg{"Display image titles"},
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{galleryview_display_image_titles} || "true"
	);		
	
	$f->input(
		type => "select",
		name => "galleryview_display_image_captions",
		prompt => $msg{"Display image captions"},
		options => [{ text => "yes", value => "true" }, { text => "no", value => "false" }],
		value => $jgallery_config{galleryview_display_image_captions} || "true"
	);		
	
	$f->input(
		type => "text",
		name => "galleryview_num_thumbs",
		prompt => $msg{"Number of thumbnails to show in filmstrip"},
		size => 10,
		value => $jgallery_config{galleryview_num_thumbs} || "9"
	);	
	
	####################################
	
	my $action = $this->{input}{jgallery_id} ? "Update" : "Add";
	$out .= $ml->h1("$action a JGallery") . $f->make();
	return $out;
}

sub do_jgallery_form {
	my $this = shift;
	my $out;
	my $type = $this->{input}{type};
	my @config_settings;
	
	
	my $input = $this->{input};
	if ($type eq "piroBox") {
		foreach my $param (keys %$input) {
			push(@config_settings, "$param=" . $input->{$param}) if ($param =~ /^pirobox_/);
		}
	}
	elsif ($type eq "GalleryView") {
		foreach my $param (keys %$input) {
			push(@config_settings, "$param=" . $input->{$param}) if ($param =~ /^galleryview_/);
		}	
	}
	elsif ($type eq "Galleriffic") {
		foreach my $param (keys %$input) {
			push(@config_settings, "$param=" . $input->{$param}) if ($param =~ /^galleriffic_/);
		}
	}
	
	my $jgallery = {
		section_id => $this->{section_id},
		library_id => $this->{input}{library_id},
		type => $type,
		config => join("&", @config_settings)
	};
	
	my $library = $share{DB}->fetch("library", $this->{input}{library_id});
	if ($this->{input}{jgallery_id}) {
		$share{DB}->update("jgallery", $jgallery, { jgallery_id => $this->{input}{jgallery_id} });
		$out .= $ml->h1("Updated " . $library->{title} . " - $type");
	}
	else {
		$share{DB}->insert("jgallery", $jgallery);
		$out .= $ml->h1("Inserted " . $library->{title} . " - $type");
	}
	
	return $out . $this->jgallery_report();
}

sub write {
	my ($this,$options) = @_;

	$ml = new ExSite::ML;
	my %opt = &DecodeString($options);
	my $cmd = $opt{cmd} || $this->{input}{cmd};
	my $out;

	if ($cmd =~ /^(display_jgallery)$/) {
		$out .= $this->$cmd(\%opt);
	}

	return $this->wrap($out);
}

sub display_jgallery {
	my $this = shift;
	my $opt = shift;
	my $out;
	
	my $jgallery = $share{DB}->fetch("jgallery", ($opt->{jgallery_id} || $this->{input}{jgallery_id}));
	my $library = $share{DB}->fetch("library",$jgallery->{library_id});
	
	if (!$config{JGallery}{exclude_jquery}) {
		my $file_path = $config{server}{HTMLpath} . "/_Modules/JGallery";
		$out .= $ml->script(undef, { type => "text/javascript", src => "$file_path/jquery-1.3.2.js" });
	}
	
	if ($jgallery->{type} eq "piroBox") {
		$out .= $this->display_pirobox($opt, $jgallery, $library);
	}
	elsif ($jgallery->{type} eq "GalleryView") {
		$out .= $this->display_galleryview($opt, $jgallery, $library);
	}
	elsif ($jgallery->{type} eq "Galleriffic") {
		$out .= $this->display_galleriffic($opt, $jgallery, $library);
	}
	
	return $ml->h1($library->{title}) . $out;
}

sub display_galleryview {
	my $this = shift;
	my $opt = shift;
	my $jgallery = shift;
	my $library = shift;
	my $out;
	
	my %jgallery_config = &DecodeString($jgallery->{config});
	my @pics = $share{DB}->fetch_child("content","page",$jgallery->{library_id},"name");
	
	my $pics;
	my $thumbs;
	my ($t_width, $t_height);
	foreach my $pic (@pics) {
		my $c = new ExSite::Content(content=>$pic);	    
		$c->set_context("child");
		next unless ($c->content_type() =~ /^image\/(jpe?g|gif|png)$/);
		
		my $url = $c->get_url;
		my $thumb = $c->get_thumb_html;
		
		$t_width = $opt->{thumbnail_size} || 75;
		$t_height = $opt->{thumbnail_size} || 75;	    	
		
		my $thumb_url = $c->get_thumb_url;
		$thumb = <<END;
<img src="$thumb_url">
END

		my $title = $ml->h3($c->get("content","name"), { class => "title" }) 
			if ($jgallery_config{galleryview_display_image_titles} eq "true");
		my $caption = $ml->p($c->get("content","description"), { class => "caption" })
			if ($jgallery_config{galleryview_display_image_captions} eq "true");
		
		#$pics .= $ml->div(
		#	$ml->img(undef, { src => $url }) .
		#	$ml->div($title . $caption, { class => "panel-overlay" }),
		#	{ class => "panel" }
		#);
		
		#$thumbs .= $ml->li($thumb);
		
		my $panel_overlay = $ml->div(
			$title .
			$caption,
			{ class => "panel-overlay" }
		) if ($title || $caption);
		
		$pics .= $ml->li(
			$ml->img(undef, { src => $url }) .
			$panel_overlay
		);
	}
	
	$out .= $ml->ul($pics, { id => "gallery" });
	#$out .= $ml->div($pics . $ml->ul($thumbs, { class => "filmstrip" }), { id => "photos", class => "galleryview" });
	
	my $file_path = $config{server}{HTMLpath} . "/_Modules/JGallery";
	my $transition_speed = $jgallery_config{galleryview_transition_duration} * 1000;
	my $transition_interval = $jgallery_config{galleryview_transition_delay} * 1000;
	my $overlay_opacity = $jgallery_config{galleryview_panel_opacity} / 100;
	my $frame_opacity = $jgallery_config{galleryview_inactive_frame_opacity} / 100;
	my $show_captions = ($jgallery_config{galleryview_display_image_titles} eq "true" || 
		$jgallery_config{galleryview_display_image_captions} eq "true") ? "true" : "false";

	$out .= <<END;
<link rel="stylesheet" href="$file_path/galleryview/galleryview.css" type="text/css" />
<script type="text/javascript" src="$file_path/galleryview/jquery.timers-1.2.js"></script>
<script type="text/javascript" src="$file_path/galleryview/jquery.galleryview-2.1.1.js"></script>
<script type="text/javascript">
	\$(document).ready(function() {
		\$('#gallery').galleryView({
			show_panels: $jgallery_config{galleryview_show_panels},		//BOOLEAN - flag to show or hide panel portion of gallery
			show_filmstrip: $jgallery_config{galleryview_show_filmstrip},		//BOOLEAN - flag to show or hide filmstrip portion of gallery

			panel_width: 600,		//INT - width of gallery panel (in pixels)
			panel_height: 400,		//INT - height of gallery panel (in pixels)
			frame_width: $t_width,		//INT - width of filmstrip frames (in pixels)
			frame_height: $t_height,		//INT - width of filmstrip frames (in pixels)

			start_frame: 1,			//INT - index of panel/frame to show first when gallery loads
			filmstrip_size: $jgallery_config{galleryview_num_thumbs},		
			transition_speed: $transition_speed,		//INT - duration of panel/frame transition (in milliseconds)
			transition_interval: $transition_interval,	//INT - delay between panel/frame transitions (in milliseconds)

			overlay_opacity: $overlay_opacity,		//FLOAT - transparency for panel overlay (1.0 = opaque, 0.0 = transparent)
			frame_opacity: $frame_opacity,		//FLOAT - transparency of non-active frames (1.0 = opaque, 0.0 = transparent)

			pointer_size: 8,		//INT - Height of frame pointer (in pixels)

			nav_theme: '$jgallery_config{galleryview_theme}',		//STRING - name of navigation theme to use (folder must exist within 'themes' directory)
			easing: 'swing',		//STRING - easing method to use for animations (jQuery provides 'swing' or 'linear', more available with jQuery UI or Easing plugin)

			filmstrip_position: '$jgallery_config{galleryview_filmstrip_position}',	//STRING - position of filmstrip within gallery (bottom, top, left, right)
			overlay_position: '$jgallery_config{galleryview_panel_position}',	//STRING - position of panel overlay (bottom, top, left, right)

			panel_scale: 'nocrop',		//STRING - cropping option for panel images (crop = scale image and fit to aspect ratio determined by panel_width and panel_height, nocrop = scale image and preserve original aspect ratio)
			frame_scale: 'crop',		//STRING - cropping option for filmstrip images (same as above)

			frame_gap: 5,			//INT - spacing between frames within filmstrip (in pixels)

			show_captions: $show_captions,		//BOOLEAN - flag to show or hide frame captions
			fade_panels: true,		//BOOLEAN - flag to fade panels during transitions or swap instantly
			pause_on_hover: false		//BOOLEAN - flag to pause slideshow when user hovers over the gallery
		});
	});
</script>
END
	return $out;
}

sub display_pirobox {
	my $this = shift;
	my $opt = shift;
	my $jgallery = shift;
	my $library = shift;
	my $out;
	
	my %jgallery_config = &DecodeString($jgallery->{config});
	my @pics = $share{DB}->fetch_child("content","page",$jgallery->{library_id},"name");
	
	my $pics;
	foreach my $pic (@pics) {
		my $c = new ExSite::Content(content=>$pic);	    
		$c->set_context("child");
		next unless ($c->content_type() =~ /^image\/(jpe?g|gif|png)$/);
		
		my $url = $c->get_url;
		my $thumb = $c->get_thumb_html;
		
		my $t_width = $opt->{thumbnail_size} || 75;
		my $t_height = $opt->{thumbnail_size} || 75;	    	
		
		my $thumb_url = $c->get_thumb_url;
		$thumb = <<END;
<img src="$thumb_url" width="$t_width" height="$t_height" border="0">
END
		
		my @img_info;
		my $title = push(@img_info, $ml->b($c->get("content","name"))) 
			if ($jgallery_config{pirobox_display_image_titles} eq "true");
		my $caption = push(@img_info, $c->get("content","description")) 
			if ($jgallery_config{pirobox_display_image_captions} eq "true");
		my $img_info = join("<br/>", @img_info);
		
		
		$out .= "<a href='$url' class='pirobox_gall' title='$img_info'>$thumb</a>";
	}
		
	my $file_path = $config{server}{HTMLpath} . "/_Modules/JGallery";
	my $delay = $jgallery_config{galleriffic_delay} * 1000;
	my $transition_duration = $jgallery_config{galleriffic_transition_duration} * 1000;
	
	my $speed = $jgallery_config{pirobox_transition_duration} * 1000;
	my $alpha = $jgallery_config{pirobox_background_opacity} / 100;
	
	$out .= <<END;
<script type="text/javascript" src="$file_path/pirobox/piroBox.1_2.js"></script>
<script type="text/javascript" src="$file_path/jquery.opacityrollover.js"></script>
<link rel="stylesheet" href="$file_path/pirobox/pirobox.css" type="text/css" />
<script type="text/javascript">
	var onMouseOutOpacity = 0.67;
	\$('a.pirobox_gall img').opacityrollover({
		mouseOutOpacity:   onMouseOutOpacity,
		mouseOverOpacity:  1.0,
		fadeSpeed:         'fast',
		exemptionSelector: '.selected'
	});

	\$(document).ready(function() {
		\$().piroBox({
				my_speed: $speed, //animation speed
				bg_alpha: $alpha, //background opacity
				radius: 4, //caption rounded corner
				scrollImage : false, // true == image follows the page, false == image remains in the same open position
				close_all : '.piro_close',// add class .piro_overlay(with comma)if you want overlay click close piroBox
				slideShow : 'slideshow', // just delete slideshow between '' if you dont want it.
				slideSpeed : $jgallery_config{pirobox_transition_delay} //slideshow duration in seconds(3 to 6 Recommended)
		});
	});
</script>
END
	
	return $out;
}

sub display_galleriffic {
	my $this = shift;
	my $opt = shift;
	my $jgallery = shift;
	my $library = shift;
	my $out;
	
	my %jgallery_config = &DecodeString($jgallery->{config});
	my @pics = $share{DB}->fetch_child("content","page",$jgallery->{library_id},"name");
	
	$out .= $ml->div(
		$ml->div(undef, { id => "controls", class => "class" }) .
		$ml->div(
			$ml->div(undef, { id => "loading", class => "loader" }) .
			$ml->div(undef, { id => "slideshow", class => "slideshow" }),
			{ class => "slideshow-container" }
		) .
		$ml->div(undef, { id => "caption", class => "caption-container" }),
		{ id => "gallery", class => "content" }
	);
	
	my $pics;
	foreach my $pic (@pics) {
		my $c = new ExSite::Content(content=>$pic);	    
		$c->set_context("child");
		next unless ($c->content_type() =~ /^image\/(jpe?g|gif|png)$/);
		
		my $url = $c->get_url;
		my $thumb = $c->get_thumb_html;
		
		my $t_width = $opt->{thumbnail_size} || 75;
		my $t_height = $opt->{thumbnail_size} || 75;	    	
		
		my $thumb_url = $c->get_thumb_url;
		$thumb = <<END;
<img src="$thumb_url" width="$t_width" height="$t_height" border="0">
END
		
		my $title = $ml->div($c->get("content","name"), { class => "image-title" }) 
			if ($jgallery_config{galleriffic_display_image_titles} eq "true");
		my $caption = $ml->div($c->get("content","description"), { class => "image-desc" }) 
			if ($jgallery_config{galleriffic_display_image_captions} eq "true");
		
		my $attr = $c->get("content_data","attributes");
		$pics .= $ml->li(
			$ml->a($thumb, { class => "thumb", href => $url }) .
			$ml->div(
				$title .
				$caption,
				{ class => "caption" }
			)
		);
	}
	
	$out .= $ml->div(
		$ml->ul($pics, { class => "thumbs noscript" }),
		{ id => "thumbs", class => "navigation" }
	);

	my $file_path = $config{server}{HTMLpath} . "/_Modules/JGallery";
	my $delay = $jgallery_config{galleriffic_delay} * 1000;
	my $transition_duration = $jgallery_config{galleriffic_transition_duration} * 1000;
	
	my $default_play_button = $ml->img(undef, { src => "$file_path/galleriffic/galleriffic_play.jpg" }) . 
		$ml->span("Play", { style => "font-size: 8pt; display: block;" });
	my $play_button = $config{JGallery}{play_button} || $default_play_button;
	
	my $default_pause_button = $ml->img(undef, { src => "$file_path/galleriffic/galleriffic_pause.jpg" }) . 
		$ml->span("Pause", { style => "font-size: 8pt; display: block;" });
	my $pause_button = $config{JGallery}{pause_button} || $default_pause_button;
	
	my $default_next_button = $ml->img(undef, {src => "$file_path/galleriffic/galleriffic_next.jpg"}) .
		$ml->span("Next", { style => "font-size: 8pt; display: block;" });
	my $next_button = $config{JGallery}{next_button} || $default_next_button;
	
	my $default_previous_button = $ml->img(undef, {src => "$file_path/galleriffic/galleriffic_previous.jpg"}) .
		$ml->span("Prev", { style => "font-size: 8pt; display: block;" });
	my $previous_button = $config{JGallery}{previous_button} || $default_previous_button;
	
	$out .= <<END;
<script type="text/javascript" src="$file_path/galleriffic/jquery.galleriffic.js"></script>
<script type="text/javascript" src="$file_path/jquery.opacityrollover.js"></script>
<link rel="stylesheet" href="$file_path/galleriffic/galleriffic.css" type="text/css" />
<script type="text/javascript">
	jQuery(document).ready(function(\$) {
		\$('div.navigation').css({'width' : '100%', 'float' : 'left'});
		\$('div.content').css('display', 'block');

		var onMouseOutOpacity = 0.67;
		\$('#thumbs ul.thumbs li').opacityrollover({
			mouseOutOpacity:   onMouseOutOpacity,
			mouseOverOpacity:  1.0,
			fadeSpeed:         'fast',
			exemptionSelector: '.selected'
		});

		var gallery = \$('#thumbs').galleriffic({
			delay:                     $delay,
			numThumbs:                 $jgallery_config{galleriffic_num_thumbs},
			preloadAhead:              8,
			enableTopPager:            $jgallery_config{galleriffic_enable_top_pager},
			enableBottomPager:         $jgallery_config{galleriffic_enable_bottom_pager},
			maxPagesToShow:            $jgallery_config{galleriffic_max_pages},
			imageContainerSel:         '#slideshow',
			controlsContainerSel:      '#controls',
			captionContainerSel:       '#caption',
			loadingContainerSel:       '#loading',
			renderSSControls:          $jgallery_config{galleriffic_render_ss_controls},
			renderNavControls:         $jgallery_config{galleriffic_render_nav_controls},
			playLinkText:              '$play_button',
			pauseLinkText:             '$pause_button',
			prevLinkText:              '$previous_button',
			nextLinkText:              '$next_button',
			nextPageLinkText:          'Next &rsaquo;',
			prevPageLinkText:          '&lsaquo; Prev',
			enableHistory:             false,
			autoStart:                 $jgallery_config{galleriffic_auto_start},
			syncTransitions:           false,
			defaultTransitionDuration: $transition_duration
		});
	});
</script>
END

	return $out;
}

sub set_thumbnail_dimension{
	my ($this,$t_width,$t_height,$dim) = @_;
	return ($t_width,$t_height) if !$dim;
	if ($dim =~ /,/){
		$dim =~ s/\s//g;
		($t_width,$t_height) = split(/,/,$dim,2);
	}else{
		if ($t_width != $t_height){
			my $ratio;
			if ($t_width > $t_height){
				$ratio = $t_width / $dim;
				$t_width = $dim;
				$t_height = sprintf("%i",($t_height / $ratio));
			}else{
				$ratio = $t_height / $dim;
				$t_height = $dim;
				$t_width = sprintf("%i",($t_width / $ratio));
			}
		}else{
			$t_width = $dim;
			$t_height = $dim;
		}
	}
	return ($t_width,$t_height);
}

sub wrap {
	my ($this,$body) = @_;
	
	my $out .= $ml->link(undef, {
		rel => "stylesheet",
		type => "text/css",
		href => "$config{server}{server}$config{server}{HTMLpath}/_Modules/JGallery/JGallery.css",
	});
	$out.= $body;
	
	return $ml->div($out, { class => "JGallery" } );
}


1;

