package Modules::QA;

#----------------------------------------------------------------------------
#
#   Copyright (C) 2009 - Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
#  This modules processes user-configurable forms.
#
#----------------------------------------------------------------------------

use strict;

use ExSite::Config;
use ExSite::Misc;
use ExSite::ML;
use ExSite::URI;
use ExSite::Util;
use ExSite::Crypt;
use ExSite::Object;
use ExSite::ObjectList;
use Modules::BaseDCD;
use Modules::QA::Form;
use Modules::QA::Library;

use vars qw(@ISA $ml $QA $maxrec);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$options) = @_;
    $ml = &get_obj("ML");
    my %opt = &DecodeString($options);
    my $in = new ExSite::Input;
    $in = $in->combine;
    my $key_id;
    if ($in->{'_qakey'}) {
	my $c = new ExSite::Crypt;
	($key_id,undef) = split /;/, $c->decrypt($in->{'_qakey'});
    }
    my $form_id = $opt{form} || $key_id;
    my $action = $this->link();
    if ($form_id) {
	my $form = new Modules::QA::Form(id=>$form_id);
	return $form->show(action=>$action);
    }
    else {
	return $this->error("No form to display!");
    }
}

# ioctl method (used by ExSite to query the module for its functionality)

sub ioctl {
    my $this = shift;
    $_ = shift;           # $_ is the ioctl request

    if (/isRestricted/) {
	return 0;
    }
    elsif (/isService/) {
	return 1;
    }
    elsif (/ModuleName/) {
	return "Web Forms";
    }
    elsif (/ModuleInfo/) {
	return "This modules processes user-configurable forms.";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/DynContentParam/) {
	$ml = &get_obj("ML");
	my $forms = $this->my_forms();
	if ($forms->count) {
	    my $out .= $ml->strong("Forms:");
	    my $option;
	    while (my $f = $forms->next) {
		$option .= $ml->option($f->name,{value=>"form=".$f->id});
	    }
	    $out .= $ml->select($option,{name=>"param"});
	    return $out;
	}
	return undef;  # no forms; use blank parameter field
    }
    elsif (/ToDo/) {
	return \&todo;
    }
}

# ToDo : find all untranslated questions

sub todo {
    my $this = shift;
    my @version = split /\|/, $share{DB}{map}->regexp("list:version");
    my @task;
    if (@version > 0) {
	
################################################

    }
    return @task;
}

# Control Panel : browse and export responses

sub ctrl_panel {
    my $this = shift;
    $this->{section_id} = $this->get_section_id();
    if (! $this->{section_id} && $config{QA}{admin_scope} ne "global") {
	return $this->set_section_id();
    }
    $this->setup_querylib();

    $ml = &get_obj("ML");
    my $out = $this->show_location();
    my $db = $share{DB};
    $db->handler("show_foreign_key",\&show_foreign_key);

    my $cmd = $this->{input}{qacmd};
    if ($this->{input}{response}) {
	if ($cmd eq "edit_response") {
	    $out .= $this->edit_response($this->{input}{form},$this->{input}{response});
	}
	else {
	    $out .= $this->show_response($this->{input}{response});
	}
    }
    elsif ($this->{input}{lib}) {
	if ($cmd eq "newq") {
	    $out .= $this->new_base_question();
	}
	elsif ($this->{input}{qb}) {
	    if ($cmd eq "trans") {
		$out .= $this->translate();
	    }
	    else {
		$out .= $this->base_question();
	    }
	}
	else {
	    $out .= $this->show_library();
	}
    }
    elsif ($this->{input}{form}) {
	if ($cmd eq "export") {
	    $out .= $this->export_responses($this->{input}{form});
	}
	elsif ($cmd eq "preview") {
	    $out .= $this->preview($this->{input}{form});
	}
	elsif ($cmd eq "responses") {
	    $out .= $this->show_responses($this->{input}{form});
	}
	elsif ($cmd eq "summary") {
	    $out .= $this->show_summary($this->{input}{form});
	}
	elsif ($cmd eq "newq") {
	    $out .= $this->add_question();
	}
	elsif ($cmd eq "copy") {
	    $out .= $this->copy_form($this->{input}{form});
	}
	elsif ($cmd =~ /^up|down$/) {
	    $out .= $this->reorder();
	}
	elsif ($this->{input}{q}) {
	    $out .= $this->question();
	}
	else {
	    $out .= $this->show_form($this->{input}{form});
	}
    }
    else {
	if ($cmd eq "newform") {
	    $out .= $this->new_form();
	}
	elsif ($cmd eq "newlib") {
	    $out .= $this->new_lib();
	}
	else {
	    my $forms = $this->my_forms();
	    my $r = new ExSite::ReportBuilder(title=>"Forms");
	    $r->headers(["Form","Title","Responses","Section","Reports"]);
	    while (my $f = $forms->next) {

		my $n_response = $f->count_responses();
		my $opt;
		if ($n_response) {
		    $opt .= 
			$ml->a("stats",{class=>"button_sm",href=>$this->link(qacmd=>"summary",form=>$f->id)}).
			$ml->a("responses",{class=>"button_sm",href=>$this->link(qacmd=>"responses",form=>$f->id)}).
			$ml->a("export",{class=>"button_sm",href=>$this->link(qacmd=>"export",form=>$f->id)});
		}
		$r->push($f->id,
			 $ml->a($f->name,{href=>$this->link(form=>$f->id())}),
			 $n_response,
			 $f->showdata("section_id"),
			 $opt)
	    }

	    $r->tools($ml->a("+ new form",{href=>$this->link(form=>undef,qacmd=>"newform"),class=>"button_sm"}));
	    #$r->foot("Forms are the different ways you actually present real questions to your users.");
	    $out .= $r->make();
	    my $libs = $this->my_libs();
	    $r->init(title=>"Question Libraries");
	    $r->headers(["Library","Title","Questions","Section"]);
	    while (my $lib = $libs->next) {
		$r->push($lib->id,
			 $ml->a($lib->name,{href=>$this->link(lib=>$lib->id())}),
			 $lib->count_questions(),
			 $lib->showdata("section_id"));
	    }
	    $r->tools($ml->a("+ new library",{href=>$this->link(form=>undef,qacmd=>"newlib"),class=>"button_sm"}));
	    #$r->foot("Question libraries are where you create and manage the questions that you will use in your forms.");
	    $out .= $r->make();
	}
    }
    return $out;
}

sub show_location {
    my $this = shift;
    my @path = $ml->a("Top",{href=>$this->link(section_id=>undef,lib=>undef,form=>undef,qacmd=>undef,qb=>undef,tr=>undef,response=>undef)});
    if ($this->{section_id}) {
	my $s = new ExSite::Section(id=>$this->{section_id});
	push @path, $ml->a($s->get_my("title"),{href=>$this->link(form=>undef,lib=>undef,qacmd=>undef,q=>undef,qb=>undef,tr=>undef,response=>undef)});
    }
    if ($this->{input}{form}) {
	$this->{form} = new Modules::QA::Form(id=>$this->{input}{form});
	push @path, $ml->a("Form: ".$this->{form}->name,{href=>$this->link(lib=>undef,qacmd=>undef,q=>undef,qb=>undef,tr=>undef,response=>undef)});
	if ($this->{input}{q}) {
	    $this->{question} = new Modules::QA::Question(id=>$this->{input}{q});
	    push @path, $ml->a($this->{question}->label,{href=>$this->link(qacmd=>undef,lib=>undef,qb=>undef,tr=>undef,response=>undef)});
	}
    }
    if ($this->{input}{lib}) {
	$this->{lib} = new Modules::QA::Library(id=>$this->{input}{lib});
	push @path, $ml->a("Library: ".$this->{lib}->name,{href=>$this->link(form=>undef,qacmd=>undef,q=>undef,qb=>undef,tr=>undef,response=>undef)});
    }
    if ($this->{input}{qb}) {
	$this->{qb} = new Modules::QA::Base(id=>$this->{input}{qb});
	push @path, $ml->a($this->{qb}->label,{href=>$this->link(qacmd=>undef,q=>undef,tr=>undef,response=>undef)});
	if ($this->{input}{tr}) {
	    $this->{tr} = new ExSite::Object(type=>"qa_translation",id=>$this->{input}{tr});
	    push @path, $ml->a($this->{tr}->showdata("version"),{href=>$this->link(qacmd=>undef,response=>undef)});
	}
    }
    return $ml->p(join("&nbsp;&gt;&nbsp;",@path));
}

sub my_forms {
    my $this = shift;
    my $db = $share{DB};
    my @form;
    if ($config{QA}{admin_scope} eq "global") {
	# show all forms
	push @form, $db->fetch_all("qa_form");
    }
    else {
	# local scope; show only forms for selected section
	if ($db->level > 2) { 
	    push @form, $db->fetch_match("qa_form",{section_id=>0});
	}
	my $site = $db->this_site();
	if ($site) { 
	    push @form, $db->fetch_match("qa_form",{section_id=>$site->{section_id}});
	}
    }
    my $forms = new ExSite::ObjectList(type=>"qa_form",list=>\@form);
    $forms->sort("name");
    return $forms;   # ObjectList
}

sub my_libs {
    my $this = shift;
    my $db = $share{DB};
    my $sid = $this->get_section_id();
    my @lib;
    if ($config{QA}{admin_scope} eq "global") {
	# show all libs
	push @lib, $db->fetch_all("qa_lib");
    }
    else {
	# local scope; show only forms for selected section
	if ($db->level > 2) {
	    push @lib, $db->fetch_match("qa_lib",{section_id=>0});
	}
	if ($sid) {
	    push @lib, $db->fetch_match("qa_lib",{section_id=>$sid});
	}
    }
    my $libs = new ExSite::ObjectList(type=>"qa_lib",list=>\@lib);
    $libs->sort("name");
    return $libs;   # ObjectList
}

sub show_library {
    my $this = shift;
    my $lib = new Modules::QA::Library(id=>$this->{input}{lib});
    if ($this->{input}{qacmd} eq "del") {
	$lib->delete();
	$ml->redirect($this->link(qacmd=>undef,lib=>undef,__plaintext=>1));
	return;
    }
    my $url = new ExSite::URI;
    my $newurl = new ExSite::URI;
    $newurl->parameter(qacmd=>"newq");
    my $out .= $lib->show(url=>$url,newurl=>$newurl);
    if ($lib->count_questions() == 0) {
	$out .= &ExSite::HTML::Button(label=>"DELETE library",url=>$this->link(qacmd=>"del"),type=>"button");
    }
    return $out;
}

sub show_form {
    my ($this,$formid) = @_;
    my $out;
    my $db = $share{DB};
    my $f = new Modules::QA::Form(id=>$formid);
    if ($this->{input}{qacmd} eq "del") {
	$f->delete();
	$ml->redirect($this->link(qacmd=>undef,form=>undef,__plaintext=>1));
	return;
    }

    # configuration
    
    my ($confout,$confstate);
    my $in = new ExSite::Input;
    my $post = $in->post;
    if (scalar keys %$post > 0) {
	if (! $post->{_qakey}) {
	    # config form was submitted
	    $confout .= $f->do_edit();
	    $confstate = 1;
	}
    }
    $confout .= $f->edit(action=>$this->link());
    my $show_form = $f->show();

    # form details

    my $n_response = $f->count_responses;
    my $response;
    if ($n_response) {
	$response .= $ml->div(
	    $ml->a("stats",{class=>"button_sm",href=>$this->link(qacmd=>"summary")})." ".
	    $ml->a("responses",{class=>"button_sm",href=>$this->link(qacmd=>"responses")})." ".
	    $ml->a("export",{class=>"button_sm",href=>$this->link(qacmd=>"export")}),
			      { class=>"ResponseButtonBox" }
			      ).
				  $n_response;
    }

    my $rpt = new ExSite::ReportBuilder(title=>$f->name());
    $rpt->headers(["Form #","Questions","Responses","Status","Last Modified"]);
    $rpt->data(
	       $f->id(),
	       $f->count_questions,
	       $response,
	       $f->showdata("status"),
	       $f->showdata("mtime")
	       );
    $rpt->tools($ml->a("+ copy form",{class=>"button_sm",class=>"button_sm",href=>$this->link(qacmd=>"copy")}));

    # questions

    my $rpt2 = new ExSite::ReportBuilder(title=>"Questions");
    $rpt2->nodata("No questions defined.");
    $rpt2->headers(["Question ID","Question","Format","Options","Sort","Last Modified"]);
    my $qlist = $f->questions();
    if ($qlist) {
	$qlist->reset();
	while (my $q = $qlist->next()) {
	    $rpt2->push(
			$q->id(),
			$ml->a($q->label(),{href=>$this->link(q=>$q->id)}),
			$q->format(),
			$q->showdata("flags"),
			$ml->a("&#9650",{href=>$this->link(q=>$q->id,qacmd=>"up")})." ".$ml->a("&#9660",{href=>$this->link(q=>$q->id,qacmd=>"down")}),
			$q->showdata("mtime")
		       );
	}
    }
    $rpt2->tools($ml->a("+ add question",{class=>"button_sm",href=>$this->link(qacmd=>"newq")}));

    my $delete = $n_response > 0 ?
	$ml->p("WARNING: deleting this form will also discard the $n_response responses that have been received so far.") :
	$ml->p("No responses have been received on this form; it is safe to delete.");
    $delete .= &ExSite::HTML::Button(label=>"DELETE form",url=>$this->link(qacmd=>"del"),type=>"button");

    return &ExSite::HTML::AccordionBox(
				       titles=>["Overview","Questions","Preview","Configure","Delete"],
				       panes=>[
					       $rpt->make,
					       $rpt2->make,
					       $show_form,
					       $confout,
					       $delete,
					       ],
				       states=>[ 1,1,0,$confstate,0 ]
				       );
}

sub edit_form {
    my $this = shift;
    my $in = new ExSite::Input;
    my $post = $in->post();
    my $query = $in->query();
    my $out;
    if (scalar keys %$post > 0) {
	if ($post->{submit_action} eq "Cancel") {
	    $ml->location($this->link(qb=>undef,__plaintext=>1));
	    return undef;
	}
	my $db = $share{DB};
	$db->get_data();
	$db->get_action();
	my @err = $db->validate();
	if (@err > 0) {
	    $out = $ml->h3("Problem:");
	    $out .= $this->error(@err);
	    $out .= $ml->p($ml->a("&lt; Back",{href=>"javascript:history.back()"}));
	}
	else {
	    my $fid = $this->{form}->id();
	    my $stat = $db->update("qa_form",$db->{form}{qa_form}{$fid});
	    if ($stat) {
		$out = $ml->h3("Problem:");
		$out .= $this->show_diagnostics("error","html");
		$out .= $ml->p($ml->a("&lt; Back",{href=>"javascript:history.back()"}));
	    }
	    else {
		$ml->location($this->link(qacmd=>undef,q=>undef,qb=>undef,__plaintext=>1));
	    }
	}
    }
    else {
	my $f = $this->{form};
	$out .= $ml->h1("Configure Form");
	$out .= $f->edit( action=>$this->link() );
    }

    return $out;
}

sub copy_form {
    my ($this,$formid) = @_;
    if ($formid) {
	my $f = new Modules::QA::Form(id=>$formid);
	my $newformid = $f->copy;
	if ($newformid) {
	    my $url = $this->link(qacmd=>undef,form=>$newformid,__plaintext=>1);
	    $ml->location($url);
	    return $ml->p("Form copied. ".$ml->a("Click here to continue.",{href=>$url}));
	}
	else {
	    return &ExSite::Config::show_diagnostics();
	}
    }
    return $this->error("cannot copy a non-existent form");
}

sub new_form {
    my ($this) = @_;
    my $in = new ExSite::Input;
    my $post = $in->post();
    my $out;
    my $f = new Modules::QA::Form();
    if (scalar keys %$post > 0) {
	if ($post->{submit_action} eq "Cancel") {
	    $ml->location($this->link(qacmd=>undef,__plaintext=>1));
	    return undef;
	}
	my $db = $share{DB};
	$db->get_data();
	$db->get_action();
	my @err = $db->validate();
	if (@err > 0) {
	    $out = $ml->h3("Problem:");
	    $out .= $this->error(@err);
	    $out .= $ml->p($ml->a("Go back to form.",{href=>"javascript:history.back()"}));
	}
	else {
	    my $qaformid = $db->insert("qa_form",$db->{form}{qa_form}{_});
	    if ($qaformid) {
		$ml->location($this->link(form=>$qaformid,lib=>undef,qacmd=>undef,__plaintext=>1));
	    }
	    else {
		$out = $ml->h3("Problem:");
		$out .= $this->show_diagnostics("error","html");
		$out .= $ml->p($ml->a("&lt Back",{href=>"javascript:history.back()"}));
	    }
	}
    }
    else {
	$out .= $ml->h1("New Form");
	$out .= $ml->p("The creation of a web form proceeds in 3 steps:");
	$out .= $ml->ol([
			 "Create the form.",
			 "Pick questions from your question library to add to this form. (You may need to create your questions if you don't already have them available.)",
			 "Publish the form somewhere on your website."
			 ]);
	$out .= $ml->p("Note that this screen addresses step 1 only. One you have created the basic form, you can start adding questions to it.");
	my $f = new Modules::QA::Form(action=>$this->link());
	$out .= $f->make(action=>$this->link(),data=>{section_id=>$this->{section_id}},hide=>["section_id"]);
    }
}

sub reorder {
    my $this = shift;
    my $f = $this->{form};
    my $qid = $this->{question}->id;
    my $dir = $this->{input}{qacmd};

    # get the current sort order
    my @qsort;
    my $questions = $f->questions();
    $questions->reset();
    my $qpos;
    while (my $q = $questions->next()) {
	push @qsort, $q;
	if ($q->id == $qid) { $qpos = $#qsort; }
    }

    # reorder
    if ($dir eq "down") {
	if ($qpos < $#qsort) {
	    my $qtmp = $qsort[$qpos + 1];
	    $qsort[$qpos + 1] = $qsort[$qpos];
	    $qsort[$qpos] = $qtmp;
	}
    }
    else { #if ($dir eq "up") {
	if ($qpos > 0) {
	    my $qtmp = $qsort[$qpos - 1];
	    $qsort[$qpos - 1] = $qsort[$qpos];
	    $qsort[$qpos] = $qtmp;
	}
    }

    # adjust sortkeys to preserve new order
    
    my $qprev;
    foreach my $q (@qsort) {
	if ($qprev) {
	    my $sort = $q->getdata("sortkey");
	    my $prevsort = $qprev->getdata("sortkey");
	    my $qid = $q->id;
	    my $qprevid = $qprev->id;
	    if ($sort lt $prevsort || 
		($sort eq $prevsort && $qid < $qprevid)) {
		# incorrect sort order
		my $newsortkey = ($qid > $qprevid) ? $prevsort : $prevsort + 1;
		$q->setdata("sortkey", $newsortkey);
		$q->save();
	    }
	}
	$qprev = $q;
    }

    # refresh form view
    $ml->location($this->link(qacmd=>undef,q=>undef,__plaintext=>1));
}

sub new_lib {
    my ($this) = @_;
    my $in = new ExSite::Input;
    my $post = $in->post();
    my $out;
    my $f = new Modules::QA::Library();
    if (scalar keys %$post > 0) {
	if ($post->{submit_action} eq "Cancel") {
	    $ml->location($this->link(qacmd=>undef,__plaintext=>1));
	    return undef;
	}
	my $db = $share{DB};
	$db->get_data();
	$db->get_action();
	my @err = $db->validate();
	if (@err > 0) {
	    $out = $ml->h3("Problem:");
	    $out .= $this->error(@err);
	    $out .= $ml->p($ml->a("Go back to form.",{href=>"javascript:history.back()"}));
	}
	else {
	    my $qalibid = $db->insert("qa_lib",$db->{form}{qa_lib}{_});
	    if ($qalibid) {
		$ml->location($this->link(lib=>$qalibid,form=>undef,qacmd=>undef,__plaintext=>1));
	    }
	    else {
		$out = $ml->h3("Problem:");
		$out .= $this->show_diagnostics("error","html");
		$out .= $ml->p($ml->a("&lt; Back",{href=>"javascript:history.back()"}));
	    }
	}
    }
    else {
	$out .= $ml->h1("New Question Library");
	my $f = new Modules::QA::Library(action=>$this->link());
	$out .= $f->make(action=>$this->link(),data=>{section_id=>$this->{section_id}},hide=>["section_id"]);
    }
}

sub new_base_question {
    my $this = shift;
    my $in = new ExSite::Input;
    my $post = $in->post();
    my $out;
    if (scalar keys %$post > 0) {
	if ($post->{submit_action} eq "Cancel") {
	    $ml->location($this->link(qacmd=>undef,__plaintext=>1));
	    return undef;
	}
	my $db = $share{DB};
	$db->get_data();
	$db->get_action();
	my @err = $db->validate();
	if (@err > 0) {
	    $out = $ml->h3("Problem:");
	    $out .= $this->error(@err);
	    $out .= $ml->p($ml->a("&lt; Back",{href=>"javascript:history.back()"}));
	}
	else {
	    my $qbid = $db->insert("qa_base",$db->{form}{qa_base}{_});
	    if ($qbid) {
		if ($this->{input}{form}) {
		    # hopefully this sends the user back to form mgmt
		    $ml->location($this->link(lib=>undef,__plaintext=>1));
		}
		else {
		    $ml->location($this->link(qacmd=>undef,__plaintext=>1));
		}
	    }
	    else {
		$out = $ml->h3("Problem:");
		$out .= $this->show_diagnostics("error","html");
		$out .= $ml->p($ml->a("&lt; Back",{href=>"javascript:history.back()"}));
	    }
	}
    }
    else {
	$out = $ml->h1("Create new question");
	my $lib = new Modules::QA::Base();
	$out .= $lib->make(action=>$this->link(),data=>{qa_lib_id=>$this->{input}{lib}});
    }
    return $out;
}

sub base_question {
    my $this = shift;
    my $db = $share{DB};
    my $in = new ExSite::Input;
    my $post = $in->post();
    my $out;
    if ($this->{input}{qacmd} eq "del") {
	$this->{qb}->delete();
	$ml->location($this->link(qb=>undef,qacmd=>undef,__plaintext=>1));
    }
    if (scalar keys %$post > 0) {
	if ($post->{submit_action} eq "Cancel") {
	    $ml->location($this->link(qacmd=>undef,__plaintext=>1));
	    return undef;
	}
	$db->get_data();
	$db->get_action();
	my @err = $db->validate();
	if (@err > 0) {
	    $out = $ml->h3("Problem:");
	    $out .= $this->error(@err);
	    $out .= $ml->p($ml->a("&lt; Back",{href=>"javascript:history.back()"}));
	}
	else {
	    my $qbid = $this->{qb}->id;
	    my $stat = $db->update("qa_base",$db->{form}{qa_base}{$qbid});
	    if ($stat) {
		$out = $ml->h3("Problem:");
		$out .= $this->show_diagnostics("error","html");
		$out .= $ml->p($ml->a("&lt; Back",{href=>"javascript:history.back()"}));
	    }
	    else {
		$ml->location($this->link(qacmd=>undef,__plaintext=>1));
	    }
	}
    }
    else {
	my $qb = $this->{qb};
	my $del;
	my @f = $db->get_query("forms using base question",$qb->id);
	if (@f > 0) {
	    $del .= $ml->p("This question is currently being used by the following forms:");
	    my $forms;
	    foreach my $f (@f) {
		my $f = new Modules::QA::Form(data=>$f);
		$forms .= $ml->li(
				  $ml->a($f->name,{href=>$this->link(qacmd=>undef,lib=>undef,qb=>undef,form=>$f->id)})
				  );
	    }
	    $del .= $ml->ul($forms);
	    $del .= $ml->p("Deleting this question may damage those forms and/or the responses they have already collected.");
	}
	else {
	    $del .= $ml->p("This question is not in use at this time, and is safe to delete.");
	}
	$del .= &ExSite::HTML::Button(label=>"DELETE this question",
					url=>$this->link(qacmd=>"del"),
					type=>"button");
	$out .= &ExSite::HTML::AccordionBox(
					    titles=>["Preview","Details","Translations","Edit","Delete"],
					    panes=>[
						    $qb->show(),
						    $qb->report(),
						    $this->show_translations(),
						    $qb->edit(action=>$this->link()),
						    $del
						    ],
					    states=>[1,1,0,0,0]
					    );
    }
    return $out;
}

sub show_translations {
    my $this = shift;
    my $r = new ExSite::ReportBuilder(title=>"Translations");
    $r->headers(["Version","Translated Question","Translated Answers"]);
    my @version = split(/\|/,$share{DB}{map}->regexp("list:version"));
    my $qb = $this->{qb};
    foreach my $v (@version) {
	my $tr = $qb->translation($v);
	my $label = $tr->{label} || "[missing]";
	my $value = $tr->{value} || ($qb->getdata("value") ? "[missing]" : "[n/a]");
	my $url = $this->link(qacmd=>"trans",tr=>$qb->{translation}{$v}{qa_translation_id});
	$r->push(
		 $v,
		 $ml->a($label,{href=>$url}),
		 $ml->a($value,{href=>$url}),
		 );
    }
    $r->foot("Click on the question or answer to update the translation.");
    return $r->make();
}

sub translate {
    my $this = shift;
    my $db = $share{DB};
    my $in = new ExSite::Input;
    my $post = $in->post();
    my $out;
    if (scalar keys %$post > 0) {
	if ($post->{version}) {
	    my $old_tr = $this->{qb}->translation($post->{version});
	    if ($old_tr) {
		$old_tr->{label} = $post->{label};
		$old_tr->{value} = $post->{value};
		my $stat = $db->update("qa_translation",$old_tr);
	    }
	    else {
		my $qbid = $this->{qb}->id;
		my %tr = (
			  label=>$post->{label},
			  value=>$post->{value},
			  version=>$post->{version},
			  qa_base_id=>$qbid,
			  );
		my $trid = $db->insert("qa_translation",\%tr);
	    }
	    $ml->location($this->link(qacmd=>undef,__plaintext=>1));
	}
    }
    else {
	# no translation - make one
	$out .= $ml->h1("$config{default_version} Version");
	my $qdata = $ml->tr(
			    $ml->th("Question:").
			    $ml->td(
				    $ml->textarea($this->{qb}->label,{cols=>80,rows=>4,readonly=>undef})
				    )
			    );
	my $ans = $this->{qb}->showdata("value");
	if ($ans) {
	    $qdata .= $ml->tr(
			      $ml->th("Answers:").
			      $ml->td(
				      $ml->textarea($ans,{cols=>80,rows=>4,readonly=>undef})
				      )
			      );
	}
	$out .= $ml->table($qdata);
    }
    my @version = split /\|/, $share{DB}{map}->regexp("list:version");
    foreach my $version (@version) {
	$out .= $this->translate_version($version);
    }
    
    return $out;
}

sub translate_version {
    my ($this,$version) = @_;
    my $out .= $ml->h1($ml->em($version)." Version");
    $this->{qb}->set("version",$version);
    $this->{qb}->load_translations();
    my $qdata = $ml->tr(
			$ml->th("Question:").
			$ml->td(
				$ml->textarea($this->{qb}->label,{name=>"label",cols=>80,rows=>4})
				)
			);
    my $ans = $this->{qb}->showdata("value");
    if ($ans) {
	$qdata .= $ml->tr(
			  $ml->th("Answers:").
			  $ml->td(
				  $ml->textarea($ans,{name=>"value",cols=>80,rows=>4})
				  )
			  );
    }
    $qdata .= $ml->tr(
		      $ml->td().
		      $ml->td(
			      $ml->input(undef,{type=>"submit",value=>"Update $version version"})
			      )
		      );
    $out .= $ml->form(
		      $ml->table($qdata).
		      $ml->input(undef,{type=>"hidden",name=>"version",value=>$version}),
		      {method=>"post",action=>$this->link()}
		      );
    return $out;
}

sub add_question {
    my $this = shift;
    my $in = new ExSite::Input;
    my $post = $in->post();
    my $query = $in->query();
    my $out;
    if (scalar keys %$post > 0) {
	if ($post->{submit_action} eq "Cancel") {
	    $ml->location($this->link(qb=>undef,q=>undef,__plaintext=>1));
	    return undef;
	}
	my $db = $share{DB};
	$db->get_data();
	$db->get_action();
	my @err = $db->validate();
	if (@err > 0) {
	    $out = $ml->h3("Problem:");
	    $out .= $this->error(@err);
	    $out .= $ml->p($ml->a("&lt; Back",{href=>"javascript:history.back()"}));
	}
	else {
	    my $stat = $db->insert("qa_question",$db->{form}{qa_question}{_});
	    if ($stat) {
		$ml->location($this->link(qacmd=>undef,qb=>undef,q=>undef,__plaintext=>1));
	    }
	    else {
		$out = $ml->h3("Problem:");
		$out .= $this->show_diagnostics("error","html");
		$out .= $ml->p($ml->a("&lt; Back",{href=>"javascript:history.back()"}));
	    }
	}
    }
    elsif ($query->{qb} && $query->{form}) {
	my $q = new Modules::QA::Question();
	$out .= $ml->h1("New Question");
	$out .= $ml->p("Adding ".
		       $ml->em($this->{qb}->label).
		       " to ".
		       $ml->em($this->{form}->name));
	$out .= $ml->h2("Advanced Options");
	$out .= $q->make(
			 data=>{
			     qa_base_id=>$this->{qb}->id,
			     qa_form_id=>$this->{form}->id,
			 },
			 hide=>[
				"qa_form_id", "qa_base_id",
				],
			 action=>$this->link(),
			 );
    }
    else {
	$out .= $ml->h1("Select question");
	$out .= $ml->p("Click on a question to add it to your form:");
	my $libs = $this->my_libs();
	while (my $lib = $libs->next()) {
	    my $questions = $lib->questions();
	    my $qlist;
	    if ($questions) {
		while (my $q = $questions->next()) {
		    $qlist .= $ml->li(
				      $ml->a(
					     $q->label(),
					     {href=>$this->link(qb=>$q->id)}
					     ).
				      #&ExSite::HTML::Popup(label=>"preview",pane=>$q->show()).
				      $ml->br
				      );
		}
		$qlist .= $ml->li($ml->a("+ add a new question to ".$ml->em($lib->name),{href=>$this->link(lib=>$lib->id)}));
	    }
	    if ($qlist) {
		$out .= 
		    $ml->h2($lib->name).
		    $ml->ul($qlist);
	    }
	}
    }

    return $out;
}

sub question {
    my $this = shift;
    my $in = new ExSite::Input;
    my $post = $in->post();
    my $query = $in->query();
    my $out;
    if ($this->{input}{qacmd} eq "del") {
	$this->{question}->delete();
	$ml->location($this->link(qb=>undef,tr=>undef,q=>undef,qacmd=>undef,__plaintext=>1));
    }
    if (scalar keys %$post > 0) {
	if ($post->{submit_action} eq "Cancel") {
	    $ml->location($this->link(qb=>undef,__plaintext=>1));
	    return undef;
	}
	my $db = $share{DB};
	$db->get_data();
	$db->get_action();
	my @err = $db->validate();
	if (@err > 0) {
	    $out = $ml->h3("Problem:");
	    $out .= $this->error(@err);
	    $out .= $ml->p($ml->a("&lt; Back",{href=>"javascript:history.back()"}));
	}
	else {
	    my $qid = $this->{question}->id();
	    my $stat = $db->update("qa_question",$db->{form}{qa_question}{$qid});
	    if ($stat) {
		$out = $ml->h3("Problem:");
		$out .= $this->show_diagnostics("error","html");
		$out .= $ml->p($ml->a("&lt; Back",{href=>"javascript:history.back()"}));
	    }
	    else {
		$ml->location($this->link(qacmd=>undef,q=>undef,qb=>undef,__plaintext=>1));
	    }
	}
    }
    else {
	my $q = $this->{question};
	$out .= &ExSite::HTML::FrameBox(title=>"Preview",pane=>$q->show(),bgcolor=>"#eee",color=>"#999");
	$out .= $ml->p(
		       &ExSite::HTML::Button(label=>"REMOVE this question from this form",
					     url=>$this->link(qacmd=>"del"),
					     type=>"button")
		       );
	$out .= $ml->h1("Configure Question");
	$out .= $q->edit(hide=>["qa_form_id"],
			 action=>$this->link() );
	$out .= $ml->p("These settings affect the question in this form only.".
		       $ml->br."To reconfigure the generic question, ".
		       $ml->a("go here",{href=>$this->link(qb=>$this->{question}->base->id,lib=>$this->{question}->base->getdata("qa_lib_id"))}).
		       ".");
    }

    return $out;
}

sub preview {
    my ($this,$formid) = @_;
    my $f = new Modules::QA::Form(id=>$formid);
    return $f->show();
}

sub get_responses {
    my ($this,$form) = @_;
    my $responses = new ExSite::ObjectList(type=>"qa_response");
    $responses->load({qa_form_id=>$form->id},"ctime desc");
    return $responses;
}

sub get_response_report {
    my ($this,$form,$options) = @_;
    my $responses = $this->get_responses($form);
    my $rpt = new ExSite::ReportBuilder(title=>"Responses - ".$form->name);
    $rpt->set("dynamic",1);
    my @head = $options ? ('Options') : ();
    if ($config{QA}{report_form_name}) {
	push @head, 'Form';
    }
    push @head, qw(Response Date Member Version);
    push @head, $form->question_labels();
    $rpt->headers(\@head);
    $rpt->tools("Export: ".
		$ml->a("excel",{href=>$this->link(qacmd=>"export")})." ".
		$ml->a("csv",{href=>$this->link(qacmd=>"export",format=>"csv")})." ".
		$ml->a("html",{href=>$this->link(qacmd=>"export",format=>"html")})
		);
    while (my $r = $responses->next()) {
	if ($r->defined()) {
	    my $rid = $r->id;
	    my @data;
	    if ($options) {
		push @data, 
		    $ml->a("view",{href=>$this->link(response=>$r->id)})." ".
		    $ml->a("edit",{href=>$this->link(qacmd=>"edit_response",response=>$r->id)})." ".
		    $ml->a("delete",{href=>"javascript:confirm_custom('Really delete response $rid?','".$this->link(qacmd=>"del",response=>$rid)."')"});
	    }
	    if ($config{QA}{report_form_name}) {
		push @data, $form->name;
	    }
	    push @data, $rid, $r->getdata("ctime"), $r->showdata("member_id"), $r->showdata("version");
	    while (my $q = $r->{questions}->next) {
		push @data, $q->answer($r->id)->show(na=>undef);
	    }
	    $rpt->push(@data);
	}
    }
    return $rpt;
}

sub get_summary_report {
    my ($this,$form) = @_;
    my $answers = $form->get_all_answers();
    my $rpt = new ExSite::ReportBuilder(title=>"Response Summary - ".$form->name);
    my @head;
    push @head, ("Question", "Answer", "Responses", "\%", "Chart");
    $rpt->headers(\@head);
    $rpt->tools("Export: ".
		$ml->a("excel",{href=>$this->link(qacmd=>"export")})." ".
		$ml->a("csv",{href=>$this->link(qacmd=>"export",format=>"csv")})." ".
		$ml->a("html",{href=>$this->link(qacmd=>"export",format=>"html")})
		);
    my %summary;
    my %response;
    my $n_response = 0;
    if (scalar @$answers > 0) {
	foreach my $a (@$answers) {
	    my $fmt = $a->{format};
	    $response{$a->{qa_response_id}} = 1;
	    next if ($fmt eq "label");
	    if ($fmt =~ /^(string|textarea|password)$/) {
		# freeform - simply count the answers
		$summary{$a->{label}}{answered}++;
	    }
	    elsif ($fmt =~ /^(multiselect|checklist)/) {
		# multiselect - may be several answers combined
		my @ans = split /$config{form}{multi_sep}/, $a->{answer};
		foreach my $ans (@ans) {
		    $summary{$a->{label}}{$ans}++;
		}
	    }
	    else {
		# fixed format - bin the answers
		$summary{$a->{label}}{$a->{answer}}++;
	    }
	}
	$n_response = scalar keys %response;
	my $questions = $form->questions();
	while (my $q = $questions->next()) {
	    my @data;
	    push @data, $q->label;
	    my $fmt = $q->format();
	    next if ($fmt eq "label");
	    if ($fmt =~ /^(string|textarea|password)$/) {
		# freeform
		my $n = $summary{$q->label}{answered} || 0;
		my $pct = int(($n / $n_response) * 100);
		my $nhash = int(($n / $n_response) * 10);
		push @data, 
		$ml->em("[all non-blank answers]"),
		$ml->div($n,{class=>"num"}),
		$ml->div("$pct\%",{class=>"num"});
		push @data, $ml->div("#" x $nhash,{class=>"bar",style=>"height:10px;width:${pct}px"});
		$rpt->push(@data);
	    }
	    else {
		# fixed format - display each bin separately
#		my @val = $q->answer_values();
		my @val = sort keys %{$summary{$q->label}};
		my $first = 1;
		foreach my $v (@val) {
		    next if (!$v);
		    my @data;
		    push @data, $first ? $q->label : "";
		    $first = 0;
		    my $n = $summary{$q->label}{$v} || 0;
		    my $pct = int(($n / $n_response) * 100);
		    my $nhash = $n ? int(($n / $n_response) * 10) + 1 : 0;
		    push @data, $v, 
		    $ml->div($n,{class=>"num"}),
		    $ml->div("$pct\%",{class=>"num"});
		    push @data, $ml->div("#" x $nhash,{class=>"bar",style=>"height:10px;width:${pct}px"});
		    $rpt->push(@data);
		}
	    }
	}
    }
    $rpt->foot("$n_response responses received.");
    return $rpt;
}

sub show_summary {
    my ($this,$formid) = @_;
    my $out;
    my $f = new Modules::QA::Form(id=>$formid);
    my $r = $this->get_summary_report($f);
    return $r->make();
}

sub show_responses {
    my ($this,$formid) = @_;
    my $out;
    my $f = new Modules::QA::Form(id=>$formid);
    my $r = $this->get_response_report($f,1);
    return $r->make();
}

sub show_response {
    my ($this,$response_id) = @_;
    my $r = new Modules::QA::Response(id=>$response_id);
    if ($this->{input}{qacmd} eq "del") {
	$r->delete();
	$ml->redirect($this->link(qacmd=>"responses",response=>undef,__plaintext=>1));
	return;
    }
    else {
	return $r->show;
    }
}

sub export_responses {
    my ($this,$formid) = @_;
    my $fmt = $this->{input}{format};
    my $f = new Modules::QA::Form(id=>$formid);
    my $r = $this->get_response_report($f,0); # no options
    $ml->cancel();
    if ($fmt =~ /csv/) {
	print "Content-type: text/comma-separated-values\ncontent-disposition: attachment; filename=responses-$formid.csv\n\n";
	print $r->export("csv");
    }
    elsif ($fmt =~ /html/) {
	$r->set("dynamic",0);
	$r->tools(undef);
	print "Content-type: text/html\n\n";
	print $r->make();
    }
    else {
	print "Content-type: application/vnd.ms-excel\n";
	print "Content-disposition: attachment; filename=responses-$formid.xls\n\n";
	$r->export("excel");
    }
    return undef;
}

sub edit_response {
    my ($this,$form_id,$response_id) = @_;
    my $r = new Modules::QA::Response(id=>$response_id);
    my $f = $r->form;
    if ($f && $f->id == $form_id) {
	return $f->show(response=>$response_id);
    }
    return $this->error("form - response mismatch");
}

sub doform {
    my ($this,$data,$goto) = @_;
    if ($data->{submit_action} eq "Cancel") {
	$ml->location($goto);
	return undef;
    }
    my $db = $share{DB};
    my $out;
    $db->get_data();
    $db->{action} = $db->get_action();
    
    my $errcnt = &AtError();
    $db->do_db($db->{action});

    # report results
    if (! &AtError($errcnt)) {
	# no errors
	$ml->location($goto);
	$out .= "Done!";
    }
    else {
	# uh-oh!
	$out = $ml->h3("Problem:");
	$out .= $this->show_diagnostics("error","html");
	$out .= $ml->p($ml->a("Go back to form.",{href=>"javascript:history.back()"}));
    }
    return $out;
}

sub setup_querylib {
    my $this = shift;
    my $db = $share{DB};
    $db->set_query("forms using base question",
		   (sql=>"select distinct f.* from qa_form f,qa_question q,qa_base b where b.qa_base_id=q.qa_base_id and f.qa_form_id=q.qa_form_id and b.qa_base_id=?",
		    nparam=>1,
		    mode=>"r",
		    keys=>["qa_form","qa_question","qa_base"]
		    ));
}

# scoped into Report class
sub show_foreign_key {
    my ($this,$table,$column,$data,$id,$datatype) = @_;
    return undef if ($table !~ /qa_form|qa_response/);
    # dereference foreign keys
    my (undef,$ftable) = split /[:=]/,$datatype;
    # look up foreign row
    my %fdata = $this->fetch($ftable,$data);
    my $fkey = $this->get_key($ftable);
    my $label = $this->record_label($ftable,$fdata{$fkey},\%fdata);
    return $label;
}

1;
