package Modules::URLshortener;

#----------------------------------------------------------------------------
#
#   Copyright (C) 2010 - Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
#  Shorten URLs for easier emailing or printing
#
#----------------------------------------------------------------------------

use strict;

# useful kernel libs; you can/should remove any of these that are not needed

use ExSite::Config;          # required
use ExSite::Input;           # optional input manager
use ExSite::Misc;            # optional utils
use ExSite::Util;            # optional utils
use ExSite::ML;              # optional markup-language generation
use ExSite::FormBuilder;     # optional form generation
use ExSite::ReportBuilder;   # optional report generation
use ExSite::Form;            # optional database manager

# recommended base class for plug-in modules

use Modules::BaseDCD;

# declare package globals

use vars qw(@ISA $ml);

@ISA = qw(Modules::BaseDCD); # inherit from this base class

sub read {
    my ($this,$options) = @_;
    my $input = new ExSite::Input;
    $this->{input} = $input->post;
    return undef;
}

sub write {
    my ($this,$options) = @_;
    return undef;
}

# ioctl method (used by ExSite to query the module for its functionality)

sub ioctl {
    my $this = shift;
    $_ = shift;           # $_ is the ioctl request

    if (/isRestricted/) {
	return 0;
    }
    elsif (/isService/) {
	return 0;
    }
    elsif (/ModuleName/) {
	return "URL Shortener";
    }
    elsif (/ModuleInfo/) {
	return "Shorten URLs for easier emailing or printing";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/Category/) {
	return "System";
    }
}

#----------------------------------------------------------------------------
# Everything after this point consists of private methods.

# ctrl_panel() generates the contents of the administrator control panel

sub ctrl_panel {
    my $this = shift;
    $ml = new ExSite::ML;
    my $out = $ml->h1("URL shortener");

    if (! (tied %token) || ! (tied %token)->enabled) {
	$out .= $this->warn("URL shortening is not available on your system. 
You require the ExSite::Token library, and either the persistent data 
store or the token table added to your database.");
	return $out;
    }

    my $f = new ExSite::FormBuilder(method=>"post");
    $f->input(name=>"url",type=>"text",prompt=>"URL",size=>80,required=>1,value=>$this->{input}{url});
    $f->input(name=>"lifetime",type=>"text",prompt=>"Expires in (days)",size=>8,value=>$this->{input}{lifetime}||30);
    $f->input(name=>"singleuse",type=>"checkbox",prompt=>"One-time use",size=>8,value=>$this->{input}{singleuse});

    $out .= $f->make();

    if ($this->{input}{url}) {
	my $url = new ExSite::URI(uri=>$this->{input}{url});
	my $singleuse = $this->{input}{singleuse} ? 1 : 0;
	my $duration = $this->{input}{lifetime} =~ /^\d+$/ ? 
	    $this->{input}{lifetime} * 86400 : 0;
	my $newurl = $url->shorten($singleuse,$duration);
	$out .= &ExSite::HTML::BasicBox(title=>"Shortened URL",
					pane=>$ml->h1($newurl));
    }
    return $out;
}

1;
